/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

//*****************************************************************************
//
// File:		MEStatus.cpp
//
// Description:		The main file of the MEStatus application.
//
//*****************************************************************************

#ifdef HAVE_CONFIG_H 
#include "config.h" 
#endif
#include "iatshareddata.h"
#include <cstdio>
#include <csignal>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/file.h>
#include <gtkmm.h>
#include <libglademm.h>
#include <glibmm/i18n.h>
#include "MEStatusDlg.h"
#include "MEStatusIcon.h"
#include "MEStatusLogic.h"
#include "MEStatusDataDefinitions.h"
#include "MEIAMTStatus.h"

#define MESTATUS_APPLICATION_EXENAME "MEStatusApp"

/* MEStatusApp: This class is the main class of the MEStatus application. It holds all the
 * actual objects that are used: the MEStatus dialog (GUI), the MEStatus Icon (GUI), 
 * the AMT Status State machine (the logic)
 * and  the IPC AMT Status object, which is used to communicate with the UNS daemon,
 *   or the MEI AMT Status object, which is used to communicate with the MEI driver.
 * This class is the glue that holds everything together: It supplies MEStatusLogic with a 
 * pointer to the dialog, through the IME(Gui/Icon)Notifications interfaces,
 *  and it also supplies MEStatusDlg / MEStatusIcon
 * with a pointer to the state machine, through the IMEGuiCommands/IMEIconCommands interfaces.
 */
class MEStatusApp
{
public:
	//ctor
	MEStatusApp();
	//dtor
	virtual ~MEStatusApp();

	//run: Runs the MEStatus application with the supplied command line parameters, and returns 
	//an error code for the OS.
	int run(int argc, char *argv[]);

	//APP_ERROR: This enum defines the return values for run()
	enum APP_ERROR
	{
		ERROR_NONE = 0,
		ERROR_NO_GLADE_FILE = 1,
		ERROR_NO_WIDGET = 2,
		ERROR_SM_INIT_FAIL = 3,
		ERROR_GUI_INIT_FAIL = 4,
		ERROR_INIT_LOCK_FAIL = 5,
		ERROR_ICON_CREAT_FAIL = 6,
		ERROR_ICON_INIT_FAIL = 7
	};

protected:
	//The timer function that will be called periodically
	virtual bool UpdateData();

private:
	void setSignalsHandler();

	MEStatusDlg *m_pDialog; //The GUI
	MEStatusIcon *m_pIcon; //The Icon

	MEIAMTStatus m_IMEAMTStatus; //The interface to communicate with MEI

	MEStatusConfig m_Config; //The config
	MEStatusLogic m_MEStatusLogic; //The logic
	sigc::connection m_TimerConnection; //The object that maintains the connection to the periodic timer

	int mestatus_lock_fd; //Lock file
	char mestatus_lock_fname[256];

	/*Constants*/
	static const Glib::ustring GLADE_FILE;
};

const Glib::ustring MEStatusApp::GLADE_FILE = MESTATUS_RESPATH "/MEStatus.glade";

//ctor
MEStatusApp::MEStatusApp() :
m_IMEAMTStatus(false)
{
	m_Config.Init();
	m_pDialog = NULL;
	m_pIcon = NULL;
	mestatus_lock_fd = -1;
}

//dtor
MEStatusApp::~MEStatusApp()
{
	//Stop periodic timer
	m_TimerConnection.disconnect();
	//Remove lock
	if (-1 != mestatus_lock_fd) {
		flock(mestatus_lock_fd, LOCK_UN);
		close(mestatus_lock_fd);
		unlink(mestatus_lock_fname);
	}

	m_IMEAMTStatus.CloseReader();

	if (NULL != m_pIcon) {
		delete m_pIcon;
		m_pIcon = NULL;
	}
}

// Handler for termination signal.
void terminationHandler(int signum, siginfo_t* sinfo, void* dummy)
{
	Gtk::Main::quit();
	exit(0);
}

//Signals action register
void MEStatusApp::setSignalsHandler()
{
	int sigSet = 0;
	// Termination signal handler.
	struct sigaction terminateAction;

	// Set up the structure to specify the termination action.
	terminateAction.sa_sigaction = terminationHandler;
	sigemptyset(&terminateAction.sa_mask);
	terminateAction.sa_flags = SA_SIGINFO;

	sigSet &= sigaction(SIGTERM, &terminateAction, NULL);
	sigSet &= sigaction(SIGQUIT, &terminateAction, NULL);
	sigSet &= sigaction(SIGHUP,  &terminateAction, NULL);
	sigSet &= sigaction(SIGINT,  &terminateAction, NULL);
	sigSet &= sigaction(SIGPIPE, &terminateAction, NULL);
	sigSet &= sigaction(SIGALRM, &terminateAction, NULL);
	sigSet &= sigaction(SIGUSR1, &terminateAction, NULL);
	sigSet &= sigaction(SIGUSR2, &terminateAction, NULL);
}

int MEStatusApp::run(int argc, char *argv[])
{
	//Initialize GTK
	Gtk::Main kit(argc, argv);

	if (argc && argv[0] && (NULL != strstr(argv[0], MESTATUS_APPLICATION_EXENAME))) {
		m_MEStatusLogic.SetModeApplication(true);
	} else {
		//One instance lock
		Glib::RefPtr<Gdk::Display> mydisplay = Gdk::Display::get_default();
		if (mydisplay) {
			int lfd;
			g_snprintf(mestatus_lock_fname, sizeof(mestatus_lock_fname), "/tmp/intel_MEStatus_%s.lock", mydisplay->get_name().c_str());
			lfd = creat(mestatus_lock_fname, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
			if (-1 == lfd) {
				Gtk::MessageDialog(MESD_RES_ERROR_OPEN_LOCK_FILE, false, Gtk::MESSAGE_ERROR).run();
				return ERROR_INIT_LOCK_FAIL;
			}
			if (-1 == flock(lfd, LOCK_EX | LOCK_NB)) {
				Gtk::MessageDialog(MESD_RES_ERROR_ANOTHER_INSTANCE, false, Gtk::MESSAGE_ERROR).run();
				return ERROR_INIT_LOCK_FAIL;
			}
			mestatus_lock_fd = lfd;
		}
	}

	setSignalsHandler();

	//Read resources from glade file
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	try
	{
		refXml = Gnome::Glade::Xml::create(GLADE_FILE);
	}
	catch (Gnome::Glade::XmlError &) {
		Gtk::MessageDialog(_("Couldn't load glade file: ") + GLADE_FILE, false, Gtk::MESSAGE_ERROR).run();
		return ERROR_NO_GLADE_FILE;
	}

	//Create GUI, but don't initialize it yet.
	refXml->get_widget_derived("servicesDlg", m_pDialog);
	if (NULL == m_pDialog) {
		Gtk::MessageDialog(_("Couldn't get MEStatusDlg widget"), false, Gtk::MESSAGE_ERROR).run();
		return ERROR_NO_WIDGET;
	}

	//Create Notification Icon
	m_pIcon = new MEStatusIcon();
	if (NULL == m_pIcon) {
		Gtk::MessageDialog(_("Failed to create Status Icon object"), false, Gtk::MESSAGE_ERROR).run();
		return ERROR_ICON_CREAT_FAIL;
	}

	//Initialize MEStatus Logic
	if (!m_MEStatusLogic.Init(&m_Config, &m_IMEAMTStatus, m_pDialog, m_pIcon)) {
		Gtk::MessageDialog(_("Failed to initialize MEStatus Logic"), false, Gtk::MESSAGE_ERROR).run();
		return ERROR_SM_INIT_FAIL;
	}

	//Initialize GUI
	Glib::ustring strError;
	if (!m_pDialog->Init(&m_MEStatusLogic, strError)) {
		Gtk::MessageDialog(strError, false, Gtk::MESSAGE_ERROR).run();
		return ERROR_GUI_INIT_FAIL;
	}
	//Initialize Icon
	if (!m_pIcon->Init(&m_MEStatusLogic, strError)) {
		Gtk::MessageDialog(strError, false, Gtk::MESSAGE_ERROR).run();
		return ERROR_ICON_INIT_FAIL;
	}

	//Load data once in the beginning, and later it'll be on the timer.
	m_MEStatusLogic.StartLogic();

	//Install a timer that will call UpdateStatus() every X milliseconds 
	//(according to configuration).
	m_TimerConnection = 
		Glib::signal_timeout().connect(sigc::mem_fun(*this, &MEStatusApp::UpdateData),
		                               m_Config.CheckAMTStatusTimeout());

	/**** MAIN GUI LOOP ****/
	kit.run();
	/***********************/

	return ERROR_NONE;
}

bool MEStatusApp::UpdateData()
{
	m_MEStatusLogic.UpdateData();
	return true; //Tell the framework this is a periodic timer
}

int main(int argc, char *argv[])
{
	umask(022);

	chdir(MESTATUS_RESPATH);

	//To let the program use the translation for the current locale
	bindtextdomain(PACKAGE_NAME, MESTATUS_RESPATH "/locale");
	bind_textdomain_codeset(PACKAGE_NAME, "UTF-8");
	textdomain(PACKAGE_NAME);

	//Create instance of application
	MEStatusApp App;

	/* Run application */
	return App.run(argc, argv);
}
