/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

//******************************************************************************
//
// File:		MEStatusData.cpp
//
// Description:		Implementation of MEStatusData class
//                      MEStatus data parser.
//
//******************************************************************************

#ifdef HAVE_CONFIG_H 
#include "config.h" 
#endif
#include "MEStatusData.h"
#include <glibmm/i18n.h>
#include "MEStatusDataDefinitions.h"
#ifdef MESD_USE_SSTREAM
#include <string>
#include <sstream>
#include <iomanip>
#endif


const Glib::ustring MEStatusData::STATUS_DATA_NOT_AVAILABLE = MESD_RES_INFORMATION_UNAVAILABLE;

//ctor
MEStatusData::MEStatusData()
{
	memset(&m_amtData, 0, sizeof(AMTStatusData));
#ifdef WITH_TPM
	memset(&m_tpmData, 0, sizeof(TPMStatusData));
#endif
	m_haveEventHistory = false;
	m_reachedIterLastSeenEvent = false;
	m_haveLastSeenEvent = false;
	m_havePrevLastEvent = false;
}

//dtor
MEStatusData::~MEStatusData()
{
	m_eventHistory.clear();
}


void MEStatusData::SetAMTStatusData(const AMTStatusData &amtData)
{
	memcpy(&m_amtData, &amtData, sizeof(AMTStatusData));
	//sanity
	m_amtData.amtVersion[sizeof(m_amtData.amtVersion) - 1] = '\0';
	m_amtData.lmsVersion[sizeof(m_amtData.lmsVersion) - 1] = '\0';
	m_amtData.powerPolicyName[sizeof(m_amtData.powerPolicyName) - 1] = '\0';
#if 0
	m_amtData.mpsHostname[sizeof(m_amtData.mpsHostname) - 1] = '\0';
#endif
}

#ifdef WITH_TPM
void MEStatusData::SetTPMStatusData(const TPMStatusData &tpmData)
{
	memcpy(&m_tpmData, &tpmData, sizeof(TPMStatusData));
}
#endif

void MEStatusData::LoadEventHistory(unsigned int maxSize)
{
	m_haveEventHistory = false;
	if (maxSize) {
		m_eventHistoryReader.GetLastEvents(m_eventHistory, maxSize);
	} else {
		m_eventHistoryReader.GetEvents(m_eventHistory);
	}
	if (!m_eventHistory.empty()) {
		m_eventHistory.reverse(); //latest events first
		m_eventHistoryIter = m_eventHistory.begin();
		m_haveEventHistory = true;
	}
	m_reachedIterLastSeenEvent = false;
}

bool MEStatusData::UpdateLastReadEvent()
{
	if (!m_haveEventHistory) {
		m_havePrevLastEvent = false;
		return false;
	}
	if (m_eventHistory.begin() != m_eventHistory.end()) {
		m_prevLastEvent = *(m_eventHistory.begin());
		m_havePrevLastEvent = true;
		return saveLastUserEvent();
	}
	return false;
}

bool MEStatusData::UpdateLastSeenEvent()
{
	if (!m_haveEventHistory) {
		m_haveLastSeenEvent = false;
		return false;
	}
	if (m_eventHistory.begin() != m_eventHistory.end()) {
		m_lastSeenEvent = *(m_eventHistory.begin());
		m_haveLastSeenEvent = true;
		return saveLastUserEvent();
	}
	return false;
}


Glib::ustring MEStatusData::GetAMTStatus() const
{
	switch (m_amtData.amtState)
	{
	case AMTSTATE_POST:
		return MESD_RES_CONFIGURED;
		break;

	case AMTSTATE_IN:
	case AMTSTATE_PRE:
		return MESD_RES_UNCONFIGURED;
		break;

	case AMTSTATE_DISABLED:
		return MESD_RES_DISABLED;
		break;

	case AMTSTATE_MEI_DISABLED:
	case AMTSTATE_NOT_DETECTED:
	default:
		return MESD_RES_NOT_DETECTED;
	}
}

Glib::ustring MEStatusData::GetAMTVersion() const
{
	if (0 != (m_amtData.content & CONTENT_AMT_VER)) {
		return Glib::ustring((char *)(m_amtData.amtVersion));
	}
	return MESD_RES_INFORMATION_UNAVAILABLE;
}

Glib::ustring MEStatusData::GetAMTMode() const
{
	switch (m_amtData.amtState)
	{
	case AMTSTATE_POST:
		if (0 != (m_amtData.content & CONTENT_PROVMODE)) {
			switch (m_amtData.provMode)
			{
			case AMTMODE_ENTERPRISE:
				return MESD_RES_CONFIGURED_ENTERPRISE;
				break;

			case AMTMODE_SB:
				return MESD_RES_CONFIGURED_SB;
				break;
			}
		}
		return MESD_RES_CONFIGURED;
		break;

	case AMTSTATE_IN:
		return MESD_RES_AWAITING_CONFIGURATION;
		break;

	case AMTSTATE_PRE:
		return MESD_RES_UNCONFIGURED;
		break;

	case AMTSTATE_DISABLED:
		return MESD_RES_DISABLED;
		break;

	case AMTSTATE_MEI_DISABLED:
	case AMTSTATE_NOT_DETECTED:
	default:
		return MESD_RES_NOT_DETECTED;
	}
	
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetLastHostResetReason() const
{
	if (0 != (m_amtData.content & CONTENT_HOST_RESET_REASON)) {
		if (HOST_RESET_REASON_REMOTE == m_amtData.resetHostReason) {
#ifdef MESD_USE_SSTREAM
			std::stringstream sst;
			sst << MESD_RES_HOST_REBOOT_REMOTE << "-" << m_amtData.resetHostTS;
			return sst.str();
#else
			gchar strTS[20];
			memset(strTS, 0, sizeof(strTS));
			g_snprintf((gchar *)strTS, 19, "-%u", m_amtData.resetHostTS);
			return Glib::ustring(MESD_RES_HOST_REBOOT_REMOTE).append((gchar *)strTS);
#endif
		} else {
			return MESD_RES_HOST_REBOOT_NOT_REMOTE;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetMEResetReason() const
{
	if (0 != (m_amtData.content & CONTENT_ME_RESET_REASON)) {
		switch (m_amtData.resetMEReason)
		{
		case ME_RESET_REASON_POWER_UP:
			return MESD_RES_ME_RESET_REASON_POWER_UP;
			break;

		case ME_RESET_REASON_FW_RESET:
			return MESD_RES_ME_RESET_REASON_FW_RESET;
			break;

		case ME_RESET_REASON_GLOBAL_SYSTEM_RESET:
			return MESD_RES_ME_RESET_REASON_GLOBAL_SYSTEM_RESET;
			break;

		case ME_RESET_REASON_UNKNOWN:
		default:
			return MESD_RES_ME_RESET_REASON_UNKNOWN;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTWebUI() const
{
	if (0 != (m_amtData.content & CONTENT_WUISTATE)) {
		switch (m_amtData.webUIState)
		{
		case WUISTATE_TLS:
			return MESD_RES_WEBUI_ENABLED_TLS;
			break;

		case WUISTATE_ENABLED:
			return MESD_RES_ENABLED;
			break;

		case WUISTATE_DISABLED:
			return MESD_RES_DISABLED;
			break;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTDefense() const
{
	if (0 != (m_amtData.content & CONTENT_FEATURES)) {
		if (0 != m_amtData.features[2]) {
			return MESD_RES_ACTIVATED;
		} else {
			return MESD_RES_NOT_ACTIVATED;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTRedirection() const
{
	if (0 != (m_amtData.content & CONTENT_FEATURES)) {
		Glib::ustring redirStatus;
		if (0 != m_amtData.features[0]) {
			redirStatus = MESD_RES_SOL_ACTIVATED;
		} else {
			redirStatus = MESD_RES_SOL_NOT_ACTIVATED;
		}
		redirStatus.append(", ");
		if (0 != m_amtData.features[1]) {
			redirStatus.append(MESD_RES_IDER_ACTIVATED);
		} else {
			redirStatus.append(MESD_RES_IDER_NOT_ACTIVATED);
		}
		return redirStatus;
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTSOLRedirection() const
{
	if (0 != (m_amtData.content & CONTENT_FEATURES)) {
		if (0 != m_amtData.features[0]) {
			return MESD_RES_ACTIVATED;
		} else {
			return MESD_RES_NOT_ACTIVATED;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTIDERRedirection() const
{
	if (0 != (m_amtData.content & CONTENT_FEATURES)) {
		if (0 != m_amtData.features[1]) {
			return MESD_RES_ACTIVATED;
		} else {
			return MESD_RES_NOT_ACTIVATED;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTPowerPolicy() const
{
	if (0 != (m_amtData.content & CONTENT_POWERPOL)) {
		return Glib::ustring((char *)(m_amtData.powerPolicyName));
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTMEIVersion() const
{
	if (0 != (m_amtData.content & CONTENT_MEI_VER)) {
#ifdef MESD_USE_SSTREAM
		std::stringstream sst;
		//sst << MESD_RES_ENABLED << " (";
		sst << (unsigned int)m_amtData.heciVersion.major;
		sst << "." << (unsigned int)m_amtData.heciVersion.minor;
		sst << "." << (unsigned int)m_amtData.heciVersion.hotfix;
		sst << "." << (unsigned int)m_amtData.heciVersion.build;
		//sst << ")";
		return sst.str();
#else
		gchar strVer[32];
		memset(strVer, 0, sizeof(strVer));
		//g_snprintf((gchar *)strVer, 31, " (%u.%u.%u.%u)",
		g_snprintf((gchar *)strVer, 31, "%u.%u.%u.%u",
			     (unsigned int)m_amtData.heciVersion.major,
			     (unsigned int)m_amtData.heciVersion.minor,
			     (unsigned int)m_amtData.heciVersion.hotfix,
			     (unsigned int)m_amtData.heciVersion.build);
		//return Glib::ustring(MESD_RES_ENABLED).append((char *)strVer);
		return Glib::ustring((char *)strVer);
#endif
	}

	if (AMTSTATE_MEI_DISABLED == m_amtData.amtState) {
		return MESD_RES_DISABLED;
	}

	return MESD_RES_INFORMATION_UNAVAILABLE;
}

Glib::ustring MEStatusData::GetAMTLMSVersion() const
{
	if (0 != (m_amtData.content & CONTENT_LMS_VER)) {
		return (Glib::ustring(((m_amtData.lmsRunning) ? MESD_RES_LMS_ENABLED : MESD_RES_LMS_DISABLED))
			+ " (" + (char *)(m_amtData.lmsVersion) + ")");
	}
	return MESD_RES_LMS_UNINSTALLED;
}


Glib::ustring MEStatusData::GetAMTNetMode(MEStatusData::NetDataIfaceType type) const
{
	int contentType = 0;
	const amtLanSettings *lan = NULL;
	if (NETDATA_IFACE_WIRED == type) {
		lan = &m_amtData.lanWired;
		contentType = CONTENT_LANWIRED;
#ifdef WITH_WIRELESS_AT
	} else if (NETDATA_IFACE_WIRELESS == type) {
		lan = &m_amtData.lanWireless;
		contentType = CONTENT_LANWLESS;
#endif
	}
	if ((0 != (m_amtData.content & contentType)) && (NULL != lan)) {
		if (LAN_DHCP_STATIC == lan->DhcpEnabled) {
			return MESD_RES_DHCP_STATIC;
		} else {
			return MESD_RES_DHCP_DHCP;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTNetMac(MEStatusData::NetDataIfaceType type) const
{
	int contentType = 0;
	const amtLanSettings *lan = NULL;
	if (NETDATA_IFACE_WIRED == type) {
		lan = &m_amtData.lanWired;
		contentType = CONTENT_LANWIRED;
#ifdef WITH_WIRELESS_AT
	} else if (NETDATA_IFACE_WIRELESS == type) {
		lan = &m_amtData.lanWireless;
		contentType = CONTENT_LANWLESS;
#endif
	}
	if ((0 != (m_amtData.content & contentType)) && (NULL != lan)) {
#ifdef MESD_USE_SSTREAM
		std::stringstream sst;
		sst << std::setfill('0') << std::setw(2) << std::hex << (unsigned int)(lan->MacAddress[0]);
		sst << ":" << std::setfill('0') << std::setw(2) << std::hex << (unsigned int)(lan->MacAddress[1]);
		sst << ":" << std::setfill('0') << std::setw(2) << std::hex << (unsigned int)(lan->MacAddress[2]);
		sst << ":" << std::setfill('0') << std::setw(2) << std::hex << (unsigned int)(lan->MacAddress[3]);
		sst << ":" << std::setfill('0') << std::setw(2) << std::hex << (unsigned int)(lan->MacAddress[4]);
		sst << ":" << std::setfill('0') << std::setw(2) << std::hex << (unsigned int)(lan->MacAddress[5]);
		return sst.str();
#else
		gchar strMac[32];
		memset(strMac, 0, sizeof(strMac));
		g_snprintf((gchar *)strMac, 31, "%.2x:%.2x:%.2x:%.2x:%.2x:%.2x",
				(unsigned int)(lan->MacAddress[0]),
				(unsigned int)(lan->MacAddress[1]),
				(unsigned int)(lan->MacAddress[2]),
				(unsigned int)(lan->MacAddress[3]),
				(unsigned int)(lan->MacAddress[4]),
				(unsigned int)(lan->MacAddress[5]));
		return Glib::ustring((char *)strMac);
#endif
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTNetIP(MEStatusData::NetDataIfaceType type) const
{
	int contentType = 0;
	const amtLanSettings *lan = NULL;
	if (NETDATA_IFACE_WIRED == type) {
		lan = &m_amtData.lanWired;
		contentType = CONTENT_LANWIRED;
#ifdef WITH_WIRELESS_AT
	} else if (NETDATA_IFACE_WIRELESS == type) {
		lan = &m_amtData.lanWireless;
		contentType = CONTENT_LANWLESS;
#endif
	}
	if ((0 != (m_amtData.content & contentType)) && (NULL != lan)) {
#ifdef MESD_USE_SSTREAM
		std::stringstream sst;
		sst << (unsigned int)(lan->Ipv4Address & 0x000000FF);
		sst << "." << (unsigned int)((lan->Ipv4Address & 0x0000FF00) >> 8);
		sst << "." << (unsigned int)((lan->Ipv4Address & 0x00FF0000) >> 16);
		sst << "." << (unsigned int)((lan->Ipv4Address & 0xFF000000) >> 24);
		return sst.str();
#else
		gchar strIP[32];
		memset(strIP, 0, sizeof(strIP));
		g_snprintf((gchar *)strIP, 31, "%u.%u.%u.%u",
				(unsigned int)(lan->Ipv4Address & 0x000000FF),
				(unsigned int)((lan->Ipv4Address & 0x0000FF00) >> 8),
				(unsigned int)((lan->Ipv4Address & 0x00FF0000) >> 16),
				(unsigned int)((lan->Ipv4Address & 0xFF000000) >> 24));
		return Glib::ustring((char *)strIP);
#endif
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTNetLinkStatus(MEStatusData::NetDataIfaceType type) const
{
	int contentType = 0;
	const amtLanSettings *lan = NULL;
	if (NETDATA_IFACE_WIRED == type) {
		lan = &m_amtData.lanWired;
		contentType = CONTENT_LANWIRED;
#ifdef WITH_WIRELESS_AT
	} else if (NETDATA_IFACE_WIRELESS == type) {
		lan = &m_amtData.lanWireless;
		contentType = CONTENT_LANWLESS;
#endif
	}
	if ((0 != (m_amtData.content & contentType)) && (NULL != lan)) {
		if (LAN_LINK_DOWN == lan->LinkStatus) {
			return MESD_RES_LINK_DOWN;
		} else {
			return MESD_RES_LINK_UP;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

#ifdef WITH_WIRELESS_AT
Glib::ustring MEStatusData::GetAMTNetWirelessConfInterface() const
{
	if (0 != (m_amtData.content & CONTENT_LANWLESS)) {
		if (LAN_DHCP_ACTIVE == m_amtData.lanWireless.DhcpIpMode) {
			return MESD_RES_DHCP_ACTIVE;
		} else {
			return MESD_RES_DHCP_PASIVE;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetAMTNetWirelessConfWireless() const
{
	if (0 != (m_amtData.content & CONTENT_LANWLESS)) {
		if (LAN_WIRELESS_DISABLED == m_amtData.lanWireless.Enabled) {
			return MESD_RES_WIRELESS_DISABLED;
		} else {
			return MESD_RES_WIRELESS_ENABLED;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}
#endif

Glib::ustring MEStatusData::GetAMTRemoteAccessConnectionStatus() const
{
	if (0 != (m_amtData.content & CONTENT_RA)) {
		switch(m_amtData.remoteAccessConnStatus)
		{
		case RA_CONN_CONNECTED:
			return MESD_RES_REMOTE_ACCESS_CONNECTED;
			break;
		case RA_CONN_CONNECTING:
			return MESD_RES_REMOTE_ACCESS_CONNECTING;
			break;
		case RA_CONN_NOT_CONNECTED:
		default:
			return MESD_RES_REMOTE_ACCESS_DISCONNECTED;
		}
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

#ifdef WITH_TPM
Glib::ustring MEStatusData::GetTPMState() const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_OPERATION_STATE)) {
		if ( (0 == m_tpmData.operationState[0])
		    && (0 == m_tpmData.operationState[1])
		    && (0 == m_tpmData.operationState[2])
		    && (0 == m_tpmData.operationState[3]) )
		{
			return MESD_RES_TPM_OPERATIONAL;
		} else {
			return MESD_RES_TPM_FAILED;
		}
	}

	if (0 != (m_tpmData.content & CONTENT_TPM_BUSY)) {
		return MESD_RES_TPM_BUSY;
	}

	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetTPMActivation() const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_ACTIVATION_STATE)) {
		if (0 != m_tpmData.activationState) {
			if (0 != (m_tpmData.content & CONTENT_TPM_TEMP_ACTIV_STATE)) {
				if (0 != m_tpmData.tempActivationState) {
					return MESD_RES_TPM_ACTIVE;
				} else {
					return MESD_RES_TPM_NON_ACTIVE;
				}
			}
			return MESD_RES_TPM_ACTIVE;
		} else {
			return MESD_RES_TPM_NON_ACTIVE;
		}
	}
	return MESD_RES_NOT_DETECTED;
}

Glib::ustring MEStatusData::GetTPMOwnership() const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_OWNERSHIP_STATE)) {
		if (0 != m_tpmData.ownershipState) {
			return MESD_RES_TPM_OWNED;
		} else {
			return MESD_RES_TPM_NOT_OWNED;
		}
	}
	return MESD_RES_NOT_DETECTED;
}

Glib::ustring MEStatusData::GetTPMEnablement() const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_ENABLEMENT_STATE)) {
		if (0 != m_tpmData.enablementState) {
			return MESD_RES_TPM_ENABLED;
		} else {
			return MESD_RES_TPM_NOT_ENABLED;
		}
	}
	return MESD_RES_NOT_DETECTED;
}

Glib::ustring MEStatusData::GetTPMFailure() const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_OPERATION_STATE)) {
		if ( (0 == m_tpmData.operationState[0])
		    && (0 == m_tpmData.operationState[1]) )
		{
			if ( (0 == m_tpmData.operationState[2])
			    && (0 == m_tpmData.operationState[3]) )
			{
				return MESD_RES_TPM_FAIL_NOFAIL;
			} else if (5 == m_tpmData.operationState[2]) {
				switch((unsigned int)(m_tpmData.operationState[3]))
				{
				case 7:
					return MESD_RES_TPM_FAIL_ME_RESET;
					break;

				case 4:
				case 5:
				case 6:
					return MESD_RES_TPM_FAIL_HW_FAILURE;
					break;
				}
			}
		}
		return MESD_RES_TPM_FAIL_UNKNOWN;
	}
	return MESD_RES_NOT_DETECTED;
}

void MEStatusData::appendSubstatesToTPMStatus(Glib::ustring &state) const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_OPERATION_SUBSTATES)) {
		bool wasappend = false;
		state.append(" - ");
		if (0 != (m_tpmData.content & CONTENT_TPM_ACTIVATION_STATE)) {
			state.append(GetTPMActivation());
			wasappend = true;
		}
		if (0 != (m_tpmData.content & CONTENT_TPM_OWNERSHIP_STATE)) {
			if (wasappend) {
				state.append(" ; ");
			}
			state.append(GetTPMOwnership());
			wasappend = true;
		}
		if (0 != (m_tpmData.content & CONTENT_TPM_ENABLEMENT_STATE)) {
			if (wasappend) {
				state.append(" ; ");
			}
			state.append(GetTPMEnablement());
		}
	}
}

Glib::ustring MEStatusData::GetTPMStatus() const
{
	Glib::ustring state = GetTPMState();

	if (0 != (m_tpmData.content & CONTENT_TPM_OPERATION_STATE)) {
		if ( (0 == m_tpmData.operationState[0])
		    && (0 == m_tpmData.operationState[1])
		    && (0 == m_tpmData.operationState[2])
		    && (0 == m_tpmData.operationState[3]) )
		{
			appendSubstatesToTPMStatus(state);
		} else {
			state.append(" - ").append(GetTPMFailure());
		}
	}
	else if (0 != (m_tpmData.content & CONTENT_TPM_BUSY)) {
		appendSubstatesToTPMStatus(state);
	} else {
		appendSubstatesToTPMStatus(state);
	}

	return state;
}

Glib::ustring MEStatusData::GetTPMTCGVersion() const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_TCG_VERSION)) {
#if 0
#ifdef MESD_USE_SSTREAM
		std::stringstream sstx;
		sstx << (unsigned int)(m_tpmData.TCGVersion.major)
		sstx << "." << (unsigned int)(m_tpmData.TCGVersion.minor);
		sstx << " Revision: " << (unsigned int)(m_tpmData.TCGVersion.revMajor);
		sstx << "." << (unsigned int)(m_tpmData.TCGVersion.revMinor);
		sstx << " Level: " << (unsigned int)(m_tpmData.TCGVersion.specLevel);
		sstx << " Errata: " << (unsigned int)(m_tpmData.TCGVersion.errata);
		return sstx.str();
#else
		gchar strVerx[80];
		memset(strVerx, 0, sizeof(strVerx));
		g_snprintf((gchar *)strVerx, 79, "%u.%u Rev: %u.%u Level: %u Errata: %u",
				(unsigned int)(m_tpmData.TCGVersion.major),
				(unsigned int)(m_tpmData.TCGVersion.minor),
				(unsigned int)(m_tpmData.TCGVersion.revMajor),
				(unsigned int)(m_tpmData.TCGVersion.revMinor),
				(unsigned int)(m_tpmData.TCGVersion.specLevel),
				(unsigned int)(m_tpmData.TCGVersion.errata));
		return Glib::ustring((char *)strVerx);
#endif
#else
#ifdef MESD_USE_SSTREAM
		std::stringstream sst;
		sst << (unsigned int)(m_tpmData.TCGVersion.major)
		sst << "." << (unsigned int)(m_tpmData.TCGVersion.minor);
		return sst.str();
#else
		gchar strVer[32];
		memset(strVer, 0, sizeof(strVer));
		g_snprintf((gchar *)strVer, 31, "%u.%u",
				(unsigned int)(m_tpmData.TCGVersion.major),
				(unsigned int)(m_tpmData.TCGVersion.minor));
		return Glib::ustring((char *)strVer);
#endif
#endif
	}
	return STATUS_DATA_NOT_AVAILABLE;
}

Glib::ustring MEStatusData::GetTPMFWVersion() const
{
	if (0 != (m_tpmData.content & CONTENT_TPM_TPM_VERSION)) {
#ifdef MESD_USE_SSTREAM
		std::stringstream sst;
		sst << (unsigned int)(m_tpmData.TPMVersion.major);
		sst << "." << (unsigned int)(m_tpmData.TPMVersion.minor);
		sst << "." << (unsigned int)(m_tpmData.TPMVersion.hotfix);
		sst << "." << (unsigned int)(m_tpmData.TPMVersion.build);
		return sst.str();
#else
		gchar strVer[32];
		memset(strVer, 0, sizeof(strVer));
		g_snprintf((gchar *)strVer, 31, "%u.%u.%u.%u",
				(unsigned int)(m_tpmData.TPMVersion.major),
				(unsigned int)(m_tpmData.TPMVersion.minor),
				(unsigned int)(m_tpmData.TPMVersion.hotfix),
				(unsigned int)(m_tpmData.TPMVersion.build));
		return Glib::ustring((char *)strVer);
#endif
	}
	return STATUS_DATA_NOT_AVAILABLE;
}
#endif

bool MEStatusData::GetEventInfo(Glib::ustring &app, Glib::ustring &type,
				Glib::ustring &reason, Glib::ustring &time,
				Glib::ustring &isnew)
{
	bool result = false;

	if (!m_haveEventHistory) {
		return false;
	}

	while (m_eventHistoryIter != m_eventHistory.end()) {
		if (!m_reachedIterLastSeenEvent) {
			if (m_haveLastSeenEvent && (m_lastSeenEvent == *m_eventHistoryIter)) {
				m_reachedIterLastSeenEvent = true;
			}
		}
		isnew  = ((m_reachedIterLastSeenEvent) ? MESD_RES_EVENT_REVIEWED : MESD_RES_EVENT_ISNEW);

		result = getEventInfo(*m_eventHistoryIter, app, type, reason, time);

		m_eventHistoryIter++;

		if (result) {
			return true;
		}
	}

	return false;
}

bool MEStatusData::PrepareEventInfo()
{
	if (!m_haveEventHistory) {
		return false;
	}

	m_eventHistoryIter = m_eventHistory.begin();
	return true;
}

bool MEStatusData::PrepareNewEventInfo()
{
	if (!m_haveEventHistory) {
		return false;
	}
	if (!m_havePrevLastEvent) {
		m_eventHistoryIter = m_eventHistory.end();
		return true;
	}
	m_eventHistoryIter = m_eventHistory.begin();
	while (m_prevLastEvent != *m_eventHistoryIter) {
		m_eventHistoryIter++;
	}
	return true;
}

bool MEStatusData::GetNewEventInfo(Glib::ustring &app, Glib::ustring &type,
				Glib::ustring &reason, Glib::ustring &time,
				Glib::ustring &isnew)
{
	if (!m_haveEventHistory) {
		return false;
	}

	while (m_eventHistoryIter != m_eventHistory.begin()) {
		m_eventHistoryIter--;
		isnew  = MESD_RES_EVENT_ISNEW;
		if (getEventInfo(*m_eventHistoryIter, app, type, reason, time)) {
			return true;
		}
	}

	return false;
}


bool MEStatusData::getEventInfo(const ATEvents::EventNode &curEvent,
				Glib::ustring &app, Glib::ustring &type,
				Glib::ustring &reason, Glib::ustring &time)
{
	char buffer[81];
	std::strftime(buffer, 80, "%x %X", localtime(&curEvent.m_ts));
	buffer[80] = '\0';
	time = Glib::ustring((char *) buffer);
	unsigned int eventForSku = 0;

	app = "???";
	type= "???";
	reason = "";
	switch (curEvent.m_id)
	{
	case ATEvents::EventNode::AMT_ENABLED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_ENABLED;
		break;

	case ATEvents::EventNode::AMT_DISABLED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_DISABLED;
		break;

	case ATEvents::EventNode::PROVISIONED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_PROVISIONED;
		break;

	case ATEvents::EventNode::UNPROVISIONED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_UNPROVISIONED;
		break;

	case ATEvents::EventNode::SOL_STARTED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_SOL_START;
		break;

	case ATEvents::EventNode::SOL_FINISHED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_SOL_FINISH;
		break;

	case ATEvents::EventNode::IDER_STARTED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_IDER_START;
		break;

	case ATEvents::EventNode::IDER_FINISHED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_IDER_FINISH;
		break;

	case ATEvents::EventNode::SD_STARTED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_SD_START;
		break;

	case ATEvents::EventNode::SD_FINISHED:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_SD_FINISH;
		break;

	case ATEvents::EventNode::REMOTE_REBOOT:
		eventForSku = MESDAT_SKU_AMT;
		type = MESD_RES_EVENT_AMT_REMOTE_REBOOT;
		break;

#ifdef WITH_TPM
	case ATEvents::EventNode::TPM_OPERATIONAL:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_OPERATIONAL;
		break;

	case ATEvents::EventNode::TPM_NOT_DETECTED:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_NOT_DETECTED;
		break;

	case ATEvents::EventNode::TPM_FAIL:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_FAIL;
		switch (curEvent.m_data1)
		{
		case ATEvents::EventNode::TPM_FAIL_ME_RESET:
			reason = MESD_RES_EVENT_TPM_FAIL_REASON_ME_RESET;
			break;

		case ATEvents::EventNode::TPM_FAIL_HW_FAILURE:
			reason = MESD_RES_EVENT_TPM_FAIL_REASON_HW_FAILURE;
			break;

		case ATEvents::EventNode::TPM_FAIL_FLASH_CORRUPT:
			reason = MESD_RES_EVENT_TPM_FAIL_REASON_FLASH_CORRUPT;
			break;

		case ATEvents::EventNode::TPM_FAIL_UNKNOWN:
		default:
			reason = MESD_RES_EVENT_TPM_FAIL_REASON_UNKNOWN;
		}
		break;
	case ATEvents::EventNode::TPM_ACTIVE:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_ACTIVE;
		break;

	case ATEvents::EventNode::TPM_NOT_ACTIVE:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_NOT_ACTIVE;
		break;

	case ATEvents::EventNode::TPM_OWNED:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_OWNED;
		break;

	case ATEvents::EventNode::TPM_NOT_OWNED:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_NOT_OWNED;
		break;

	case ATEvents::EventNode::TPM_ENABLE:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_ENABLE;
		break;

	case ATEvents::EventNode::TPM_NOT_ENABLE:
		eventForSku = MESDAT_SKU_TPM;
		type = MESD_RES_EVENT_TPM_NOT_ENABLED;
		break;
#endif

	default:
		return false;
	}

	if ((eventForSku & m_sku) == 0) {
		app = "!!!";
		type= "!!!";
		reason = "";
		return false;
	}

	switch(eventForSku)
	{
	case MESDAT_SKU_AMT:
		if (0 != (m_sku & MESDAT_SKU_AMTFUND)) {
			app = MESD_RES_EVENT_APP_AMT_BASIC;
		} else {
			app = MESD_RES_EVENT_APP_AMT;
		}
		break;

#ifdef WITH_TPM
	case MESDAT_SKU_TPM:
		app = MESD_RES_EVENT_APP_TPM;
		break;
#endif
	}

	return true;
}

bool MEStatusData::saveLastUserEvent()
{
	if (!m_haveEventHistory) {
		return false;
	}
	if (!m_havePrevLastEvent && !m_haveLastSeenEvent) {
		return false;
	}
	if (0 == m_eventHistoryReader.SaveUserLastEvent(
	    ((m_havePrevLastEvent) ? m_prevLastEvent : ATEvents::EventNode()),
	    ((m_haveLastSeenEvent) ? m_lastSeenEvent : ATEvents::EventNode())
	    ) ) {
		return true;
	}
	return false;
}

bool MEStatusData::LoadLastUserEvent()
{
	if (0 != m_eventHistoryReader.LoadUserLastEvent(m_prevLastEvent, m_lastSeenEvent)) {
		return false;
	}
	if (!m_prevLastEvent.empty()) {
		m_havePrevLastEvent = true;
	}
	if (!m_lastSeenEvent.empty()) {
		m_haveLastSeenEvent = true;
	}
	return true;
}

