/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

//******************************************************************************
//
// File:		MEStatusData.h
//
// Description:		Interface for MEStatusData class - MEStatus data parser.
//
//******************************************************************************

#ifndef _ME_STATUS_DATA_H
#define _ME_STATUS_DATA_H

#include <gtkmm.h>
#include "AMTStatusData.h"
#ifdef WITH_TPM
#include "TPMStatusData.h"
#endif
#include "EventNode.h"
#include "EventHistory.h"

#define MESDAT_SKU_AMT      (1 << 0)
#ifdef WITH_TPM
#define MESDAT_SKU_TPM      (1 << 1)
#endif
#define MESDAT_SKU_AMTFUND  (1 << 2)

/* MEStatusData: This class is MEStatus data parser.
 */
class MEStatusData
{
public:
	enum NetDataIfaceType {NETDATA_IFACE_WIRED, NETDATA_IFACE_WIRELESS};

	//ctor
	MEStatusData();
	//dtor
	virtual ~MEStatusData();

	//set Sku
	void SetSku(unsigned int sku) { m_sku = sku; };
	//set AMT status data
	void SetAMTStatusData(const AMTStatusData &amtData);
#ifdef WITH_TPM
	//set TPM status data
	void SetTPMStatusData(const TPMStatusData &tpmData);
#endif
	//load EventHistory
	void LoadEventHistory(unsigned int maxSize = 0);

	//data getters
	Glib::ustring GetAMTStatus() const;
	Glib::ustring GetAMTVersion() const;
	Glib::ustring GetAMTMode() const;
	Glib::ustring GetLastHostResetReason() const;
	Glib::ustring GetMEResetReason() const;
	Glib::ustring GetAMTWebUI() const;
	Glib::ustring GetAMTDefense() const;
	Glib::ustring GetAMTRedirection() const;
	Glib::ustring GetAMTSOLRedirection() const;
	Glib::ustring GetAMTIDERRedirection() const;
	Glib::ustring GetAMTPowerPolicy() const;
	Glib::ustring GetAMTMEIVersion() const;
	Glib::ustring GetAMTLMSVersion() const;

	Glib::ustring GetAMTNetMode(NetDataIfaceType type) const;
	Glib::ustring GetAMTNetMac(NetDataIfaceType type) const;
	Glib::ustring GetAMTNetIP(NetDataIfaceType type) const;
	Glib::ustring GetAMTNetLinkStatus(NetDataIfaceType type) const;
#ifdef WITH_WIRELESS_AT
	Glib::ustring GetAMTNetWirelessConfInterface() const;
	Glib::ustring GetAMTNetWirelessConfWireless() const;
#endif

	Glib::ustring GetAMTRemoteAccessConnectionStatus() const;

#ifdef WITH_TPM
	Glib::ustring GetTPMState() const;
	Glib::ustring GetTPMActivation() const;
	Glib::ustring GetTPMOwnership() const;
	Glib::ustring GetTPMEnablement() const;
	Glib::ustring GetTPMFailure() const;
	Glib::ustring GetTPMStatus() const;
	Glib::ustring GetTPMTCGVersion() const;
	Glib::ustring GetTPMFWVersion() const;
#endif

	//get all event infos - in order from latest to oldest
	bool GetEventInfo(Glib::ustring &app, Glib::ustring &type,
			  Glib::ustring &reason, Glib::ustring &time,
			  Glib::ustring &isnew);

	//prepare for getting event infos - required between LoadEventHistory and GetEventInfo
	bool PrepareEventInfo();
	//prepare for getting new event infos - required between LoadEventHistory and GetNewEventInfo
	bool PrepareNewEventInfo();
	//get new event infos - in order from oldest to latest
	bool GetNewEventInfo(Glib::ustring &app, Glib::ustring &type,
				Glib::ustring &reason, Glib::ustring &time,
				Glib::ustring &isnew);

	//save last received event
	bool UpdateLastReadEvent();
	//save last seen event
	bool UpdateLastSeenEvent();
	//load previous received/seen events
	bool LoadLastUserEvent();

private:
	static const Glib::ustring STATUS_DATA_NOT_AVAILABLE;

	unsigned int m_sku;
	AMTStatusData m_amtData;
#ifdef WITH_TPM
	TPMStatusData m_tpmData;
#endif

	ATEvents::EventHistory m_eventHistoryReader;
	std::list<ATEvents::EventNode> m_eventHistory;
	bool m_haveEventHistory;
	std::list<ATEvents::EventNode>::const_iterator m_eventHistoryIter;
	ATEvents::EventNode m_lastSeenEvent;
	ATEvents::EventNode m_prevLastEvent;
	bool m_reachedIterLastSeenEvent;
	bool m_haveLastSeenEvent;
	bool m_havePrevLastEvent;

	bool getEventInfo(const ATEvents::EventNode &curEvent,
			  Glib::ustring &app, Glib::ustring &type,
			  Glib::ustring &reason, Glib::ustring &time);

	bool saveLastUserEvent();
	void appendSubstatesToTPMStatus(Glib::ustring &state) const;
};

#endif //_ME_STATUS_DATA_H
