/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

//******************************************************************************
//
// File:		MEStatusDlg.cpp
//
// Description:		Implementation for MEStatusDlg class
//                      The GUI of the MEStatus application.
//
//******************************************************************************

#ifdef HAVE_CONFIG_H 
#include "config.h" 
#endif
#include "iatshareddata.h"
#include "MEStatusDlg.h"
#include "IMEGuiCommands.h"

#include <glibmm/i18n.h>
#include "MEStatusDataDefinitions.h" 

#include "ConfigFile.h"
#include "Lock.h"
#include <fstream>

static const std::string lNames[] = {
	"labelgenamtstatus",
	"labelgentpmstatus",
	"labelamtstatus",
	"labelamtversion",
	"labelamtmode",
	"labelamtreset",
	"labelamtwebui",
	"labelamtdefense",
	"labelamtsolredir",
	"labelamtiderredir",
#ifdef WITH_TPM
	"labeltpmstatus",
	"labeltpmtcgver",
	"labeltpmversion",
#endif
	"labelamtepowerpol",
	"labelamtelms",
	"labelamteheci",
	"labelnetnamewire",
	"labelnetnameiface",
	"labelnetiface",
	"labelnetwire",
	"labelnetip",
	"labelnetlinkstatus",
	"labelnetmac",
	"labelnetmode",
	"labelgenamtstatusname",
	"labelgentpmstatusname",
#ifdef WITH_TPM
	"labeltabtpm",
	"labelframetpm",
#endif
	"labeltabamt",
	"labelframeamt",
	"labelamtexplain"
};

//ctor
MEStatusDlg::MEStatusDlg(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml> &refGlade)
: Gtk::Window(cobject), //Base class ctor
m_refGlade(refGlade)  //Reference to Glade Xml object
{
	m_url = "";
#ifdef WITH_WIRELESS_AT
	m_bAMTNetSelWireless = true;
#endif
	m_eventsMaxSize = 0;

	m_pLinkButtonBox = NULL;
	m_pLinkButton = NULL;
	m_pBrowserNotFoundDlg = NULL;
	m_pAMTExtendedDlg = NULL;
	m_pAMTNetworkDlg = NULL;
	m_pHelpDlg = NULL;

	for (int i = 0; i < MESTATUS_LABEL_COUNT; i++) {
		m_pLabels[i] = NULL;
	}

	m_pTabs = NULL;

	m_pNotifyChk = NULL;
	m_pPopupChk = NULL;

	m_pEventList = NULL;
	m_pNetIfacesList = NULL;

	m_pMEGuiCommands = NULL;

	m_dialog = NULL;

	m_pageAmt = NULL;
#ifdef WITH_TPM
	m_pageTpm = NULL;
#endif
}

//dtor
MEStatusDlg::~MEStatusDlg()
{
	closeMsgDialog();

	if (NULL != m_pLinkButton) {
		if (NULL != m_pLinkButtonBox) {
			m_pLinkButtonBox->remove(*m_pLinkButton);
		}
		delete m_pLinkButton;
		m_pLinkButton = NULL;
	}
}


/* Init: Initialize the dialog. 
 *  pMEGuiCommands is a pointer to the commands object, 
 *    which must already be initialized.
 *  strError is a reference to a string that will hold an error message
 *    in case initialization fails.
 **/
bool MEStatusDlg::Init(IMEGuiCommands *pMEGuiCommands, Glib::ustring &strError)
{
	//Reset strError
	strError = "";

	//Check parameters
	if (NULL == pMEGuiCommands) {
		strError = _("pMEGuiCommands is NULL");
		return false;
	}
	m_pMEGuiCommands = pMEGuiCommands;

	m_url = m_pMEGuiCommands->GetOEMUrl();
	m_eventsMaxSize = m_pMEGuiCommands->GetEventHistoryMaxReadSize();

	/***** Initialize Widgets *****/
	if (!initMainWidgets(strError)) {
		return false;
	}
	if (!initExtendedInfoDlgWidgets(strError)) {
		return false;
	}
	if (!initNetworkInfoDlgWidgets(strError)) {
		return false;
	}
	if (!initOtherDlgWidgets(strError)) {
		return false;
	}

	m_pageAmt = m_pTabs->get_nth_page(1);
	if (NULL == m_pageAmt) {
		strError = _("AMT page not defined in GUI");
		return false;
	}
#ifdef WITH_TPM
	m_pageTpm = m_pTabs->get_nth_page(2);
	if (NULL == m_pageTpm) {
		strError = _("TPM page not defined in GUI");
		return false;
	}
#endif
	m_pageAmt->hide_all();
#ifdef WITH_TPM
	m_pageTpm->hide_all();
#else
	m_pTabs->remove_page(2);
#endif
	if (NULL != m_pLabels[LABEL_GEN_NAME_TPM_status]) {
		m_pLabels[LABEL_GEN_NAME_TPM_status]->hide();
	}
	if (NULL != m_pLabels[LABEL_GEN_TPM_status]) {
		m_pLabels[LABEL_GEN_TPM_status]->hide();
	}

	//All done
	return true;
}

// MAIN WINDOW WIDGETS
bool MEStatusDlg::initMainWidgets(Glib::ustring &strError)
{
	Gtk::Button *pCloseButton = NULL;

	//Init close button
	m_refGlade->get_widget("closeservices", pCloseButton);
	if (NULL == pCloseButton) {
		strError = _("Couldn't get Close button");
		return false;
	}
	pCloseButton->signal_clicked().connect(
	    sigc::mem_fun(*this, &MEStatusDlg::OnCloseButton));

	//Init labels
	if (MESTATUS_LABEL_COUNT != (sizeof(lNames) / sizeof(lNames[0]))) {
		strError = _("Label names count and Label count mismatch");
		return false;
	}
	for (int i = 0; i < MESTATUS_LABEL_COUNT; i++) {
		m_refGlade->get_widget(lNames[i], m_pLabels[i]);
		/* //labels can be missing - if OEM decided to change GUI
		if (NULL == m_pLabels[i]) {
			strError = _("Couldn't get ") + lNames[i] + _(" label");
			return false;
		}
		*/
	}

	//Init tabs
	m_refGlade->get_widget("vprotabs", m_pTabs);
	if (NULL == m_pTabs) {
		strError = _("Couldn't get vprotabs container");
		return false;
	}

	//Init startup notification checkbox
	m_refGlade->get_widget("checkbuttonshowevery", m_pNotifyChk);
	if (NULL == m_pNotifyChk) {
		strError = _("Couldn't get checkButtonShowEvery");
		return false;
	}
	if (m_pNotifyChk->is_visible()) {
		m_pNotifyChk->set_active(m_pMEGuiCommands->GetStatusIconAlways());
		m_pNotifyChk->signal_toggled().connect(
		    sigc::mem_fun(*this, &MEStatusDlg::OnShowEveryChk));
		m_pMEGuiCommands->SetShowAlways(MES_SHOW_ALWAYS_USER);
	} else {
		m_pMEGuiCommands->SetShowAlways((m_pNotifyChk->get_active()) ?
			MES_SHOW_ALWAYS_YES :
			MES_SHOW_ALWAYS_NO);
	}

	//Init popup notification checkbox
	m_refGlade->get_widget("checkbuttonshowpopup", m_pPopupChk);
	if (NULL == m_pPopupChk) {
		strError = _("Couldn't get checkButtonShowPopup");
		return false;
	}
	m_pPopupChk->set_active(m_pMEGuiCommands->GetPopupForCurrentUser());
	m_pPopupChk->signal_toggled().connect(
	    sigc::mem_fun(*this, &MEStatusDlg::OnPopupChk));

	if (!initEventsList(strError)) {
		return false;
	}

	return true;
}

bool MEStatusDlg::initEventsList(Glib::ustring &strError)
{
	//Init event list
	m_refGlade->get_widget("eventlist", m_pEventList);
	if (NULL == m_pEventList) {
		strError = _("Couldn't get event list");
		return false;
	}
	m_refEventsStore = Gtk::ListStore::create(m_eventsColumns);
	m_pEventList->set_model(m_refEventsStore);
	m_pEventList->append_column(MESD_RES_EVENT_LIST_COL_NAME_APPLICATION,
				    m_eventsColumns.m_application);
	m_pEventList->append_column(MESD_RES_EVENT_LIST_COL_NAME_TYPE,
				    m_eventsColumns.m_type);
	m_pEventList->append_column(MESD_RES_EVENT_LIST_COL_NAME_TIME,
				    m_eventsColumns.m_time);
	m_pEventList->insert_column(MESD_RES_EVENT_LIST_COL_NAME_COMMENT,
				    m_eventsColumns.m_comment, 0);

	return true;
}

bool MEStatusDlg::initExtendedInfoDlgWidgets(Glib::ustring &strError)
{
	Gtk::Button *pAMTExtendedButton = NULL;
	Gtk::Button *pAMTExtendedClose = NULL;

	//Init AMT "Extended System Details" button
	m_refGlade->get_widget("buttonamtextended", pAMTExtendedButton);
	if (NULL == pAMTExtendedButton) {
		strError = _("Couldn't get AMT Extended System Details button");
		return false;
	}
	pAMTExtendedButton->signal_clicked().connect(
	    sigc::mem_fun(*this, &MEStatusDlg::OnAMTExtendedButton));

	//Init AMT Extended System Details dialog
	m_refGlade->get_widget("amtextendedDlg", m_pAMTExtendedDlg);
	if (NULL == m_pAMTExtendedDlg) {
		strError = _("Couldn't get AMT Extended System Details dialog");
		return false;
	}

	//Init AMT Extended System Details dialog close button
	m_refGlade->get_widget("closeamtextended", pAMTExtendedClose);
	if (NULL != pAMTExtendedClose) {
		pAMTExtendedClose->signal_clicked().connect(
		    sigc::mem_fun(*this, &MEStatusDlg::OnAMTExtendedCloseButton));
	}

	return true;
}

bool MEStatusDlg::initNetworkInfoDlgWidgets(Glib::ustring &strError)
{
	Gtk::Button *pAMTNetworkButton = NULL;
	Gtk::Button *pAMTNetworkClose = NULL;

	//Init AMT "Network Information" button
	m_refGlade->get_widget("buttonamtnetworkinfo", pAMTNetworkButton);
	if (NULL == pAMTNetworkButton) {
		strError = _("Couldn't get AMT Network Information button");
		return false;
	}
	pAMTNetworkButton->signal_clicked().connect(
	    sigc::mem_fun(*this, &MEStatusDlg::OnAMTNetworkButton));

	//Init AMT Network Information dialog
	m_refGlade->get_widget("amtnetworkinfoDlg", m_pAMTNetworkDlg);
	if (NULL == m_pAMTNetworkDlg) {
		strError = _("Couldn't get AMT Network Information dialog");
		return false;
	}

	//Init AMT Network Information dialog close button
	m_refGlade->get_widget("closeamtnetworkinfo", pAMTNetworkClose);
	if (NULL != pAMTNetworkClose) {
		pAMTNetworkClose->signal_clicked().connect(
		    sigc::mem_fun(*this, &MEStatusDlg::OnAMTNetworkCloseButton));
	}

	if (!initNetworkInfoList(strError)) {
		return false;
	}

	return true;
}

bool MEStatusDlg::initNetworkInfoList(Glib::ustring &strError)
{
	//Init AMT Network Interfaces list
	m_refGlade->get_widget("listnetifaces", m_pNetIfacesList);
	if (NULL == m_pNetIfacesList) {
		strError = _("Couldn't get AMT Network Interfaces list");
		return false;
	}
	Glib::RefPtr<Gdk::Pixbuf> iconWired;
#ifdef WITH_WIRELESS_AT
	Glib::RefPtr<Gdk::Pixbuf> iconWireless;
#endif
	try
	{
		//Load icons for network interface list from files
		iconWired    = Gdk::Pixbuf::create_from_file(MESTATUS_RESPATH "/wired.ico");
#ifdef WITH_WIRELESS_AT
		iconWireless = Gdk::Pixbuf::create_from_file(MESTATUS_RESPATH "/wireless.ico");
#endif
	}
	catch (Glib::FileError &) {
		strError = _("Couldn't load AMT Network Interfaces list icon file");
		return false;
	}
	Gtk::TreeModel::iterator iter;
	Gtk::TreeModel::Row row;
	m_refNetIfacesStore = Gtk::ListStore::create(m_netIfacesColumns);
	if (!m_refNetIfacesStore) {
		strError = _("Couldn't create AMT Network Interfaces list store");
		return false;
	}
#ifdef WITH_WIRELESS_AT
	iter = m_refNetIfacesStore->append();
	row = *iter;
	row[m_netIfacesColumns.m_icon] = iconWireless;
	row[m_netIfacesColumns.m_name] = MESD_RES_AMTNET_WIRELESS_CONN_LIST_ROW_NAME;
#endif
	iter = m_refNetIfacesStore->append();
	row = *iter;
	row[m_netIfacesColumns.m_icon] = iconWired;
	row[m_netIfacesColumns.m_name] = MESD_RES_AMTNET_WIRED_CONN_LIST_ROW_NAME;
	m_pNetIfacesList->set_model(m_refNetIfacesStore);
	m_pNetIfacesList->append_column(_("Icon"), m_netIfacesColumns.m_icon);
	m_pNetIfacesList->append_column(_("Interface"), m_netIfacesColumns.m_name);

	Gtk::TreeViewColumn* column = m_pNetIfacesList->get_column(0);
	column->set_fixed_width(40);
	column->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
	column = m_pNetIfacesList->get_column(1);
	column->set_sizing(Gtk::TREE_VIEW_COLUMN_FIXED);
	m_pNetIfacesList->set_fixed_height_mode();

	m_refNetIfacesSelection = m_pNetIfacesList->get_selection();
	m_refNetIfacesSelection->signal_changed().connect(
	    sigc::mem_fun(*this, &MEStatusDlg::OnNetIfaceSelChanged));
	m_refNetIfacesSelection->select(m_refNetIfacesStore->children().begin());

	return true;
}

bool MEStatusDlg::initOtherDlgWidgets(Glib::ustring &strError)
{
	if (!initBrowserNotFoundDlgWidgets(strError)) {
		return false;
	}

	Gtk::EventBox *pLinkButton = NULL;
	m_refGlade->get_widget("eventboxoemlogo", pLinkButton);
	if (NULL != pLinkButton) {
		pLinkButton->set_events(Gdk::BUTTON_PRESS_MASK);
		pLinkButton->signal_button_press_event().connect(
		    sigc::mem_fun(*this, &MEStatusDlg::OnLinkEButton));
	}

	if (!initHelpDlgWidgets(strError)) {
		return false;
	}

	return true;
}

bool MEStatusDlg::initBrowserNotFoundDlgWidgets(Glib::ustring &strError)
{
	Gtk::Entry *pUrl = NULL;
	Gtk::Button *pBrowserNotFoundOk = NULL;

	//Init "Browser Not Found" dialog
	m_refGlade->get_widget("BrowserNotFoundDlg", m_pBrowserNotFoundDlg);
	if (NULL == m_pBrowserNotFoundDlg) {
		strError = _("Couldn't get Browser Not Found dialog");
		return false;
	}

	m_refGlade->get_widget("BrowserNotFoundUrl", pUrl);
	if (NULL == pUrl) {
		strError = _("Couldn't get browser not found url");
		return false;
	}
	if (m_url == m_pMEGuiCommands->GetDefaultOEMUrl()) {
		m_url = pUrl->get_text();
	} else {
		pUrl->set_text(m_url);
	}

	//Init "Browser Not Found" dialog OK button
	m_refGlade->get_widget("BrowserNotFoundOk", pBrowserNotFoundOk);
	if (NULL != pBrowserNotFoundOk) {
		pBrowserNotFoundOk->signal_clicked().connect(
		    sigc::mem_fun(*this, &MEStatusDlg::OnBrowserNotFoundOkButton));
	}

	return true;
}

bool MEStatusDlg::initHelpDlgWidgets(Glib::ustring &strError)
{
	Gtk::Button *pHelpButton = NULL;
	//Gtk::Button *pHelpClose = NULL;

	//Init help button
	m_refGlade->get_widget("buttonhelp", pHelpButton);
	if (NULL == pHelpButton) {
		strError = _("Couldn't get Help button");
		return false;
	}
	pHelpButton->signal_clicked().connect(
	    sigc::mem_fun(*this, &MEStatusDlg::OnHelpButton));

	//Init "Help" dialog
	m_refGlade->get_widget("helpDlg", m_pHelpDlg);
	if (NULL == m_pHelpDlg) {
		strError = _("Couldn't get Help dialog");
		return false;
	}

	/*
	//Init Help dialog close button
	m_refGlade->get_widget("closehelp", pHelpClose);
	if (NULL != pHelpClose) {
		pHelpClose->signal_clicked().connect(
		    sigc::mem_fun(*this, &MEStatusDlg::OnHelpCloseButton));
	}
	*/

	m_refGlade->get_widget("helpLinksButtonBox", m_pLinkButtonBox);
	if (NULL != m_pLinkButtonBox) {
		m_pLinkButton = new Gtk::LinkButton(m_url,
		    MESD_RES_HELP_MORE_DETAILS);
		if (NULL == m_pLinkButton) {
			strError = _("Couldn't create link button");
			return false;
		}
		m_pLinkButton->signal_clicked().connect(
		    sigc::mem_fun(*this, &MEStatusDlg::OnLinkButton));
		m_pLinkButtonBox->add(*m_pLinkButton);
		m_pLinkButton->show();
	}

	return true;
}


//load Event history data
void MEStatusDlg::LoadInitialEventHistoryData()
{
	m_data.LoadLastUserEvent();
	m_data.LoadEventHistory(m_eventsMaxSize);
	loadEventHistory();
	m_data.UpdateLastReadEvent();
}



//Update AMT status data
void MEStatusDlg::OnAMTStatusData(AMTStatusData &amtData)
{
	m_data.SetAMTStatusData(amtData);
	updateAMTData();
}

#ifdef WITH_TPM
//Update TPM status data
void MEStatusDlg::OnTPMStatusData(TPMStatusData &tpmData)
{
	m_data.SetTPMStatusData(tpmData);
	updateTPMData();
}
#endif

//Update Event history data
void MEStatusDlg::OnEventHistory()
{
	m_data.LoadEventHistory(m_eventsMaxSize);
	updateEventHistory();
	m_data.UpdateLastReadEvent();
}

//Display notification popup
bool MEStatusDlg::Notify(const Glib::ustring &title, const Glib::ustring &body, int show)
{
	Gtk::MessageType mtype = (((show & MESGUI_NOTIFY_WARNING) != 0) ? Gtk::MESSAGE_WARNING
			    : ((show & MESGUI_NOTIFY_ERROR) != 0) ? Gtk::MESSAGE_ERROR
			    : ((show & MESGUI_NOTIFY_QUESTION) != 0) ? Gtk::MESSAGE_QUESTION
			    : Gtk::MESSAGE_INFO);

	closeMsgDialog();

	PresentStatusDialog();

	Lock mdl(m_msgDialogSem);

	m_dialog = new Gtk::MessageDialog(*this, body,
					  ((show & MESGUI_NOTIFY_USEMARKUP) != 0),
					  mtype);
	if (NULL == m_dialog) {
		return false;
	}
	m_dialog->set_title((title == MESD_RES_NOTIF_ANY) ? MESD_RES_NOTIF_ANY_DLG : title);
	m_dialog->signal_response().connect(
	    sigc::mem_fun(*this, &MEStatusDlg::OnNotifyClosed));
	m_dialog->present();
	return true;
}

//Nofitication popup closed
void MEStatusDlg::OnNotifyClosed(int response_id)
{
	closeMsgDialog();

	if (NULL != m_pMEGuiCommands) {
		m_pMEGuiCommands->NotificationClosed();
	}
}

void MEStatusDlg::closeMsgDialog()
{
	Lock mdl(m_msgDialogSem);

	if (NULL != m_dialog) {
		m_dialog->hide();
		delete m_dialog;
		m_dialog = NULL;
	}
}

//Order to present dialog
void MEStatusDlg::PresentStatusDialog()
{
	if (NULL != m_pTabs) {
		m_pTabs->set_current_page(0);
	}
	present();
	if (NULL != m_pMEGuiCommands) {
		m_pMEGuiCommands->SetAppPresence(true);
	}
}

//show/hide SKU tabs
void MEStatusDlg::OnSku(unsigned int sku)
{
	m_data.SetSku(sku);

	if (NULL != m_pageAmt) {
		if (NULL != m_pLabels[LABEL_GEN_NAME_AMT_status]) {
			if ((sku & MESGUI_SKU_AMTFUND) != 0) {
				m_pLabels[LABEL_GEN_NAME_AMT_status]->set_label(MESD_RES_GEN_NAME_AMT_BASIC);
			} else {
				m_pLabels[LABEL_GEN_NAME_AMT_status]->set_label(MESD_RES_GEN_NAME_AMT);
			}
		}
		if ((sku & MESGUI_SKU_AMT) != 0) {
			if ((sku & MESGUI_SKU_AMTFUND) != 0) {
				if (NULL != m_pLabels[LABEL_TAB_amt]) {
					m_pLabels[LABEL_TAB_amt]->set_label(MESD_RES_TAB_AMT_BASIC);
				}
				if (NULL != m_pLabels[LABEL_FRAME_amt]) {
					m_pLabels[LABEL_FRAME_amt]->set_label(MESD_RES_FRAME_AMT_BASIC);
				}
				if (NULL != m_pLabels[LABEL_AMT_explain]) {
					m_pLabels[LABEL_AMT_explain]->set_label(MESD_RES_AMT_EXPLAIN_BASIC);
				}
				if (NULL != m_pAMTNetworkDlg) {
					m_pAMTNetworkDlg->set_title(MESD_RES_NETWORK_DLG_TITLE_BASIC);
				}
			} else {
				if (NULL != m_pLabels[LABEL_TAB_amt]) {
					m_pLabels[LABEL_TAB_amt]->set_label(MESD_RES_TAB_AMT);
				}
				if (NULL != m_pLabels[LABEL_FRAME_amt]) {
					m_pLabels[LABEL_FRAME_amt]->set_label(MESD_RES_FRAME_AMT);
				}
				if (NULL != m_pLabels[LABEL_AMT_explain]) {
					m_pLabels[LABEL_AMT_explain]->set_label(MESD_RES_AMT_EXPLAIN);
				}
				if (NULL != m_pAMTNetworkDlg) {
					m_pAMTNetworkDlg->set_title(MESD_RES_NETWORK_DLG_TITLE);
				}
			}
			m_pageAmt->show_all();
		} else {
			m_pageAmt->hide_all();
		}
	}
#ifdef WITH_TPM
	if (NULL != m_pageTpm) {
		if ((sku & MESGUI_SKU_TPM) != 0) {
			m_pageTpm->show_all();
			if (NULL != m_pLabels[LABEL_GEN_NAME_TPM_status]) {
				m_pLabels[LABEL_GEN_NAME_TPM_status]->show();
			}
			if (NULL != m_pLabels[LABEL_GEN_TPM_status]) {
				m_pLabels[LABEL_GEN_TPM_status]->show();
			}
		} else {
			m_pageTpm->hide_all();
			if (NULL != m_pLabels[LABEL_GEN_NAME_TPM_status]) {
				m_pLabels[LABEL_GEN_NAME_TPM_status]->hide();
			}
			if (NULL != m_pLabels[LABEL_GEN_TPM_status]) {
				m_pLabels[LABEL_GEN_TPM_status]->hide();
			}
		}
	}
#endif

	loadEventHistory();
}

//delete event received
bool MEStatusDlg::on_delete_event(GdkEventAny *event)
{
	OnCloseButton();
	return true;
}

//Main window Close button was clicked
void MEStatusDlg::OnCloseButton()
{
	//close child popup windows
	if (NULL != m_pAMTExtendedDlg) {
		m_pAMTExtendedDlg->hide();
	}
	if (NULL != m_pAMTNetworkDlg) {
		m_pAMTNetworkDlg->hide();
	}
	if (NULL != m_pHelpDlg) {
		m_pHelpDlg->hide();
	}

	//save last seen event
	m_data.UpdateLastSeenEvent();

	if (NULL != m_pMEGuiCommands) {
		//tell logic we closed window
		m_pMEGuiCommands->SetAppPresence(false);

		//remove "New" flags from events in event history list
		if (!(m_pMEGuiCommands->GetModeApplication())) {
			Lock l(m_eventsStoreSem);

			Gtk::TreeModel::Children cont = m_refEventsStore->children();
			Gtk::TreeModel::iterator iter = cont.begin();
			Gtk::TreeModel::iterator iend = cont.end();
			while ((iter != iend)
			    && (MESD_RES_EVENT_ISNEW ==
				((Gtk::TreeModel::Row)*iter)[m_eventsColumns.m_comment])
			    )
			{
				((Gtk::TreeModel::Row)*iter)[m_eventsColumns.m_comment] =
				    MESD_RES_EVENT_REVIEWED;
				iter++;
			}
		}
	}

	hide();

	if (NULL != m_pMEGuiCommands && m_pMEGuiCommands->GetModeApplication()) {
		Gtk::Main::quit();
	}
}

void MEStatusDlg::OnShowEveryChk()
{
	//save checkbox changed state
	if (m_pMEGuiCommands->GetStatusIconAlways()) {
		if (m_pNotifyChk && (!(m_pNotifyChk->get_active()))) {
			m_pMEGuiCommands->SetStatusIconAlways(false);
		}
	} else {
		if (m_pNotifyChk && (m_pNotifyChk->get_active())) {
			m_pMEGuiCommands->SetStatusIconAlways(true);
		}
	}
}

void MEStatusDlg::OnPopupChk()
{
	//save checkbox changed state
	if (m_pMEGuiCommands->GetPopupForCurrentUser()) {
		if (m_pNotifyChk && (!(m_pPopupChk->get_active()))) {
			m_pMEGuiCommands->SetPopupForCurrentUser(false);
		}
	} else {
		if (m_pNotifyChk && (m_pPopupChk->get_active())) {
			m_pMEGuiCommands->SetPopupForCurrentUser(true);
		}
	}
}

//Main window Help button was clicked
void MEStatusDlg::OnHelpButton()
{
	if (NULL != m_pHelpDlg) {
		m_pHelpDlg->present();
	}
}

//Help window Close button was clicked
void MEStatusDlg::OnHelpCloseButton()
{
	if (NULL != m_pHelpDlg) {
		m_pHelpDlg->hide();
	}
}

// "Browser not found" message dialog's Ok button was clicked
void MEStatusDlg::OnBrowserNotFoundOkButton()
{
	if (NULL != m_pBrowserNotFoundDlg) {
		m_pBrowserNotFoundDlg->hide();
	}
}
// "AMT Extended System Details" button was clicked
void MEStatusDlg::OnAMTExtendedButton()
{
	if (NULL != m_pAMTExtendedDlg) {
		m_pAMTExtendedDlg->present();
	}
}
// "AMT Extended System Details" message dialog's Close button was clicked
void MEStatusDlg::OnAMTExtendedCloseButton()
{
	if (NULL != m_pAMTExtendedDlg) {
		m_pAMTExtendedDlg->hide();
	}
}
// "AMT Network Information" button was clicked
void MEStatusDlg::OnAMTNetworkButton()
{
	if (NULL != m_pAMTNetworkDlg) {
		if (m_refNetIfacesStore) {
			m_refNetIfacesSelection->select(m_refNetIfacesStore->children().begin());
		}
		m_pAMTNetworkDlg->present();
	}
}
// "AMT Network Information" message dialog's Close button was clicked
void MEStatusDlg::OnAMTNetworkCloseButton()
{
	if (NULL != m_pAMTNetworkDlg) {
		m_pAMTNetworkDlg->hide();
	}
}

//Network interface selection changed
void MEStatusDlg::OnNetIfaceSelChanged()
{
	Gtk::TreeModel::const_iterator iter = m_refNetIfacesSelection->get_selected();
	Gtk::TreeModel::Row row = *iter;
	Glib::ustring selniface = row[m_netIfacesColumns.m_name];
#ifdef WITH_WIRELESS_AT
	m_bAMTNetSelWireless = (0 == selniface.compare(
	    Glib::ustring(MESD_RES_AMTNET_WIRELESS_CONN_LIST_ROW_NAME)));

	if (m_bAMTNetSelWireless) {
		if (NULL != m_pLabels[LABEL_NET_NAME_wire]) {
			m_pLabels[LABEL_NET_NAME_wire]->show();
		}
		if (NULL != m_pLabels[LABEL_NET_wire]) {
			m_pLabels[LABEL_NET_wire]->show();
		}
		/*
		if (NULL != m_pLabels[LABEL_NET_NAME_iface]) {
			m_pLabels[LABEL_NET_NAME_iface]->show();
		}
		if (NULL != m_pLabels[LABEL_NET_iface]) {
			m_pLabels[LABEL_NET_iface]->show();
		}
		*/
	} else
#endif
	{
		if (NULL != m_pLabels[LABEL_NET_NAME_wire]) {
			m_pLabels[LABEL_NET_NAME_wire]->hide();
		}
		if (NULL != m_pLabels[LABEL_NET_wire]) {
			m_pLabels[LABEL_NET_wire]->hide();
		}
		/*
		if (NULL != m_pLabels[LABEL_NET_NAME_iface]) {
			m_pLabels[LABEL_NET_NAME_iface]->hide();
		}
		if (NULL != m_pLabels[LABEL_NET_iface]) {
			m_pLabels[LABEL_NET_iface]->hide();
		}
		*/
	}

	updateAMTNetworkData();
}

//The "click here" link button was clicked
bool MEStatusDlg::OnLinkEButton(const GdkEventButton *)
{
	OnLinkButton();
	return true;
}
void MEStatusDlg::OnLinkButton()
{
	bool bBrowserConfigured = false;
	std::string browser;

	std::ifstream infile("~/.IntelAMTCheck.browser.conf");
	if (infile.is_open()) {
		ConfigFile browserConfig;
		infile >> browserConfig;
		if (browserConfig.readInto(browser, "browser")) {
			bBrowserConfigured = true;
		}
	}

	static const char *browsers[] = {"firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape"};
	static const int num_browsers = sizeof(browsers) / sizeof(browsers[0]);

	int i = 0;
	bool bInvokeSuccess = false;
	if (!bBrowserConfigured) {
		browser = browsers[i++];
	}

	do {
		bInvokeSuccess = InvokeBrowser(browser, m_url);
		browser = browsers[i++];
	} while ((!bInvokeSuccess) && (i < num_browsers));

	if (!bInvokeSuccess) {
		m_pBrowserNotFoundDlg->show();
	}
}

//Invoke specified web browser with specified url. 
//Returns true on success, false otherwise.
bool MEStatusDlg::InvokeBrowser(const std::string &browser, const std::string &url)
{
	if (Glib::find_program_in_path(browser) == "") {
		//browser was not found in path - invoke failed.
		return false;
	}

	system((browser + " " + url + " &").c_str());
	return true;
}

//Update displayed AMT data
void MEStatusDlg::updateAMTData()
{
	updateAMTMainData();
	updateAMTExtendedData();
	updateAMTNetworkData();
}

//Update displayed AMT data in GUI main window
void MEStatusDlg::updateAMTMainData()
{
	if (NULL != m_pLabels[LABEL_GEN_AMT_status]) {
		m_pLabels[LABEL_GEN_AMT_status]->set_text(m_data.GetAMTStatus());
	}
	if (NULL != m_pLabels[LABEL_AMT_status]) {
		m_pLabels[LABEL_AMT_status]->set_text(m_data.GetAMTStatus());
	}
	if (NULL != m_pLabels[LABEL_AMT_version]) {
		m_pLabels[LABEL_AMT_version]->set_text(m_data.GetAMTVersion());
	}
	if (NULL != m_pLabels[LABEL_AMT_mode]) {
		m_pLabels[LABEL_AMT_mode]->set_text(m_data.GetAMTMode());
	}
	if (NULL != m_pLabels[LABEL_AMT_reset]) {
		m_pLabels[LABEL_AMT_reset]->set_text(m_data.GetMEResetReason());
	}
	if (NULL != m_pLabels[LABEL_AMT_webui]) {
		m_pLabels[LABEL_AMT_webui]->set_text(m_data.GetAMTWebUI());
	}
	if (NULL != m_pLabels[LABEL_AMT_defense]) {
		m_pLabels[LABEL_AMT_defense]->set_text(m_data.GetAMTDefense());
	}
	if (NULL != m_pLabels[LABEL_AMT_solredir]) {
		m_pLabels[LABEL_AMT_solredir]->set_text(m_data.GetAMTSOLRedirection());
	}
	if (NULL != m_pLabels[LABEL_AMT_iderredir]) {
		m_pLabels[LABEL_AMT_iderredir]->set_text(m_data.GetAMTIDERRedirection());
	}
}

//Update displayed AMT data in GUI extended info window
void MEStatusDlg::updateAMTExtendedData()
{
	if (NULL != m_pLabels[LABEL_AMTe_powerpol]) {
		m_pLabels[LABEL_AMTe_powerpol]->set_text(m_data.GetAMTPowerPolicy());
	}
	if (NULL != m_pLabels[LABEL_AMTe_heci]) {
		m_pLabels[LABEL_AMTe_heci]->set_text(m_data.GetAMTMEIVersion());
	}
	if (NULL != m_pLabels[LABEL_AMTe_lms]) {
		m_pLabels[LABEL_AMTe_lms]->set_text(m_data.GetAMTLMSVersion());
	}
}

//Update displayed AMT data in GUI network interfaces info window
void MEStatusDlg::updateAMTNetworkData()
{
#ifdef WITH_WIRELESS_AT
	if (m_bAMTNetSelWireless) {
		if (NULL != m_pLabels[LABEL_NET_mode]) {
			m_pLabels[LABEL_NET_mode]->set_text(
			    //m_data.GetAMTNetMode(MEStatusData::NETDATA_IFACE_WIRELESS));
			    m_data.GetAMTNetWirelessConfInterface());
		}
		if (NULL != m_pLabels[LABEL_NET_mac]) {
			m_pLabels[LABEL_NET_mac]->set_text(
			    m_data.GetAMTNetMac(MEStatusData::NETDATA_IFACE_WIRELESS));
		}
		if (NULL != m_pLabels[LABEL_NET_ip]) {
			m_pLabels[LABEL_NET_ip]->set_text(
			    m_data.GetAMTNetIP(MEStatusData::NETDATA_IFACE_WIRELESS));
		}
		if (NULL != m_pLabels[LABEL_NET_linkstatus]) {
			m_pLabels[LABEL_NET_linkstatus]->set_text(
			    m_data.GetAMTNetLinkStatus(MEStatusData::NETDATA_IFACE_WIRELESS));
		}
		/*
		if (NULL != m_pLabels[LABEL_NET_iface]) {
			m_pLabels[LABEL_NET_iface]->set_text(
			    m_data.GetAMTNetWirelessConfInterface());
		}
		*/
		if (NULL != m_pLabels[LABEL_NET_wire]) {
			m_pLabels[LABEL_NET_wire]->set_text(
			    m_data.GetAMTNetWirelessConfWireless());
		}
	} else
#endif
	{
		if (NULL != m_pLabels[LABEL_NET_mode]) {
			m_pLabels[LABEL_NET_mode]->set_text(
			    m_data.GetAMTNetMode(MEStatusData::NETDATA_IFACE_WIRED));
		}
		if (NULL != m_pLabels[LABEL_NET_mac]) {
			m_pLabels[LABEL_NET_mac]->set_text(
			    m_data.GetAMTNetMac(MEStatusData::NETDATA_IFACE_WIRED));
		}
		if (NULL != m_pLabels[LABEL_NET_ip]) {
			m_pLabels[LABEL_NET_ip]->set_text(
			    m_data.GetAMTNetIP(MEStatusData::NETDATA_IFACE_WIRED));
		}
		if (NULL != m_pLabels[LABEL_NET_linkstatus]) {
			m_pLabels[LABEL_NET_linkstatus]->set_text(
			    m_data.GetAMTNetLinkStatus(MEStatusData::NETDATA_IFACE_WIRED));
		}
	}
}

#ifdef WITH_TPM
//Update displayed TPM data
void MEStatusDlg::updateTPMData()
{
	if (NULL != m_pLabels[LABEL_GEN_TPM_status]) {
		m_pLabels[LABEL_GEN_TPM_status]->set_text(m_data.GetTPMState());
	}
	if (NULL != m_pLabels[LABEL_TPM_status]) {
		m_pLabels[LABEL_TPM_status]->set_text(m_data.GetTPMStatus());
	}
	if (NULL != m_pLabels[LABEL_TPM_tcgver]) {
		m_pLabels[LABEL_TPM_tcgver]->set_text(m_data.GetTPMTCGVersion());
	}
	if (NULL != m_pLabels[LABEL_TPM_version]) {
		m_pLabels[LABEL_TPM_version]->set_text(m_data.GetTPMFWVersion());
	}
}
#endif

//load displayed EventHistory
void MEStatusDlg::loadEventHistory()
{
	Glib::RefPtr<Gtk::ListStore> zeromodel;
	Glib::ustring app;
	Glib::ustring type;
	Glib::ustring reason;
	Glib::ustring time;
	Glib::ustring isnew;

	Lock l(m_eventsStoreSem);

	m_pEventList->set_model(zeromodel);

	m_refEventsStore->clear();

	if (m_data.PrepareEventInfo()) {
		while (m_data.GetEventInfo(app, type, reason, time, isnew)) {
			Gtk::TreeModel::iterator iter = m_refEventsStore->append();
			Gtk::TreeModel::Row row = *iter;
			row[m_eventsColumns.m_application] = app;
			row[m_eventsColumns.m_type] = ((reason.empty()) ? (type) : (type + " - " + reason));
			row[m_eventsColumns.m_time] = time;
			row[m_eventsColumns.m_comment] = isnew;
		}
	}

	m_pEventList->set_model(m_refEventsStore);
}

//update displayed EventHistory
void MEStatusDlg::updateEventHistory()
{
	Glib::ustring app;
	Glib::ustring type;
	Glib::ustring reason;
	Glib::ustring time;
	Glib::ustring isnew;

	Lock l(m_eventsStoreSem);

	if (m_data.PrepareNewEventInfo()) {
		while (m_data.GetNewEventInfo(app, type, reason, time, isnew)) {
			Gtk::TreeModel::iterator iter = m_refEventsStore->prepend();
			Gtk::TreeModel::Row row = *iter;
			row[m_eventsColumns.m_application] = app;
			row[m_eventsColumns.m_type] = ((reason.empty()) ? (type) : (type + " - " + reason));
			row[m_eventsColumns.m_time] = time;
			row[m_eventsColumns.m_comment] = isnew;
			m_pEventList->scroll_to_row(m_refEventsStore->get_path(iter));
		}
	}
}

