/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

//******************************************************************************
//
// File:		MEStatusDlg.h
//
// Description:		Interface for MEStatusDlg class
//                      The GUI of the MEStatus application.
//
//******************************************************************************

#ifndef _ME_STATUS_DLG_H
#define _ME_STATUS_DLG_H

#include <gtk/gtkversion.h>
#if (!GTK_CHECK_VERSION(2,10,0))
#include <igtkxmm/linkbutton.h>
#endif
#include <gtkmm.h>
#include <libglademm.h>
#include "IMEGuiCommands.h"
#include "IMEGuiNotifications.h"
#include "MEStatusData.h"
#include "Semaphore.h"

/* MEStatusDlg: This class is the main application window. It gets displayed when the
 * application is loaded (configuration-dependant), when the user clicks the status
 * icon, or when the user chooses "Show status" from the status icon's popup menu.
 */
class MEStatusDlg : public Gtk::Window, public IMEGuiNotifications
{
public:
	//ctor
	MEStatusDlg(BaseObjectType *cobject, const Glib::RefPtr<Gnome::Glade::Xml> &refGlade);
	//dtor
	virtual ~MEStatusDlg();
	/*Init: Initialize the dialog. pMEGuiCommands is a pointer to the commands object, which must
		already be initialized. strError is a reference to a string that will hold an error message
		in case initialization fails.*/
	bool Init(IMEGuiCommands *pMEGuiCommands, Glib::ustring &strError);

	/*From IMEGuiNotifications*/
	//Notification of AMT status data change
	virtual void OnAMTStatusData(AMTStatusData &amtData);
#ifdef WITH_TPM
	//Notification of TPM status data change
	virtual void OnTPMStatusData(TPMStatusData &tpmData);
#endif
	/* GUI widgets data load subroutines */
	virtual void LoadInitialEventHistoryData();
	//Notification of Event history data change
	virtual void OnEventHistory();
	//Display notification popup
	virtual bool Notify(const Glib::ustring &title, const Glib::ustring &body, int show = MESGUI_NOTIFY_INFO);
	//Order to present dialog
	virtual void PresentStatusDialog();
	//Display/hide SKU tabs
	virtual void OnSku(unsigned int sku);

protected:

	enum LabelId {
		LABEL_GEN_AMT_status = 0,
		LABEL_GEN_TPM_status,
		LABEL_AMT_status,
		LABEL_AMT_version,
		LABEL_AMT_mode,
		LABEL_AMT_reset,
		LABEL_AMT_webui,
		LABEL_AMT_defense,
		LABEL_AMT_solredir,
		LABEL_AMT_iderredir,
#ifdef WITH_TPM
		LABEL_TPM_status,
		LABEL_TPM_tcgver,
		LABEL_TPM_version,
#endif
		LABEL_AMTe_powerpol,
		LABEL_AMTe_lms,
		LABEL_AMTe_heci,
		LABEL_NET_NAME_wire,
		LABEL_NET_NAME_iface,
		LABEL_NET_iface,
		LABEL_NET_wire,
		LABEL_NET_ip,
		LABEL_NET_linkstatus,
		LABEL_NET_mac,
		LABEL_NET_mode,
		LABEL_GEN_NAME_AMT_status,
		LABEL_GEN_NAME_TPM_status,
#ifdef WITH_TPM
		LABEL_TAB_tpm,
		LABEL_FRAME_tpm,
#endif
		LABEL_TAB_amt,
		LABEL_FRAME_amt,
		LABEL_AMT_explain,
		MESTATUS_LABEL_COUNT
	};

	class NetIfacesColumns : public Gtk::TreeModel::ColumnRecord
	{
	public:
		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> >  m_icon;
		Gtk::TreeModelColumn<Glib::ustring>                m_name;

		NetIfacesColumns() { add(m_icon); add(m_name); }
	};

	class EventsColumns : public Gtk::TreeModel::ColumnRecord
	{
	public:
		Gtk::TreeModelColumn<Glib::ustring>                m_application;
		Gtk::TreeModelColumn<Glib::ustring>                m_type;
		Gtk::TreeModelColumn<Glib::ustring>                m_time;
		Gtk::TreeModelColumn<Glib::ustring>                m_comment;

		EventsColumns() { add(m_application); add(m_type); add(m_time); add(m_comment); }
	};

	/*Signal handlers*/
	//delete event received
	virtual bool on_delete_event(GdkEventAny *event);
	//Main window Close button was clicked
	virtual void OnCloseButton();
	// "Show On Every State" checkbox was toggled
	virtual void OnShowEveryChk();
	// "Show user notification popup" checkbox was toggled
	virtual void OnPopupChk();
	//Main window Help button was clicked
	virtual void OnHelpButton();
	// "Help" window Close button was clicked
	virtual void OnHelpCloseButton();
	// "Browser not found" message dialog's Ok button was clicked
	virtual void OnBrowserNotFoundOkButton();
	// "AMT Extended System Details" button was clicked
	virtual void OnAMTExtendedButton();
	// "AMT Extended System Details" dialog's Close button was clicked
	virtual void OnAMTExtendedCloseButton();
	// "AMT Network Information" button was clicked
	virtual void OnAMTNetworkButton();
	// "AMT Network Information" dialog's Close button was clicked
	virtual void OnAMTNetworkCloseButton();
	// Changed selection in AMT Network Interfaces list
	virtual void OnNetIfaceSelChanged();
	//Invoke browser with OEM url
	virtual bool OnLinkEButton(const GdkEventButton *);
	virtual void OnLinkButton();
	//Nofitication popup closed
	virtual void OnNotifyClosed(int response_id);

	//Invoke specified web browser with specified url.
	//Returns true on success, false otherwise.
	bool InvokeBrowser(const std::string &browser, const std::string &url);

private:
	/* GUI widgets initialization subroutines */
	bool initMainWidgets(Glib::ustring &strError);
	bool initEventsList(Glib::ustring &strError);
	bool initExtendedInfoDlgWidgets(Glib::ustring &strError);
	bool initNetworkInfoDlgWidgets(Glib::ustring &strError);
	bool initNetworkInfoList(Glib::ustring &strError);
	bool initOtherDlgWidgets(Glib::ustring &strError);
	bool initBrowserNotFoundDlgWidgets(Glib::ustring &strError);
	bool initHelpDlgWidgets(Glib::ustring &strError);

	/* GUI widgets data update subroutines */
	void updateAMTData();
	void updateAMTMainData();
	void updateAMTExtendedData();
	void updateAMTNetworkData();
#ifdef WITH_TPM
	void updateTPMData();
#endif
	void loadEventHistory();
	void updateEventHistory();

	void closeMsgDialog();

	//Data members:
	//Pointer to GUI commands that do the actual work
	IMEGuiCommands *m_pMEGuiCommands;
	Glib::ustring m_url;

	Glib::RefPtr<Gnome::Glade::Xml> m_refGlade; //Glade object - contains resources

	Gtk::LinkButton *m_pLinkButton; //OEM url button
	Gtk::HButtonBox *m_pLinkButtonBox;

	Gtk::Window *m_pBrowserNotFoundDlg; //Browser not found dialog
	Gtk::Window *m_pAMTExtendedDlg; //AMT Extended System Details dialog
	Gtk::Window *m_pAMTNetworkDlg; //AMT Network Information dialog
	Gtk::Window *m_pHelpDlg; //Help dialog

	/* Changeable Labels */
	Gtk::Label *m_pLabels[MESTATUS_LABEL_COUNT];

	//Event list
	EventsColumns m_eventsColumns;
	Gtk::TreeView *m_pEventList;
	Glib::RefPtr<Gtk::ListStore> m_refEventsStore;
	Semaphore m_eventsStoreSem;
	unsigned int m_eventsMaxSize;

	//Network Interface list
	NetIfacesColumns m_netIfacesColumns;
	Gtk::TreeView *m_pNetIfacesList;
	Glib::RefPtr<Gtk::ListStore> m_refNetIfacesStore;
	Glib::RefPtr<Gtk::TreeSelection> m_refNetIfacesSelection;

	Gtk::Notebook *m_pTabs; //Tabs container

	Gtk::CheckButton *m_pNotifyChk; //The activate notification check box
	Gtk::CheckButton *m_pPopupChk; //The popup notification check box

	MEStatusData m_data; //AMT/TPM status data

#ifdef WITH_WIRELESS_AT
	bool m_bAMTNetSelWireless;
#endif

	Gtk::MessageDialog *m_dialog;
	Semaphore m_msgDialogSem;

	Gtk::Widget *m_pageAmt;
#ifdef WITH_TPM
	Gtk::Widget *m_pageTpm;
#endif
};

#endif //_ME_STATUS_DLG_H
