/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

//******************************************************************************
//
// File:		MEStatusIcon.cpp
//
// Description:		Implementation for MEStatusIcon class
//                      The Icon of the MEStatus application.
//
//******************************************************************************

#ifdef HAVE_CONFIG_H 
#include "config.h" 
#endif
#include "iatshareddata.h"
#include "MEStatusIcon.h"
#include "IMEIconCommands.h"

#include <glibmm/i18n.h>
#include "MEStatusDataDefinitions.h"

#include "Lock.h"

#define MES_NOTIF_TIMEOUT 9000

//ctor
MEStatusIcon::MEStatusIcon()
{
	m_pMenuPopup = NULL;
	m_pMEIconCommands = NULL;
	m_canIconWarn = true;
	m_bWarned = false;
	m_sIconState = SICON_STATE_INITIAL;
	m_bIconVisible = false;
}

//dtor
MEStatusIcon::~MEStatusIcon()
{
	m_bIconVisible = false;
	if (m_statusIcon) {
		m_statusIcon->set_visible(false);
	}

	if (notify_is_initted()) {
		notify_uninit();
	}
}

/* Init: Initialize the nitification icon.
 *  pMEIconCommands is a pointer to the commands object, 
 *    which must already be initialized.
 *  strError is a reference to a string that will hold an error message
 *    in case initialization fails.
 **/
bool MEStatusIcon::Init(IMEIconCommands *pMEIconCommands, Glib::ustring &strError)
{
	//Reset strError
	strError = "";

	//Check parameters
	if (pMEIconCommands == NULL) {
		strError = _("pMEIconCommands is NULL");
		return false;
	}
	m_pMEIconCommands = pMEIconCommands;

	if (m_pMEIconCommands->GetModeApplication()) {
		return true;
	}

	if (!initStatusIcon(strError)) {
		return false;
	}
	if (!initPopupMenu(strError)) {
		return false;
	}

	if (!notify_is_initted() && !notify_init("MEStatusIcon")) {
		strError = _("notify_init failed");
		return false;
	}

	//All done
	return true;
}

bool MEStatusIcon::initStatusIcon(Glib::ustring &strError)
{
	//Load icons for status icon from files
	try
	{
		m_iconOn = Gdk::Pixbuf::create_from_file(MESTATUS_RESPATH "/ATChk-on.ico");
		m_iconOff = Gdk::Pixbuf::create_from_file(MESTATUS_RESPATH "/ATChk-off.ico");
		m_iconWarn = Gdk::Pixbuf::create_from_file(MESTATUS_RESPATH "/ATChk-warn.ico");
		m_iconUnknown = Gdk::Pixbuf::create_from_file(MESTATUS_RESPATH "/ATChk-unk.ico");
		m_iconAllOff = Gdk::Pixbuf::create_from_file(MESTATUS_RESPATH "/ATChk-aoff.ico");
	}
	catch (Glib::FileError &) {
		strError = _("Couldn't load icon file");
		return false;
	}

	//Initialize status icon
	m_statusIcon = Gtk::StatusIcon::create(m_iconUnknown);
	if (!m_statusIcon) {
		strError = _("Error creating status icon");
		return false;
	}

	if ((NULL != m_pMEIconCommands) && (m_pMEIconCommands->GetStatusIconAlways())) {
		m_bIconVisible = true;
	}
	m_statusIcon->set_visible(m_bIconVisible);

	m_statusIcon->set_tooltip(MESD_RES_ICON_TOOLTIP);
	/* I had to connect the signal handlers for the status icon the ugly way with GTK+, since
	 * gtkmm doesn't provide a way to do it.
	 **/
	g_signal_connect(m_statusIcon->gobj(), "popup-menu",
			 GTK_SIGNAL_FUNC(OnStatusIconPopupMenu), this);
	g_signal_connect(m_statusIcon->gobj(), "activate",
			 GTK_SIGNAL_FUNC(OnStatusIconClicked), this);

	return true;
}

bool MEStatusIcon::initPopupMenu(Glib::ustring &strError)
{
	//Create actions for popup menu
	m_refActionGroup = Gtk::ActionGroup::create();
	if (!m_refActionGroup) {
		strError = _("Error creating action group");
		return false;
	}
	m_refActionGroup->add(Gtk::Action::create("Open", MESD_RES_POPUP_MENU_ITEM_OPEN),
			sigc::mem_fun(*this, &MEStatusIcon::OnMenuShowStatus));
	m_refActionGroup->add(Gtk::Action::create("Exit", MESD_RES_POPUP_MENU_ITEM_EXIT),
			sigc::mem_fun(*this, &MEStatusIcon::OnMenuExit));

	//Create UI Manager and connect it to action group
	m_refUIManager = Gtk::UIManager::create();
	if (!m_refUIManager) {
		strError = _("Error creating action group UI manager");
		return false;
	}
	m_refUIManager->insert_action_group(m_refActionGroup);

	//Layout the actions in menu:
	Glib::ustring ui_info =
		"<ui>"
		"  <popup name='PopupMenu'>"
		"    <menuitem action='Open'/>"
		"    <menuitem action='Exit'/>"
		"  </popup>"
		"</ui>";
	try 
	{
		m_refUIManager->add_ui_from_string(ui_info);
	}
	catch(const Glib::Error &ex) {
		strError = _("Building menus failed: ") + ex.what();
		return false;
	}

	//Initialize popup menu
	m_pMenuPopup = dynamic_cast<Gtk::Menu *>(m_refUIManager->get_widget("/PopupMenu"));
	if (NULL == m_pMenuPopup) {
		strError = _("Menu not found");
		return false;
	}

	return true;
}


//Notification of state change from IMEIconNotifications
void MEStatusIcon::SetState(int sist)
{
	SIconState oldSIconState = m_sIconState;

	switch (sist)
	{
	case SICON_ST_UNK:
		m_sIconState = SICON_STATE_UNKNOWN;
		break;
	case SICON_ST_ON:
		m_sIconState = SICON_STATE_ON;
		break;
	case SICON_ST_OFF_SOME:
		m_sIconState = SICON_STATE_OFF_SOME;
		break;
	case SICON_ST_OFF:
	default:
		m_sIconState = SICON_STATE_OFF;
	}

	if (oldSIconState != m_sIconState) {
		updateStatusIcon();
	}
}


//The "open" option from the popup menu was selected
void MEStatusIcon::OnMenuShowStatus()
{
	if (NULL != m_pMEIconCommands) {
		m_pMEIconCommands->PresentStatusDialog();
	}
}

//The "exit" option from the popup menu was selected
void MEStatusIcon::OnMenuExit()
{
	Gtk::Main::quit();
}

//The status icon was clicked
void MEStatusIcon::OnStatusIconClicked(GtkStatusIcon *icon, gpointer func_data)
{
	MEStatusIcon *pIcon = (MEStatusIcon *)func_data;

	if (NULL != pIcon) {
		if (pIcon->iconClickActionConnection.connected()) {
			pIcon->iconClickActionConnection.disconnect();
			if (pIcon->m_pMEIconCommands) {
				pIcon->m_pMEIconCommands->PresentStatusDialog();
			}
		} else {
			pIcon->iconClickActionConnection =
			    Glib::signal_timeout().connect(
				sigc::mem_fun(pIcon, &MEStatusIcon::IconLClicked), 500);
		}
	}
}

//The status icon was single-clicked
bool MEStatusIcon::IconLClicked()
{
	if (NULL != m_pMenuPopup) {
		m_pMenuPopup->popup(0, gtk_get_current_event_time());
	}
	return false;
}

//The status icon was right-clicked
void MEStatusIcon::OnStatusIconPopupMenu(GtkStatusIcon *status_icon,
					guint          button,
					guint          activate_time,
					gpointer       user_data)
{
	MEStatusIcon *pIcon = (MEStatusIcon *)user_data;

	if ((NULL != pIcon) && (NULL != pIcon->m_pMenuPopup)) {
		pIcon->m_pMenuPopup->popup(button, activate_time);
	}
}

//Notification bubble show callback C-proxy
static void NotificationShowSlotCb(NotifyNotification *notification,
					gchar *action, gpointer user_data)
{
	MEStatusIcon::NotificationShowCb(notification, user_data);
}

//Notification bubble default no action callback C-proxy
static void NotificationNASlotCb(NotifyNotification *notification,
					gchar *action, gpointer user_data)
{
}

//Notification bubble show callback
void MEStatusIcon::NotificationShowCb(NotifyNotification *notification, gpointer user_data)
{
	MEStatusIcon *pIcon = (MEStatusIcon *)user_data;

	if ((NULL != pIcon) && (NULL != pIcon->m_pMEIconCommands)) {
		pIcon->m_pMEIconCommands->PresentStatusDialog();
	}
}

//Notification bubble close callback
void MEStatusIcon::NotificationClosedCb(NotifyNotification *notification, gpointer user_data)
{
	MEStatusIcon *pIcon = (MEStatusIcon *)user_data;

	if ((NULL != pIcon) && (NULL != pIcon->m_pMEIconCommands)) {
		pIcon->m_pMEIconCommands->NotificationClosed();
	}
	g_object_unref(G_OBJECT(notification));
}

//Show notification bubble
bool MEStatusIcon::Notify(const gchar *title, const gchar *body, bool warn, int show)
{
	NotifyNotification *notification;
	bool bAddShow;
	
	{
		Lock l(m_sIconSem);

		bAddShow = ((m_canIconWarn) && (0 != (show & SICON_NOTIFY_SHOW_SHOW)));

		if ((warn) && (m_canIconWarn)) {
			bool oldWarned = m_bWarned;
			m_bWarned = true;
			if (!oldWarned) {
				updateStatusIcon();
			}
		}
	}

	if (!notify_is_initted()) {
		return false;
	}
	
	if (!m_bIconVisible) {
		return false;
	}

	if (0 != (show & SICON_NOTIFY_SHOW_NOBUBBLE)) {
		return false;
	}

	notification = notify_notification_new(title, body, NULL);
	if (!notification) {
		return false;
	}

	g_signal_connect(notification, "closed",
			 GTK_SIGNAL_FUNC(NotificationClosedCb), this);
	notify_notification_set_timeout(notification, MES_NOTIF_TIMEOUT);
	notify_notification_set_urgency(notification,
		(((show & SICON_NOTIFY_SHOW_CRITICAL) != 0)
			? NOTIFY_URGENCY_CRITICAL
			: (((show & SICON_NOTIFY_SHOW_LOW) != 0)
				? NOTIFY_URGENCY_LOW
				: NOTIFY_URGENCY_NORMAL)
		));
	if (bAddShow) {
		notify_notification_add_action(notification,
					       "show",
					       MESD_RES_BUBBLE_BUTTON_SHOW,
					       NotificationShowSlotCb,
					       this, NULL);
		notify_notification_add_action(notification,
					       "default",
					       MESD_RES_BUBBLE_BUTTON_SHOW,
					       NotificationShowSlotCb,
					       this, NULL);
	} else {
		notify_notification_add_action(notification,
					       "default",
					       MESD_RES_BUBBLE_BUTTON_CLOSE,
					       NotificationNASlotCb,
					       this, NULL);
	}
	if (!notify_notification_show(notification, NULL)) {
		if (bAddShow) {
			notify_notification_clear_actions(notification);
		}
		g_object_unref(G_OBJECT(notification));
		return false;
	}

	return true;
}

//set permition for change icon to warn state
void MEStatusIcon::SetNotifyWarnPermition(bool permit)
{
	Lock l(m_sIconSem);

	m_canIconWarn = permit;

	if (!permit) {
		bool oldWarned = m_bWarned;
		m_bWarned = false;
		if (oldWarned) {
			updateStatusIcon();
		}
	}
}

//update status icon image
void MEStatusIcon::updateStatusIcon()
{
	if (!m_statusIcon) {
		return;
	}

	if ((!m_bIconVisible) &&
	    ((SICON_STATE_ON == m_sIconState) ||
	     (SICON_STATE_OFF_SOME == m_sIconState))) {
		m_bIconVisible = true;
		m_statusIcon->set_visible(true);
	}

	if (m_bWarned) {
		m_statusIcon->set(m_iconWarn);
	} else {
		switch (m_sIconState)
		{
		case SICON_STATE_INITIAL:
		case SICON_STATE_UNKNOWN:
			m_statusIcon->set(m_iconUnknown);
			break;
		case SICON_STATE_OFF:
			m_statusIcon->set(m_iconAllOff);
			break;
		case SICON_STATE_OFF_SOME:
			m_statusIcon->set(m_iconOff);
			break;
		case SICON_STATE_ON:
			m_statusIcon->set(m_iconOn);
			break;
		}
	}
}
