/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H 
#include "config.h" 
#endif
#include <gtkmm.h>
#include "MEStatusLogic.h"

#include <glibmm/i18n.h>
#include "MEStatusDataDefinitions.h"

#include "Lock.h"
#include <string>
#include <sstream>
#include <arpa/inet.h>

#ifdef WITH_TPM
#define TPM_VALID_VENDOR TPM_VENDOR_INTC
#endif

//******************************************************************************
//
// Class:	    MEStatusLogic
//
// Description:	This class handles all the logic of the MEStatus application,
//              and it communicates with the outside world through 4 interfaces:
//               IMEGuiCommands, IMEIconCommands,
//                which it inherits from, and 
//               IMEGuiNotifications, IMEIconNotifications, which it points to.
//              The idea is for the user of this class to treat it 
//                as an IME[Gui/Icon]Commands object. 
//              The user would pass it a pointer to IME[Gui/Icon]Notifications
//              objects which would receive notifications that 
//              are caused by activating commands from IME[Gui/Icon]Commands.
//
//******************************************************************************

MEStatusLogic::MEStatusLogic()
{
	m_notifOpen = false;
#ifdef MES_NOTIFY_MANY_EVENTS
	m_eventsNotifiable = false;
#endif
	m_appPresent = false;
	m_showAlways = MES_SHOW_ALWAYS_NO;
	m_modeApp = false;
	m_State = STATE_INITIAL_UNKNOWN;
	m_unsConnState = STATE_INITIAL_DISCONNECTED;
#ifdef WITH_TPM
	m_TPMState = STATE_INITIAL_UNKNOWN;
	m_nSetTPMCount = 0;
#endif
	m_sku = 0;
	m_nSetStatusCount = 0;
	m_nUnsConnCount = 0;

	m_pConfig = NULL;
	m_pMEAMTDataReader = NULL;
	m_pMEGuiNotifications = NULL;
	m_pMEIconNotifications = NULL;

	m_started = false;
}

MEStatusLogic::~MEStatusLogic()
{
	m_MEUNSDataReader.CloseReader();

	m_pConfig = NULL;
	m_pMEAMTDataReader = NULL;
	m_pMEGuiNotifications = NULL;
	m_pMEIconNotifications = NULL;
}

void MEStatusLogic::SetModeApplication(bool modeapp)
{
	m_modeApp = modeapp;
}

bool MEStatusLogic::Init(MEStatusConfig *pConfig,
				 IMEAMTStatus *pMEAMTDataReader,
				 IMEGuiNotifications *pMEGuiNotifications,
				 IMEIconNotifications *pMEIconNotifications)
{
	m_State = STATE_INITIAL_UNKNOWN;
	m_unsConnState = STATE_INITIAL_DISCONNECTED;
#ifdef WITH_TPM
	m_TPMState = STATE_INITIAL_UNKNOWN;
	m_nSetTPMCount = 0;
#endif
	m_sku = 0;
	m_nSetStatusCount = 0;
	m_nUnsConnCount = 0;

	if (NULL == pConfig) {
	        return false;
	}
	m_pConfig = pConfig;
	if (NULL == pMEAMTDataReader) {
	        return false;
	}
	m_pMEAMTDataReader = pMEAMTDataReader;
	if (NULL == pMEGuiNotifications) {
	        return false;
	}
	m_pMEGuiNotifications = pMEGuiNotifications;
	if (NULL == pMEIconNotifications) {
	        return false;
	}
	m_pMEIconNotifications = pMEIconNotifications;

	if (!(m_pMEAMTDataReader->InitReader())) {
		return false;
	}

	if (!(m_MEUNSDataReader.InitReader())) {
		return false;
	}

	return true;
}

void MEStatusLogic::StartLogic()
{
	m_State = STATE_INITIAL_UNKNOWN;
	m_unsConnState = STATE_INITIAL_DISCONNECTED;
#ifdef WITH_TPM
	m_TPMState = STATE_INITIAL_UNKNOWN;
	m_nSetTPMCount = 0;
#endif
	m_nSetStatusCount = 0;
	m_nUnsConnCount = 0;

	//m_AMTStatusColl.Init(false); //we do not generate events - so no need to synchronize with UNS
	AMTStatusData amtData;
	memset(&amtData, 0, sizeof(amtData));
	m_AMTStatusColl.SetBaseData(amtData);
#ifdef WITH_TPM
	//m_TPMStatusColl.Init(false);
	TPMStatusData tpmData;
	memset(&tpmData, 0, sizeof(tpmData));
	m_TPMStatusColl.SetBaseData(tpmData);
#endif

	if (m_pMEGuiNotifications != NULL) {
		m_pMEGuiNotifications->LoadInitialEventHistoryData();
	}

	{
		Lock l(m_eventsSem);
		m_lEvents.clear();
#ifdef MES_NOTIFY_MANY_EVENTS
		m_eventsNotifiable = false;
#endif
	}

	UpdateData();

	if ((m_pMEGuiNotifications != NULL) && (m_modeApp)) {
		m_pMEGuiNotifications->PresentStatusDialog();
	}

	m_started = true;
}

void MEStatusLogic::UpdateData()
{
	bool checkForQuit = false;
	bool haveConn;
	bool haveConnChanged;
	bool needNotifyState;
	AMTStatusData amtData;
#ifdef WITH_TPM
	TPMStatusData tpmData;
#endif

	memset(&amtData, 0, sizeof(amtData));
#ifdef WITH_TPM
	memset(&tpmData, 0, sizeof(tpmData));
	haveConn = fetchStatusData(amtData, tpmData);
#else
	haveConn = fetchStatusData(amtData);
#endif

	if (STATE_INITIAL_UNKNOWN == m_State) {
		checkForQuit = true;
	}

	haveConnChanged = setConnState(haveConn);
	needNotifyState = setState(amtData.amtState);
#ifdef WITH_TPM
	if ((m_sku & IPCiATStatus::SKU_TPM) != 0) {
		setTPMState(tpmData);
	}
#endif

	if (checkForQuit) {
		checkQuitLogic();
	}

	setStateToIcon();

	notificationOpen(haveConnChanged, needNotifyState);

	//Save AMT enabled/disabled state to configuration for next time we run.
	if (m_pConfig != NULL) {
		m_pConfig->SetAMTLastEnabled((AMTSTATE_POST == amtData.amtState));
	}
}

void MEStatusLogic::checkQuitLogic()
{
	if (STATE_INITIAL_UNKNOWN == m_State) {
		return;
	}

	if (((m_sku & IPCiATStatus::SKU_AMT) == 0)
#ifdef WITH_TPM
	    && ((m_sku & IPCiATStatus::SKU_TPM) == 0)
#endif
	    ) {
		if (m_started) {
			Gtk::Main::quit();
		}
		if (m_modeApp) {
			Gtk::MessageDialog(MESD_RES_ERROR_NO_FEAT_ON_SKU, false, Gtk::MESSAGE_ERROR).run();
		}
		exit(0);
	}

	if (m_modeApp) {
		return;
	}

	if ((m_sku & IPCiATStatus::SKU_AMT) != 0) {
		if (STATE_ENABLED == m_State) {
			//AMT is enabled - no quit
			return;
		}
	}
#ifdef WITH_TPM
	if ((m_sku & IPCiATStatus::SKU_TPM) != 0) {
		if (STATE_INITIAL_UNKNOWN == m_TPMState) {
			//delay decision
			return;
		}
		if (STATE_ENABLED == m_TPMState) {
			//iTPM is enabled - no quit
			return;
		}
	}
#endif

	switch (m_showAlways) {
	case MES_SHOW_ALWAYS_YES:
		break;

	case MES_SHOW_ALWAYS_USER:
		if ((m_pConfig != NULL) && (!m_pConfig->StatusIconAlways())) {
			if (m_started) {
				Gtk::Main::quit();
			}
			exit(0);
		}
		break;

	default:
		if (m_started) {
			Gtk::Main::quit();
		}
		exit(0);
	}
}

void MEStatusLogic::notificationOpen(bool haveConnChanged, bool needNotifyState)
{
	Lock nl(m_notifSem);

#ifdef MES_NOTIFY_UNS_CONNECTION_CHANGE
	if (haveConnChanged) {
		if (m_unsConnState == STATE_CONNECTED) {
			notify(MESD_RES_NOTIF_ANY,
			       MESD_RES_NOTIF_AMT_UNS_RECONNECT, AMT_NOTIFY_INFO);
		} else {
			notify(MESD_RES_NOTIF_ANY,
			       MESD_RES_NOTIF_AMT_NO_UNS, AMT_NOTIFY_ERROR);
		}
	} else
#endif
	{
		bool hadEvents = false;

		while (notifyEvents()) {
			hadEvents = true;
			if (m_notifOpen) {
				return;
			}
		}
	}
}


#define MESL_EVENT_TYPE_AMT   (1 << 0)
#ifdef WITH_TPM
#define MESL_EVENT_TYPE_TPM   (1 << 1)
#endif
#define MESL_EVENT_TYPE_OTHER (1 << 5)


bool MEStatusLogic::fetchStatusData(AMTStatusData &amtData
#ifdef WITH_TPM
, TPMStatusData &tpmData
#endif
)
{
	IPCiATStatus::Enablement enablement;
	bool haveConn  = false;
	bool haveAdata = false;
	bool haveAdiff = false;
#ifdef WITH_TPM
	bool haveTdata = false;
	bool haveTdiff = false;
#endif
	std::list<ATEvents::EventNode> newEvents;
	unsigned int sku = 0;

	haveConn = m_MEUNSDataReader.GetEnablement(enablement);
	if (!haveConn) {
		if ((m_pMEAMTDataReader != NULL) &&
		    (m_pMEAMTDataReader->GetAMTStatusData(amtData))) {
			if (AMTSTATE_MEI_DISABLED != amtData.amtState) {
				haveAdata = true;
				sku |= IPCiATStatus::SKU_AMT | IPCiATStatus::SKU_AMTFUND;
			}
		}
#ifdef WITH_TPM
		if (m_TPMDataReader.GetTPMStatusData(tpmData, TPM_VALID_VENDOR)) {
			haveTdata = true;
			sku |= IPCiATStatus::SKU_TPM;
		}
#endif
	} else {
		sku = m_MEUNSDataReader.GetSku();
		if (m_MEUNSDataReader.GetEvents(newEvents)) {
			int newEventTypes = eventTypes(newEvents);
			if ((sku & IPCiATStatus::SKU_AMT) != 0) {
				if ((newEventTypes & MESL_EVENT_TYPE_AMT) != 0) {
					haveAdiff = true;
				}
			}
#ifdef WITH_TPM
			if ((sku & IPCiATStatus::SKU_TPM) != 0) {
				if ((newEventTypes & MESL_EVENT_TYPE_TPM) != 0) {
					haveTdiff = true;
				}
			}
#endif
		}
		if ((sku & IPCiATStatus::SKU_AMT) != 0) {
			if (m_MEUNSDataReader.GetAMTStatusData(amtData)) {
				haveAdata = true;
			}
		}
#ifdef WITH_TPM
		if ((sku & IPCiATStatus::SKU_TPM) != 0) {
			if (m_MEUNSDataReader.GetTPMStatusData(tpmData, TPM_VALID_VENDOR)) {
				haveTdata = true;
			}
		}
#endif
	}

	if ((haveAdata) && (!haveAdiff)) {
		if (0 < m_AMTStatusColl.Collate(amtData, NULL)) {
			haveAdiff = true;
		}
	}
#ifdef WITH_TPM
	if ((haveTdata) && (!haveTdiff)) {
		if (0 < m_TPMStatusColl.Collate(tpmData, NULL)) {
			haveTdiff = true;
		}
	}
#endif

	if (sku != m_sku) {
		m_sku = sku;
		setSku(m_sku);
	}

	if (0 < newEvents.size()) {
		addNewEvents(newEvents);

		newEvents.clear();

		setEventHistory();
	}
	
	if (haveAdiff) {
		updateAMTStatusData(haveAdata, amtData);
	}

#ifdef WITH_TPM
	if (haveTdiff) {
		updateTPMStatusData(haveTdata, tpmData);
	}
#endif

	return haveConn;
}

void MEStatusLogic::addNewEvents(std::list<ATEvents::EventNode> &newEvents)
{
	Lock l(m_eventsSem);

	std::list<ATEvents::EventNode>::const_iterator nit = newEvents.begin();
	std::list<ATEvents::EventNode>::const_iterator nend = newEvents.end();
	for (; nit != nend; nit++) {
#ifdef MES_NOTIFY_MANY_EVENTS
		if (isEventNotifiable(*nit)) {
			m_eventsNotifiable = true;
		}
#endif
		m_lEvents.push_back((*nit));
	}
}

bool MEStatusLogic::notifyEvents()
{
	Glib::ustring title;
	Glib::ustring body;
	int show = AMT_NOTIFY_EVENT;
	bool notif = false;

	{
		Lock l(m_eventsSem);

		if (!(m_lEvents.size() > 0)) {
			return false;
		}

		if (m_appPresent) {
			//clear events if application is displayed
			m_lEvents.clear();
#ifdef MES_NOTIFY_MANY_EVENTS
			m_eventsNotifiable = false;
#endif
			return true;
		}

#ifdef MES_NOTIFY_MANY_EVENTS
		if (m_lEvents.size() > 1) {
			if (!m_eventsNotifiable) {
				show |= AMT_NOTIFY_NOBUBBLE;
			}
			std::stringstream sst;
			sst << m_lEvents.size();
			m_lEvents.clear();
			m_eventsNotifiable = false;
			title = MESD_RES_NOTIF_ANY;
			body = Glib::ustring(sst.str()) + MESD_RES_NOTIF_MANY_EVENTS;
			notif = true;
		} else 
#endif
		{
			std::list<ATEvents::EventNode>::iterator eit = m_lEvents.begin();
			if (eit != m_lEvents.end()) {
				eventToNotify((*eit), title, body);
				if (!isEventNotifiable(*eit)) {
					show |= AMT_NOTIFY_NOBUBBLE;
				}
				m_lEvents.erase(eit);
				notif = true;
			}
		}
	}

	if (notif) {
		title = MESD_RES_NOTIF_ANY_DLG;
		body = MESD_RES_NOTIF_ANY;

		notify(title, body, show);
	}

	return notif;
}

bool MEStatusLogic::setConnState(bool haveConn)
{
	bool connChanged = false;

	switch (m_unsConnState)
	{
	case STATE_INITIAL_DISCONNECTED:
		if (haveConn) {
			m_unsConnState = STATE_CONNECTED;
		} else if ((m_pConfig != NULL) &&
			    (++m_nUnsConnCount >= m_pConfig->AMTStatusInitMaxRetries())) {
			m_unsConnState = STATE_DISCONNECTED;
			connChanged = true;
		}
		break;

	case STATE_CONNECTED:
		if (!haveConn) {
			m_unsConnState = STATE_DISCONNECTED;
			connChanged = true;
		}
		break;

	case STATE_DISCONNECTED:
		if (haveConn) {
			m_unsConnState = STATE_CONNECTED;
			connChanged = true;
		}
		break;
	}

	return connChanged;
}

void MEStatusLogic::updateAMTStatusData(bool haveAmtData, AMTStatusData &amtData)
{
	if ((!haveAmtData) && (m_pMEAMTDataReader != NULL)) {
		m_pMEAMTDataReader->GetAMTStatusData(amtData);
	}
	setAMTStatusData(amtData);
}

void MEStatusLogic::setAMTStatusData(AMTStatusData &amtData)
{
	m_AMTStatusColl.SetBaseData(amtData);

	if (m_pMEGuiNotifications != NULL) {
		m_pMEGuiNotifications->OnAMTStatusData(amtData);
	}
}

#ifdef WITH_TPM
void MEStatusLogic::updateTPMStatusData(bool haveTpmData, TPMStatusData &tpmData)
{
	if (!haveTpmData) {
		m_TPMDataReader.GetTPMStatusData(tpmData, TPM_VALID_VENDOR);
	}
	setTPMStatusData(tpmData);
}

void MEStatusLogic::setTPMStatusData(TPMStatusData &tpmData)
{
	m_TPMStatusColl.SetBaseData(tpmData);

	if (m_pMEGuiNotifications != NULL) {
		m_pMEGuiNotifications->OnTPMStatusData(tpmData);
	}
}
#endif

void MEStatusLogic::setEventHistory()
{
	if (m_pMEGuiNotifications != NULL) {
		m_pMEGuiNotifications->OnEventHistory();
	}
}

void MEStatusLogic::setSku(unsigned int sku)
{
	unsigned int dlgSku = 0;
	if ((sku & IPCiATStatus::SKU_AMT) != 0) {
		dlgSku |= MESGUI_SKU_AMT;
		if ((sku & IPCiATStatus::SKU_AMTFUND) != 0) {
			dlgSku |= MESGUI_SKU_AMTFUND;
		}
	}
#ifdef WITH_TPM
	if ((sku & IPCiATStatus::SKU_TPM) != 0) {
		dlgSku |= MESGUI_SKU_TPM;
	}
#endif
	if (m_pMEGuiNotifications != NULL) {
		m_pMEGuiNotifications->OnSku(dlgSku);
	}
}

void MEStatusLogic::setStateToIcon()
{
	if (m_modeApp) {
		return;
	}

	if (m_pMEIconNotifications == NULL) {
		return;
	}

	int sist;

	if ((m_sku & IPCiATStatus::SKU_AMT) != 0) {
		switch(m_State)
		{
		case STATE_ENABLED:
			sist = SICON_ST_ON;
#ifdef WITH_TPM
			if ((m_sku & IPCiATStatus::SKU_TPM) != 0) {
				if (STATE_ENABLED != m_TPMState) {
					sist = SICON_ST_OFF_SOME;
				}
			}
#endif
			break;

		case STATE_INITIAL_UNKNOWN:
			sist = SICON_ST_UNK;
			break;

		default:
			sist = SICON_ST_OFF;
#ifdef WITH_TPM
			if ((m_sku & IPCiATStatus::SKU_TPM) != 0) {
				if (STATE_ENABLED == m_TPMState) {
					sist = SICON_ST_OFF_SOME;
				}
			}
#endif
			break;
		}
	} else {
		sist = SICON_ST_OFF;
#ifdef WITH_TPM
		if ((m_sku & IPCiATStatus::SKU_TPM) != 0) {
			if (STATE_ENABLED == m_TPMState) {
				sist = SICON_ST_ON;
			} else if (STATE_INITIAL_UNKNOWN == m_TPMState) {
				sist = SICON_ST_UNK;
			}
		}
#endif
	}

	m_pMEIconNotifications->SetState(sist);
}

#ifdef WITH_TPM
bool MEStatusLogic::setTPMState(TPMStatusData &tpmData)
{
	bool needNotify = false;

	switch (m_TPMState)
	{
	case STATE_INITIAL_UNKNOWN:
		needNotify = setTPMStateOnInitialUnknownState(tpmData);
		break;

	case STATE_DISABLED:
	case STATE_UNKNOWN:
	case STATE_ENABLED:
		m_TPMState = getStateByTPMData(tpmData);
		break;
	}

	return needNotify;
}

bool MEStatusLogic::setTPMStateOnInitialUnknownState(TPMStatusData &tpmData)
{
	bool needNotify = false;
	int tOState = m_TPMStatusColl.TPMOperationalState(tpmData);

	switch (tOState)
	{
	case TPM_COLL_OPER_UNKNOWN:
		m_TPMState = STATE_INITIAL_UNKNOWN;
		if ((m_pConfig != NULL) &&
		    (++m_nSetTPMCount >= m_pConfig->AMTStatusInitMaxRetries())) {
			m_TPMState = STATE_UNKNOWN; 
		}
		break;

	case TPM_COLL_OPER_OPERATIONAL:
		m_TPMState = STATE_ENABLED;
		break;

	default:
		m_TPMState = STATE_DISABLED;
	}

	return needNotify;
}

MEStatusLogic::State MEStatusLogic::getStateByTPMData(TPMStatusData &tpmData) const
{
	int tOState = m_TPMStatusColl.TPMOperationalState(tpmData);

	switch (tOState)
	{
	case TPM_COLL_OPER_OPERATIONAL:
		return STATE_ENABLED;
		break;

	case TPM_COLL_OPER_UNKNOWN:
		return STATE_UNKNOWN;
		break;

	default:
		return STATE_DISABLED;
	}
}
#endif

bool MEStatusLogic::setState(unsigned char amtState)
{
	bool needNotify = false;

	switch (m_State)
	{
	case STATE_INITIAL_UNKNOWN:
		needNotify = setStateOnInitialUnknownState(amtState);
		break;

	case STATE_DISABLED:
	case STATE_UNKNOWN:
		if (AMTSTATE_POST == amtState) {
			needNotify = true;
		}
	case STATE_ENABLED:
		m_State = getStateByAMTState(amtState);
		break;
	}

	return needNotify;
}


bool MEStatusLogic::setStateOnInitialUnknownState(unsigned char amtState)
{
	bool needNotify = false;

	switch (amtState)
	{
	case AMTSTATE_POST:
		if ((m_pConfig != NULL) && (!m_pConfig->AMTLastEnabled())) {
			/* If last time we ran, AMT was not enabled, we show the popup. */
			needNotify = true;
		}
		m_State = STATE_ENABLED;
		break;

	case AMTSTATE_PRE:
	case AMTSTATE_IN:
	case AMTSTATE_DISABLED:
	case AMTSTATE_MEI_DISABLED:
		m_State = STATE_DISABLED;
		break;

	case AMTSTATE_NOT_DETECTED:
	default:
		m_State = STATE_INITIAL_UNKNOWN;
		if ((m_pConfig != NULL) &&
		    (++m_nSetStatusCount >= m_pConfig->AMTStatusInitMaxRetries())) {
			/*We were set to UNKNOWN status too many times - we already notified the 
			  GUI that the status is unknown, and now we tell the GUI to popup with 
			  this message.*/
			needNotify = true;
			m_State = STATE_UNKNOWN; 
		}
	}

	return needNotify;
}

MEStatusLogic::State MEStatusLogic::getStateByAMTState(unsigned char amtState) const
{
	switch (amtState)
	{
	case AMTSTATE_POST:
		return STATE_ENABLED;
		break;

	case AMTSTATE_PRE:
	case AMTSTATE_IN:
	case AMTSTATE_DISABLED:
	case AMTSTATE_MEI_DISABLED:
		return STATE_DISABLED;
		break;

	case AMTSTATE_NOT_DETECTED:
	default:
		return STATE_UNKNOWN;
	}
}



void MEStatusLogic::SetShowAlways(int rule)
{
	m_showAlways = rule;
}

void MEStatusLogic::SetStatusIconAlways(bool bAlwaysIcon)
{
	if (m_pConfig != NULL) {
		m_pConfig->SetStatusIconAlways(bAlwaysIcon);
	}
}

bool MEStatusLogic::GetStatusIconAlways() const
{
	if (NULL == m_pConfig) {
		return false;
	}
	return m_pConfig->StatusIconAlways();
}

void MEStatusLogic::SetPopupForCurrentUser(bool bPopupForCurrentUser)
{
	if (m_pConfig != NULL) {
		m_pConfig->SetPopupForCurrentUser(bPopupForCurrentUser);
	}
}

bool MEStatusLogic::GetPopupForCurrentUser() const
{
	if (NULL == m_pConfig) {
		return true;
	}
	return m_pConfig->PopupForCurrentUser();
}

std::string MEStatusLogic::GetOEMUrl() const
{
	if (NULL == m_pConfig) {
		return "http://";
	}
	return m_pConfig->GetOemUrl();
}

std::string MEStatusLogic::GetDefaultOEMUrl() const
{
	if (NULL == m_pConfig) {
		return "http://";
	}
	return m_pConfig->GetDefaultOemUrl();
}

unsigned int MEStatusLogic::GetEventHistoryMaxReadSize() const
{
	if (NULL == m_pConfig) {
		return 0x100;
	}
	unsigned long max = m_pConfig->EventHistoryMaxReadSize();
	if (max < UINT_MAX) {
		return (unsigned int)max;
	}
	return 0;
}

void MEStatusLogic::PresentStatusDialog()
{
	if (m_pMEGuiNotifications != NULL) {
		m_pMEGuiNotifications->PresentStatusDialog();
	}
}

void MEStatusLogic::SetAppPresence(bool present)
{
	if (m_pMEIconNotifications != NULL) {
		m_pMEIconNotifications->SetNotifyWarnPermition(!present);
	}

	{
		Lock l(m_eventsSem);

		m_appPresent = present;
		if (present) {
			m_lEvents.clear();
#ifdef MES_NOTIFY_MANY_EVENTS
			m_eventsNotifiable = false;
#endif
		}
	}
}

void MEStatusLogic::NotificationClosed()
{
	Lock nl(m_notifSem);

	m_notifOpen = false;

	while ((!m_notifOpen) && (m_queuedNotif.size() > 0)) {
		struct MEStatusLogic::Notification tmpNotif;
		std::list<struct MEStatusLogic::Notification>::iterator qit;

		qit = m_queuedNotif.begin();
		tmpNotif.title = qit->title;
		tmpNotif.body = qit->body;
		tmpNotif.show = qit->show;
		m_queuedNotif.erase(qit);
		notify(tmpNotif);
	}

	while (!m_notifOpen) {
		if (!notifyEvents()) {
			return;
		}
	}
}

bool MEStatusLogic::GetModeApplication() const
{
	return m_modeApp;
}

void MEStatusLogic::notify(const struct MEStatusLogic::Notification &qNotif)
{
	bool canPopup = true;

	if (0 != (AMT_NOTIFY_EVENT & qNotif.show)) {
		canPopup = false;
	}
	if (0 != (AMT_NOTIFY_NOBUBBLE & qNotif.show)) {
		canPopup = false;
	}
	if (!((0 != (AMT_NOTIFY_ERROR & qNotif.show)) || (m_modeApp))) {
		canPopup = false;
	}

	if ((!m_modeApp) && (m_pMEIconNotifications != NULL)) {
		int bushow = 0;
		if (0 != (AMT_NOTIFY_EVENT & qNotif.show)) {
			bushow |= SICON_NOTIFY_SHOW_SHOW;
			if ( (0 != (AMT_NOTIFY_NOBUBBLE & qNotif.show))
#ifdef MES_NOTIFY_MANY_EVENTS
			    || (!m_eventsNotifiable)
#endif
			    || (!GetPopupForCurrentUser()) )
			{
				bushow |= SICON_NOTIFY_SHOW_NOBUBBLE;
			}
		}
		if (0 != (AMT_NOTIFY_ERROR & qNotif.show)) {
			bushow |= SICON_NOTIFY_SHOW_CRITICAL;
		}
		if (m_pMEIconNotifications->Notify(qNotif.body.c_str(),
		                                   (qNotif.title + " :\n" + qNotif.body).c_str(),
		                                   ((AMT_NOTIFY_EVENT & qNotif.show) != 0),
		                                   bushow)) {
			canPopup = false;
			m_notifOpen = true;
		}
	}
	if ((canPopup) && (m_pMEGuiNotifications != NULL)) {
		if (m_pMEGuiNotifications->Notify(qNotif.title, qNotif.body,
		                                 (((AMT_NOTIFY_ERROR & qNotif.show) != 0) ?
		                                   MESGUI_NOTIFY_ERROR :
		                                   MESGUI_NOTIFY_INFO))) {
			m_notifOpen = true;
		}
	}
}


void MEStatusLogic::notify(const Glib::ustring &title, const Glib::ustring &body, int show)
{
	struct MEStatusLogic::Notification qNotif;

	qNotif.title = title;
	qNotif.body = body;
	qNotif.show = show;

	if (m_notifOpen) {
		m_queuedNotif.push_back(qNotif);
		return;
	}

	notify(qNotif);
}

bool MEStatusLogic::eventToNotify(const ATEvents::EventNode &curEvent,
				Glib::ustring &title,
				Glib::ustring &body) const
{
	unsigned int eventtype = 0;

	switch (curEvent.m_id)
	{
	case ATEvents::EventNode::AMT_ENABLED:
		eventtype = MESL_EVENT_TYPE_AMT;
		if ((m_sku & IPCiATStatus::SKU_AMTFUND) != 0) {
			body = MESD_RES_NOTIF_AMT_ENABLED_BASIC;
		} else {
			body = MESD_RES_NOTIF_AMT_ENABLED;
		}
		break;

	case ATEvents::EventNode::AMT_DISABLED:
		eventtype = MESL_EVENT_TYPE_AMT;
		if ((m_sku & IPCiATStatus::SKU_AMTFUND) != 0) {
			body = MESD_RES_NOTIF_AMT_DISABLED_BASIC;
		} else {
			body = MESD_RES_NOTIF_AMT_DISABLED;
		}
		break;

	case ATEvents::EventNode::PROVISIONED:
		eventtype = MESL_EVENT_TYPE_AMT;
		if ((m_sku & IPCiATStatus::SKU_AMTFUND) != 0) {
			body = MESD_RES_NOTIF_AMT_PROVISIONED_BASIC;
		} else {
			body = MESD_RES_NOTIF_AMT_PROVISIONED;
		}
		break;

	case ATEvents::EventNode::UNPROVISIONED:
		eventtype = MESL_EVENT_TYPE_AMT;
		if ((m_sku & IPCiATStatus::SKU_AMTFUND) != 0) {
			body = MESD_RES_NOTIF_AMT_UNPROVISIONED_BASIC;
		} else {
			body = MESD_RES_NOTIF_AMT_UNPROVISIONED;
		}
		break;

	case ATEvents::EventNode::SOL_STARTED:
		eventtype = MESL_EVENT_TYPE_AMT;
		body = MESD_RES_NOTIF_AMT_SOL_START;
		break;

	case ATEvents::EventNode::SOL_FINISHED:
		eventtype = MESL_EVENT_TYPE_AMT;
		body = MESD_RES_NOTIF_AMT_SOL_FINISH;
		break;

	case ATEvents::EventNode::IDER_STARTED:
		eventtype = MESL_EVENT_TYPE_AMT;
		body = MESD_RES_NOTIF_AMT_IDER_START;
		break;

	case ATEvents::EventNode::IDER_FINISHED:
		eventtype = MESL_EVENT_TYPE_AMT;
		body = MESD_RES_NOTIF_AMT_IDER_FINISH;
		break;

	case ATEvents::EventNode::SD_STARTED:
		eventtype = MESL_EVENT_TYPE_AMT;
		body = MESD_RES_NOTIF_AMT_SD_START;
		break;

	case ATEvents::EventNode::SD_FINISHED:
		eventtype = MESL_EVENT_TYPE_AMT;
		body = MESD_RES_NOTIF_AMT_SD_FINISH;
		break;

	case ATEvents::EventNode::REMOTE_REBOOT:
		eventtype = MESL_EVENT_TYPE_AMT;
		body = MESD_RES_NOTIF_AMT_REMOTE_REBOOT;
		break;

#ifdef WITH_TPM
	case ATEvents::EventNode::TPM_OPERATIONAL:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_OPERATIONAL;
		break;

	case ATEvents::EventNode::TPM_NOT_DETECTED:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_NOT_DETECTED;
		break;

	case ATEvents::EventNode::TPM_FAIL:
		eventtype = MESL_EVENT_TYPE_TPM;
		switch (curEvent.m_data1)
		{
		case ATEvents::EventNode::TPM_FAIL_ME_RESET:
			body = MESD_RES_NOTIF_TPM_FAIL_ME_RESET;
			break;

		case ATEvents::EventNode::TPM_FAIL_HW_FAILURE:
			body = MESD_RES_NOTIF_TPM_FAIL_HW_FAILURE;
			break;

		case ATEvents::EventNode::TPM_FAIL_FLASH_CORRUPT:
			body = MESD_RES_NOTIF_TPM_FAIL_FLASH_CORRUPT;
			break;

		case ATEvents::EventNode::TPM_FAIL_UNKNOWN:
		default:
			body = MESD_RES_NOTIF_TPM_FAIL_UNKNOWN;
		}
		break;

	case ATEvents::EventNode::TPM_ACTIVE:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_ACTIVE;
		break;

	case ATEvents::EventNode::TPM_NOT_ACTIVE:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_NOT_ACTIVE;
		break;

	case ATEvents::EventNode::TPM_OWNED:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_OWNED;
		break;

	case ATEvents::EventNode::TPM_NOT_OWNED:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_NOT_OWNED;
		break;

	case ATEvents::EventNode::TPM_ENABLE:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_ENABLE;
		break;

	case ATEvents::EventNode::TPM_NOT_ENABLE:
		eventtype = MESL_EVENT_TYPE_TPM;
		body = MESD_RES_NOTIF_TPM_NOT_ENABLED;
		break;
#endif

	default:
		body = MESD_RES_NOTIF_ANY_EVENT;
		return false;
	}

	switch (eventtype)
	{
	case MESL_EVENT_TYPE_AMT:
		if ((m_sku & IPCiATStatus::SKU_AMTFUND) != 0) {
			title = MESD_RES_NOTIF_AMT_BASIC;
		} else {
			title = MESD_RES_NOTIF_AMT;
		}
		break;

#ifdef WITH_TPM
	case MESL_EVENT_TYPE_TPM:
		title = MESD_RES_NOTIF_TPM;
		break;
#endif
	default:
		title = MESD_RES_NOTIF_ANY;
	}

	return true;
}

int MEStatusLogic::eventTypes(std::list<ATEvents::EventNode> &events) const
{
	int evTypes = 0;

	std::list<ATEvents::EventNode>::const_iterator iter = events.begin();
	std::list<ATEvents::EventNode>::const_iterator end = events.end();

	for (; iter != end; iter++) {
		switch (iter->m_id)
		{
		case ATEvents::EventNode::AMT_ENABLED:
		case ATEvents::EventNode::AMT_DISABLED:
		case ATEvents::EventNode::PROVISIONED:
		case ATEvents::EventNode::UNPROVISIONED:
		case ATEvents::EventNode::SOL_STARTED:
		case ATEvents::EventNode::SOL_FINISHED:
		case ATEvents::EventNode::IDER_STARTED:
		case ATEvents::EventNode::IDER_FINISHED:
		case ATEvents::EventNode::SD_STARTED:
		case ATEvents::EventNode::SD_FINISHED:
		case ATEvents::EventNode::REMOTE_REBOOT:
			evTypes |= MESL_EVENT_TYPE_AMT;
			break;
#ifdef WITH_TPM
		case ATEvents::EventNode::TPM_OPERATIONAL:
		case ATEvents::EventNode::TPM_NOT_DETECTED:
		case ATEvents::EventNode::TPM_FAIL:
		case ATEvents::EventNode::TPM_ACTIVE:
		case ATEvents::EventNode::TPM_NOT_ACTIVE:
		case ATEvents::EventNode::TPM_OWNED:
		case ATEvents::EventNode::TPM_NOT_OWNED:
		case ATEvents::EventNode::TPM_ENABLE:
		case ATEvents::EventNode::TPM_NOT_ENABLE:
			evTypes |= MESL_EVENT_TYPE_TPM;
			break;
#endif

		default:
			evTypes |= MESL_EVENT_TYPE_OTHER;
		}
	}

	return evTypes;
}

bool MEStatusLogic::isEventNotifiable(const ATEvents::EventNode &curEvent)
{
	switch (curEvent.m_id)
	{
	case ATEvents::EventNode::AMT_ENABLED:
	case ATEvents::EventNode::AMT_DISABLED:
#ifdef WITH_TPM
	case ATEvents::EventNode::TPM_OPERATIONAL:
	case ATEvents::EventNode::TPM_NOT_DETECTED:
#endif
		return true;
		break;

	default:
		return false;
	}

	return true;
}

