/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifndef _ME_STATUS_LOGIC_H
#define _ME_STATUS_LOGIC_H

#include "IMEGuiCommands.h"
#include "IMEIconCommands.h"
#include "IMEGuiNotifications.h"
#include "IMEIconNotifications.h"
#include "Semaphore.h"
#include "IMEAMTStatus.h"
#include "IPCiATStatus.h"
#include "MEStatusConfig.h"
#include "AMTStatusColl.h"

#ifdef WITH_TPM
#include "TPMStatus.h"
#include "TPMStatusColl.h"
#endif

#define AMT_NOTIFY_INFO        0
#define AMT_NOTIFY_EVENT       (1 << 0)
#define AMT_NOTIFY_ERROR       (1 << 1)
#define AMT_NOTIFY_ERROR_EVENT (AMT_NOTIFY_EVENT | AMT_NOTIFY_ERROR)
#define AMT_NOTIFY_NOBUBBLE    (1 << 3)

//******************************************************************************
//
// Class:	    MEStatusLogic
//
// Description:	This class handles all the logic of the MEStatus application,
//              and it communicates with the outside world through 4 interfaces:
//               IMEGuiCommands, IMEIconCommands,
//                which it inherits from, and
//               IMEGuiNotifications, IMEIconNotifications, which it points to.
//              The idea is for the user of this class to treat it
//                as an IME[Gui/Icon]Commands object.
//              The user would pass it a pointer to IME[Gui/Icon]Notifications
//              objects which would receive notifications that
//              are caused by activating commands from IME[Gui/Icon]Commands.
//
//******************************************************************************
class MEStatusLogic : public IMEGuiCommands, public IMEIconCommands
{
public:
	//ctor
	MEStatusLogic();
	//dtor
	virtual ~MEStatusLogic();

	//changes to application or notification mode - must be set before Init
	void SetModeApplication(bool modeApp);
	//initialization
	bool Init(MEStatusConfig *pConfig,
		  IMEAMTStatus *pMEAMTDataReader,
		  IMEGuiNotifications *pMEGuiNotifications,
		  IMEIconNotifications *pMEIconNotifications);

	void StartLogic();

	void UpdateData();

	//from IMEGuiCommands and IMEIconCommands
	virtual void SetStatusIconAlways(bool bAlwaysIcon);
	virtual bool GetStatusIconAlways() const;
	virtual void SetPopupForCurrentUser(bool bPopupForCurrentUser);
	virtual bool GetPopupForCurrentUser() const;
	virtual std::string GetOEMUrl() const;
	virtual std::string GetDefaultOEMUrl() const;
	virtual unsigned int GetEventHistoryMaxReadSize() const;
	virtual void SetAppPresence(bool present);
	virtual void PresentStatusDialog();
	virtual bool GetModeApplication() const;
	virtual void NotificationClosed();
	virtual void SetShowAlways(int rule);

protected:
	enum State {STATE_INITIAL_UNKNOWN, STATE_ENABLED, STATE_DISABLED, STATE_UNKNOWN};
	enum ConnState {STATE_INITIAL_DISCONNECTED, STATE_CONNECTED, STATE_DISCONNECTED};

	struct Notification {
		Glib::ustring title;
		Glib::ustring body;
		int show;
	};

private:
#ifdef WITH_TPM
	bool fetchStatusData(AMTStatusData &amtData, TPMStatusData &tpmData);
#else
	bool fetchStatusData(AMTStatusData &amtData);
#endif
	void updateAMTStatusData(bool haveAmtData, AMTStatusData &amtData);
	void setAMTStatusData(AMTStatusData &amtData);
#ifdef WITH_TPM
	void updateTPMStatusData(bool haveTpmData, TPMStatusData &tpmData);
	void setTPMStatusData(TPMStatusData &tpmData);
#endif
	void setSku(unsigned int sku);

	bool setConnState(bool haveConn);

	bool setState(unsigned char amtState);
	bool setStateOnInitialUnknownState(unsigned char amtState);
	//Return internal state corresponding to AMT state.
	State getStateByAMTState(unsigned char amtState) const;
#ifdef WITH_TPM
	bool setTPMState(TPMStatusData &tpmData);
	bool setTPMStateOnInitialUnknownState(TPMStatusData &tpmData);
	//Return internal state corresponding to TPM data.
	State getStateByTPMData(TPMStatusData &tpmData) const;
#endif

	void setStateToIcon();
	void checkQuitLogic();

	void notificationOpen(bool haveConnChanged, bool needNotifyState);
	void notify(const Glib::ustring &title, const Glib::ustring &body, int show = AMT_NOTIFY_EVENT);
	void notify(const struct MEStatusLogic::Notification &qNotif);
	bool notifyEvents();
	bool eventToNotify(const ATEvents::EventNode &curEvent, Glib::ustring &title, Glib::ustring &body) const;
	void addNewEvents(std::list<ATEvents::EventNode> &newEvents);
	void setEventHistory();
	int eventTypes(std::list<ATEvents::EventNode> &events) const;
	bool isEventNotifiable(const ATEvents::EventNode &curEvent);


	bool      m_modeApp;
	ConnState m_unsConnState;
	State     m_State;
#ifdef WITH_TPM
	State     m_TPMState;
#endif
	unsigned int m_sku;
	unsigned long m_nUnsConnCount;
	unsigned long m_nSetStatusCount;
#ifdef WITH_TPM
	unsigned long m_nSetTPMCount;
#endif
	bool      m_appPresent;
	int       m_showAlways;
	std::list<ATEvents::EventNode> m_lEvents;
#ifdef MES_NOTIFY_MANY_EVENTS
	bool      m_eventsNotifiable;
#endif
	Semaphore m_eventsSem;
	std::list<struct MEStatusLogic::Notification> m_queuedNotif;
	bool      m_notifOpen;
	Semaphore m_notifSem;

	IPCiATStatus  m_MEUNSDataReader;
	AMTStatusColl m_AMTStatusColl;
#ifdef WITH_TPM
	TPMStatus     m_TPMDataReader;
	TPMStatusColl m_TPMStatusColl;
#endif

	MEStatusConfig *m_pConfig;
	IMEAMTStatus *m_pMEAMTDataReader;
	IMEGuiNotifications *m_pMEGuiNotifications;
	IMEIconNotifications *m_pMEIconNotifications;

	bool m_started;
};

#endif //_ME_STATUS_LOGIC_H
