/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "iatshareddata.h"
#include "AMTStatusColl.h"
#include <memory>
#include <cstring>
#include <fstream>
#include <unistd.h>

const std::string AMTStatusColl::defaultLastAMTCollStateFileName = IATSHAREDDATADIR "/iAMTCollState.dat";

AMTStatusColl::AMTStatusColl()
{
	memset(&m_baseAMTStatusData, 0, sizeof(AMTStatusData));
	m_bInitialized = false;
	m_bBaseDataSaved = false;
	m_bAutosave = false;
	m_lastAMTCollStateFileName = defaultLastAMTCollStateFileName;
}

int AMTStatusColl::Init(bool autosave, std::string fname)
{
	m_bAutosave = autosave;
	m_lastAMTCollStateFileName = fname;
	m_bInitialized = true;
	return loadBaseData();
}

void AMTStatusColl::GetBaseData(AMTStatusData &baseAMTStatusData)
{
	memcpy(&baseAMTStatusData, &m_baseAMTStatusData, sizeof(AMTStatusData));
}

void AMTStatusColl::SetBaseData(const AMTStatusData &baseAMTStatusData)
{
	memcpy(&m_baseAMTStatusData, &baseAMTStatusData, sizeof(AMTStatusData));
	m_bInitialized = true;
}

/*
 * collate new status data with base one, set events
 * return: <0 - on error, 0 - if no difference, >0 - 1 + number of difference based events
 **/
int AMTStatusColl::Collate(const AMTStatusData &newAMTStatusData, std::list<ATEvents::EventNode> *events)
{
	int diffs = 0;
	bool operational = false;

	if (!m_bInitialized) {
		return -2;
	}

	//ignore TS
	m_baseAMTStatusData.dataTS = newAMTStatusData.dataTS;

	if (0 == memcmp(&m_baseAMTStatusData, &newAMTStatusData, sizeof(AMTStatusData))) {
		return 0;
	}

	if ((AMTSTATE_NOT_DETECTED == m_baseAMTStatusData.amtState) ||
	    (AMTSTATE_NOT_DETECTED == newAMTStatusData.amtState) ||
	    (NULL == events)
	    ) {
		updateBaseData(newAMTStatusData);
		return 1;
	}

	diffs = 1;

	//here events born

	if (AMTSTATE_POST == newAMTStatusData.amtState) {
		operational = true;
	}

	if ((operational)
	    && (0 != (newAMTStatusData.content & CONTENT_HOST_RESET_REASON))
	    && (HOST_RESET_REASON_REMOTE == newAMTStatusData.resetHostReason))
	{
		//remote reset happened
		if ((0 == (m_baseAMTStatusData.content & CONTENT_HOST_RESET_REASON))
		    || (HOST_RESET_REASON_REMOTE != m_baseAMTStatusData.resetHostReason))
		{
			//remote reboot not reported before
			events->push_back(ATEvents::EventNode(ATEvents::EventNode::REMOTE_REBOOT, newAMTStatusData.resetHostTS));
			diffs++;
		}
	}

	if (m_baseAMTStatusData.amtState != newAMTStatusData.amtState) {
		if (AMTSTATE_DISABLED == m_baseAMTStatusData.amtState) {
			events->push_back(ATEvents::EventNode(ATEvents::EventNode::AMT_ENABLED));
			diffs++;
		}
		if (AMTSTATE_POST == newAMTStatusData.amtState) {
			events->push_back(ATEvents::EventNode(ATEvents::EventNode::PROVISIONED));
		} else {
			events->push_back(ATEvents::EventNode(ATEvents::EventNode::UNPROVISIONED));
		}
		diffs++;
		if (AMTSTATE_DISABLED == newAMTStatusData.amtState) {
			events->push_back(ATEvents::EventNode(ATEvents::EventNode::AMT_DISABLED));
			diffs++;
		}
	}

	if ((operational) && (0 != (newAMTStatusData.content & CONTENT_FEATURES))) {
		if (0 != newAMTStatusData.features[2]) {
			//SD is started
			if ((0 == (m_baseAMTStatusData.content & CONTENT_FEATURES))
			    || (0 == m_baseAMTStatusData.features[2]))
			{
				//SD was not started or unknown
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::SD_STARTED));
				diffs++;
			}
		} else {
			//SD is not started
			if ((0 != (m_baseAMTStatusData.content & CONTENT_FEATURES))
			    && (0 != m_baseAMTStatusData.features[2]))
			{
				//SD was previously started
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::SD_FINISHED));
				diffs++;
			}
		}

		if (0 != newAMTStatusData.features[0]) {
			//SOL is started
			if ((0 == (m_baseAMTStatusData.content & CONTENT_FEATURES))
			    || (0 == m_baseAMTStatusData.features[0]))
			{
				//SOL was not started or unknown
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::SOL_STARTED));
				diffs++;
			}
		} else {
			//SOL is not started
			if ((0 != (m_baseAMTStatusData.content & CONTENT_FEATURES))
			    && (0 != m_baseAMTStatusData.features[0]))
			{
				//SOL was previously started
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::SOL_FINISHED));
				diffs++;
			}
		}

		if (0 != newAMTStatusData.features[1]) {
			//IDER is started
			if ((0 == (m_baseAMTStatusData.content & CONTENT_FEATURES))
			    || (0 == m_baseAMTStatusData.features[1]))
			{
				//IDER was not started or unknown
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::IDER_STARTED));
				diffs++;
			}
		} else {
			//IDER is not started
			if ((0 != (m_baseAMTStatusData.content & CONTENT_FEATURES))
			    && (0 != m_baseAMTStatusData.features[1]))
			{
				//IDER was previously started
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::IDER_FINISHED));
				diffs++;
			}
		}
	}

	updateBaseData(newAMTStatusData);

	return diffs;
}

int AMTStatusColl::updateBaseData(const AMTStatusData &newAMTStatusData)
{
	m_baseAMTStatusData.dataTS = newAMTStatusData.dataTS;
	if (AMTSTATE_NOT_DETECTED != newAMTStatusData.amtState) {
		m_baseAMTStatusData.amtState = newAMTStatusData.amtState;
	}
	if (0 != (newAMTStatusData.content & CONTENT_ME_RESET_REASON)) {
		m_baseAMTStatusData.resetMEReason = newAMTStatusData.resetMEReason;
		m_baseAMTStatusData.content |= CONTENT_ME_RESET_REASON;
	}
	if (0 != (newAMTStatusData.content & CONTENT_HOST_RESET_REASON)) {
		m_baseAMTStatusData.resetHostReason = newAMTStatusData.resetHostReason;
		m_baseAMTStatusData.resetHostTS = newAMTStatusData.resetHostTS;
		m_baseAMTStatusData.content |= CONTENT_HOST_RESET_REASON;
	}
	if (0 != (newAMTStatusData.content & CONTENT_AMT_VER)) {
		memcpy(m_baseAMTStatusData.amtVersion, newAMTStatusData.amtVersion, sizeof(m_baseAMTStatusData.amtVersion));
		m_baseAMTStatusData.content |= CONTENT_AMT_VER;
	}
	if (0 != (newAMTStatusData.content & CONTENT_PROVMODE)) {
		m_baseAMTStatusData.provMode = newAMTStatusData.provMode;
		m_baseAMTStatusData.content |= CONTENT_PROVMODE;
	}
	if (0 != (newAMTStatusData.content & CONTENT_WUISTATE)) {
		m_baseAMTStatusData.webUIState = newAMTStatusData.webUIState;
		m_baseAMTStatusData.content |= CONTENT_WUISTATE;
	}
	if (0 != (newAMTStatusData.content & CONTENT_FEATURES)) {
		memcpy(m_baseAMTStatusData.features, newAMTStatusData.features, sizeof(m_baseAMTStatusData.features));
		m_baseAMTStatusData.content |= CONTENT_FEATURES;
	}
	if (0 != (newAMTStatusData.content & CONTENT_LANWIRED)) {
		memcpy(&m_baseAMTStatusData.lanWired, &newAMTStatusData.lanWired, sizeof(amtLanSettings));
		m_baseAMTStatusData.content |= CONTENT_LANWIRED;
	}
	if (0 != (newAMTStatusData.content & CONTENT_LANWLESS)) {
		memcpy(&m_baseAMTStatusData.lanWireless, &newAMTStatusData.lanWireless, sizeof(amtLanSettings));
		m_baseAMTStatusData.content |= CONTENT_LANWLESS;
	}
	if (0 != (newAMTStatusData.content & CONTENT_POWERPOL)) {
		memcpy(m_baseAMTStatusData.powerPolicyName, newAMTStatusData.powerPolicyName, sizeof(m_baseAMTStatusData.powerPolicyName));
		m_baseAMTStatusData.content |= CONTENT_POWERPOL;
	}
	if (0 != (newAMTStatusData.content & CONTENT_LMS_VER)) {
		memcpy(m_baseAMTStatusData.lmsVersion, newAMTStatusData.lmsVersion, sizeof(m_baseAMTStatusData.lmsVersion));
		m_baseAMTStatusData.lmsRunning = newAMTStatusData.lmsRunning;
		m_baseAMTStatusData.content |= CONTENT_LMS_VER;
	}
	if (0 != (newAMTStatusData.content & CONTENT_MEI_VER)) {
		memcpy(&m_baseAMTStatusData.heciVersion, &newAMTStatusData.heciVersion, sizeof(m_baseAMTStatusData.heciVersion));
		m_baseAMTStatusData.content |= CONTENT_MEI_VER;
	}
	if (0 != (newAMTStatusData.content & CONTENT_RA)) {
		m_baseAMTStatusData.amtNetConnStatus        = newAMTStatusData.amtNetConnStatus;
		m_baseAMTStatusData.remoteAccessConnStatus  = newAMTStatusData.remoteAccessConnStatus;
		m_baseAMTStatusData.remoteAccessConnTrigger = newAMTStatusData.remoteAccessConnTrigger;
#if 0
		memcpy(&m_baseAMTStatusData.mpsHostname, &newAMTStatusData.mpsHostname, sizeof(m_baseAMTStatusData.mpsHostname));
#endif
		m_baseAMTStatusData.content |= CONTENT_RA;
	}

	if (m_bAutosave) {
		m_bBaseDataSaved = ((0 == saveBaseData()) ? true : false);
	}

	return 0;
}

int AMTStatusColl::loadBaseData()
{
	AMTStatusData tmpData;

	std::ifstream file(m_lastAMTCollStateFileName.c_str(), std::ios_base::in | std::ios_base::binary);
	if (!file.is_open()) {
		return -1;
	}
	file.read((char *)&tmpData, sizeof(AMTStatusData));
	if (!file.good()) {
		file.close();
		return -2;
	}
	file.close();

	//sanity
	tmpData.amtVersion[sizeof(tmpData.amtVersion) - 1] = '\0';
	tmpData.lmsVersion[sizeof(tmpData.lmsVersion) - 1] = '\0';
	tmpData.powerPolicyName[sizeof(tmpData.powerPolicyName) - 1] = '\0';

	memcpy(&m_baseAMTStatusData, &tmpData, sizeof(AMTStatusData));

	return 0;
}

int AMTStatusColl::saveBaseData()
{
	std::ofstream file(m_lastAMTCollStateFileName.c_str(), std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
	if (!file.is_open()) {
		unlink(m_lastAMTCollStateFileName.c_str());
		return -1;
	}
	file.write((const char *)&m_baseAMTStatusData, sizeof(AMTStatusData));
	if (file.fail()) {
		file.close();
		unlink(m_lastAMTCollStateFileName.c_str());
		return -2;
	}
	file.flush();
	if (file.fail()) {
		file.close();
		unlink(m_lastAMTCollStateFileName.c_str());
		return -4;
	}
	file.close();
	return 0;
}

