/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "MEIAMTStatus.h"
#include "ATVersion.h"

#include <sstream>
#include <string>
#include <iomanip>
#include <cstdlib>

#define PROVISIONING_STATE_AMT_NOT_DETECTED -1
#define PROVISIONING_STATE_AMT_DISABLED     -2
#define PROVISIONING_STATE_MEI_DISABLED     -3

// Constructor
MEIAMTStatus::MEIAMTStatus(bool verbose) :
pthiCommand(verbose), fwulCommand(verbose)
{
	memset(&m_AMTStatusData, 0, sizeof(m_AMTStatusData));
	m_AMTStatusData.amtState = AMTSTATE_NOT_DETECTED;
	m_AMTStatusData.content = 0;
	m_wasAMTDisabled = false;
	m_verbose = verbose;
	m_cachedLmsTs = 0;
	m_cachedLmsIsVer = false;
	memset(m_cachedLmsVer, 0, sizeof(m_cachedLmsVer));
}

int MEIAMTStatus::getAMTStatus()
{
	int provisioningState = PROVISIONING_STATE_AMT_NOT_DETECTED;
	CODE_VERSIONS codeVersions;
	HECI_VERSION  heciVersion;
	AMT_PROVISIONING_STATE provState;

	if (!pthiCommand.PTHIClient.Init()) {
		debugLog("Got status from MEI: MEI init failed.");
		if (pthiCommand.PTHIClient.GetHeciVersion(heciVersion)) {
			m_AMTStatusData.heciVersion.major  = heciVersion.major;
			m_AMTStatusData.heciVersion.minor  = heciVersion.minor;
			m_AMTStatusData.heciVersion.hotfix = heciVersion.hotfix;
			m_AMTStatusData.heciVersion.build  = heciVersion.build;
			m_AMTStatusData.content |= CONTENT_MEI_VER;

			m_AMTStatusData.amtState = AMTSTATE_DISABLED;
			return PROVISIONING_STATE_AMT_DISABLED;
		} else {
			m_AMTStatusData.amtState = AMTSTATE_MEI_DISABLED;
			return PROVISIONING_STATE_MEI_DISABLED;
		}
	}

	if (AMT_STATUS_SUCCESS != pthiCommand.GetHeciVersion(heciVersion)) {
		debugLog("Got status from MEI: MEI command failed.");
		m_AMTStatusData.amtState = AMTSTATE_NOT_DETECTED;
		pthiCommand.PTHIClient.Deinit();
		return PROVISIONING_STATE_AMT_NOT_DETECTED;
	}

	m_AMTStatusData.heciVersion.major  = heciVersion.major;
	m_AMTStatusData.heciVersion.minor  = heciVersion.minor;
	m_AMTStatusData.heciVersion.hotfix = heciVersion.hotfix;
	m_AMTStatusData.heciVersion.build  = heciVersion.build;
	m_AMTStatusData.content |= CONTENT_MEI_VER;

	if (AMT_STATUS_SUCCESS != pthiCommand.GetCodeVersions(codeVersions)) {
		debugLog("Got status from MEI: MEI command failed.");
		m_AMTStatusData.amtState = AMTSTATE_DISABLED;
		pthiCommand.PTHIClient.Deinit();
		return PROVISIONING_STATE_AMT_DISABLED;
	}

	unsigned int i = 0;
	std::string AMTVersion = "";
	for (i = 0; i < codeVersions.VersionsCount; ++i) {
		if (!strcmp("AMT", (char *)codeVersions.Versions[i].Description.String)) {
			AMTVersion.assign((char *)codeVersions.Versions[i].Version.String,
					  codeVersions.Versions[i].Version.Length);
			m_AMTStatusData.content |= CONTENT_AMT_VER;
		}
	}
	for (i = 0; i < codeVersions.VersionsCount; ++i) {
		if (!strcmp("Build Number", (char *)codeVersions.Versions[i].Description.String)) {
			AMTVersion.append(".");
			AMTVersion.append((char *)codeVersions.Versions[i].Version.String,
					  codeVersions.Versions[i].Version.Length);
		}
	}
	memset(m_AMTStatusData.amtVersion, 0, sizeof(m_AMTStatusData.amtVersion));
	strncpy((char *)m_AMTStatusData.amtVersion, AMTVersion.c_str(), sizeof(m_AMTStatusData.amtVersion) - 1);

	if (isDebugEnabled()) {
		std::stringstream ss;
		ss << "AMT version: " << AMTVersion;
		debugLog(ss.str().c_str());
	}

	if (AMT_STATUS_SUCCESS != pthiCommand.GetProvisioningState(provState)) {
		debugLog("Got status from MEI: MEI command failed.");
		m_AMTStatusData.amtState = AMTSTATE_NOT_DETECTED;
		pthiCommand.PTHIClient.Deinit();
		return PROVISIONING_STATE_AMT_NOT_DETECTED;
	}
	provisioningState = provState;

	if (provisioningState == PROVISIONING_STATE_IN) {
		m_AMTStatusData.amtState = AMTSTATE_IN;
		debugLog("Got status from MEI: AMT is IN provisioning");
	}
	else if (provisioningState == PROVISIONING_STATE_PRE) {
		m_AMTStatusData.amtState = AMTSTATE_PRE;
		debugLog("Got status from MEI: AMT is PRE provisioning");
	}
	else {
		m_AMTStatusData.amtState = AMTSTATE_POST;
		debugLog("Got status from MEI: AMT is POST provisioning");
	}

	pthiCommand.PTHIClient.Deinit();

	return provisioningState;
}

bool MEIAMTStatus::isAMTStatusDataAMTVersionValid()
{
	if (0 == (m_AMTStatusData.content & CONTENT_AMT_VER)) {
		return false;
	}
	if (m_AMTStatusData.amtVersion[0] < '4') {
		return false;
	}
	if (m_AMTStatusData.amtVersion[0] > '5') {
		return false;
	}
	if (m_AMTStatusData.amtVersion[1] == '.') {
		return true;
	}
	return false;
}

bool MEIAMTStatus::getAMTInfo()
{
	bool result = false;

	if ((AMTSTATE_NOT_DETECTED == m_AMTStatusData.amtState)
	    || (AMTSTATE_DISABLED == m_AMTStatusData.amtState)
	    || (AMTSTATE_MEI_DISABLED == m_AMTStatusData.amtState)
	    )
	{
		return false;
	}

	if (AMTSTATE_POST == m_AMTStatusData.amtState) {
		if (!pthiCommand.PTHIClient.Init()) {
			debugLog("Got info from MEI: MEI init failed.");
			return false;
		}
		result = getAMTHECIInfo();
		pthiCommand.PTHIClient.Deinit();
	}

	m_AMTStatusData.dataTS = time(NULL);

	return result;
}

bool MEIAMTStatus::getAMTHECIInfo()
{
	bool result = true;

	if (!getProvisioningMode()) {
		result = false;
	}

	if (isAMTStatusDataAMTVersionValid()) {
		if (!getLastHostResetReason()) {
			result = false;
		}
		if (!getMEResetReason()) {
			result = false;
		}
		if (!getNetworkSettings()) {
			result = false;
		}
		if (!getRemoteAccessStatus()) {
			result = false;
		}
		if (!getFeaturesStatus()) {
			result = false;
		}
		if (!getWebUIState()) {
			result = false;
		}
		if (!getPowerPolicy()) {
			result = false;
		}
	}

	return result;
}

bool MEIAMTStatus::getProvisioningMode()
{
	UINT32 hres;
	CFG_PROVISIONING_MODE provMode;

	if (AMT_STATUS_SUCCESS != (hres = pthiCommand.GetProvisioningMode(provMode))) {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Provisioning mode - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
		return false;
	}

	switch (provMode)
	{
	case CFG_PROVISIONING_MODE_ENTERPRISE:
		m_AMTStatusData.provMode = AMTMODE_ENTERPRISE;
		break;

	case CFG_PROVISIONING_MODE_SMALL_BUSINESS:
		m_AMTStatusData.provMode = AMTMODE_SB;
		break;

	case CFG_PROVISIONING_MODE_REMOTE_ASSISTANCE:
		m_AMTStatusData.provMode = AMTMODE_REMOTE;
		break;

	default:
		m_AMTStatusData.provMode = AMTMODE_NONE;
	}
	m_AMTStatusData.content |= CONTENT_PROVMODE;

	if (isDebugEnabled()) {
		std::stringstream ss;
		ss << "ProvisioningMode: " << (int)m_AMTStatusData.provMode;
		debugLog(ss.str().c_str());
	}

	return true;
}


bool MEIAMTStatus::getFeaturesStatus()
{
	AMT_BOOLEAN redirectionStatus[2];
	AMT_BOOLEAN systemDefense[2];
	UINT32 hres;
	UINT32 requestID = REDIRECTION_SESSION;

	if (AMT_STATUS_SUCCESS != (hres = pthiCommand.GetFeaturesState(requestID, redirectionStatus))) {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Features (ses redirection) - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
		return false;
	}
	requestID = SYSTEM_DEFENSE;
	if (AMT_STATUS_SUCCESS != (hres = pthiCommand.GetFeaturesState(requestID, systemDefense))) {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Features (defense) - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
		return false;
	}

	m_AMTStatusData.features[0] = redirectionStatus[0];
	m_AMTStatusData.features[1] = redirectionStatus[1];
	m_AMTStatusData.features[2] = systemDefense[0];
	m_AMTStatusData.content |= CONTENT_FEATURES;

	if (isDebugEnabled()) {
		std::stringstream ss;
		ss << "Features: " << (int)redirectionStatus[0] << " : " << (int)redirectionStatus[0] << " Defense: " << (int)systemDefense[0];
		debugLog(ss.str().c_str());
	}

	return true;
}


bool MEIAMTStatus::getWebUIState()
{
	AMT_BOOLEAN webUIActive[2];
	AMT_BOOLEAN tlsEnabled;
	UINT32 hres;
	UINT32 requestID = WEB_UI;

	if (AMT_STATUS_SUCCESS != (hres = pthiCommand.GetFeaturesState(requestID, webUIActive))) {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Web UI state - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
		return false;
	}
	if (0 != webUIActive[0]) {
		if ((AMT_STATUS_SUCCESS == (hres = pthiCommand.GetTLSEnabled(tlsEnabled)))
		    && (0 != tlsEnabled)) {
			m_AMTStatusData.webUIState = WUISTATE_TLS;
		}
		else {
			m_AMTStatusData.webUIState = WUISTATE_ENABLED;
		}
	}
	else {
		m_AMTStatusData.webUIState = WUISTATE_DISABLED;
	}
	m_AMTStatusData.content |= CONTENT_WUISTATE;

	if (isDebugEnabled()) {
		std::stringstream ss;
		ss << "WebUIState: " << (int)m_AMTStatusData.webUIState;
		debugLog(ss.str().c_str());
	}

	return true;
}

bool MEIAMTStatus::getPowerPolicy()
{
	UINT32 hres;
	AMT_ANSI_STRING powerPolicyName;

	powerPolicyName.Buffer = NULL;
	if (AMT_STATUS_SUCCESS != (hres = pthiCommand.GetCurrentPowerPolicy(powerPolicyName))) {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Power policy - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
		return false;
	}
	memset(m_AMTStatusData.powerPolicyName, 0, sizeof(m_AMTStatusData.powerPolicyName));
	if (powerPolicyName.Length < sizeof(m_AMTStatusData.powerPolicyName)) {
		memcpy(m_AMTStatusData.powerPolicyName, powerPolicyName.Buffer, powerPolicyName.Length);
	}
	else {
		memcpy(m_AMTStatusData.powerPolicyName, powerPolicyName.Buffer, sizeof(m_AMTStatusData.powerPolicyName) - 1);
	}
	m_AMTStatusData.content |= CONTENT_POWERPOL;

	if (isDebugEnabled()) {
		std::stringstream ss;
		ss << "Power policy: " << m_AMTStatusData.powerPolicyName;
		debugLog(ss.str().c_str());
	}

	if (powerPolicyName.Buffer) {
		free(powerPolicyName.Buffer);
	}

	return true;
}

bool MEIAMTStatus::getMEResetReason()
{
	UINT32 hres = 0;
	UINT8 resetReason;

	if (AMT_STATUS_SUCCESS == (hres = pthiCommand.GetFWResetReason(resetReason))) {
		m_AMTStatusData.content |= CONTENT_ME_RESET_REASON;
		m_AMTStatusData.resetMEReason = resetReason;

		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "MEResetReason: " << (unsigned int)resetReason;
			debugLog(ss.str().c_str());
		}

		return true;
	} else {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "ME reset reason - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
	}
	return false;
}

bool MEIAMTStatus::getLastHostResetReason()
{
	UINT32 hres = 0;
	UINT32 resetReason;
	UINT32 resetTS;

	if (AMT_STATUS_SUCCESS == (hres = pthiCommand.GetLastHostResetReason(resetReason, resetTS))) {
		m_AMTStatusData.content |= CONTENT_HOST_RESET_REASON;
		m_AMTStatusData.resetHostReason = resetReason;
		m_AMTStatusData.resetHostTS = resetTS;

		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "LastHostResetReason: " << (unsigned int)resetReason;
			debugLog(ss.str().c_str());
		}

		return true;
	} else {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Last host reset reason - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
	}
	return false;
}

bool MEIAMTStatus::getNetworkSettings()
{
	LAN_SETTINGS lanSettings;
	bool result = true;
	UINT32 AMTinterface = WIRED;
	UINT32 hres;

	if (AMT_STATUS_SUCCESS == (hres = pthiCommand.GetLanInterfaceSettings(AMTinterface, lanSettings))) {
		m_AMTStatusData.lanWired.Enabled     = lanSettings.Enabled;
		m_AMTStatusData.lanWired.Ipv4Address = lanSettings.Ipv4Address;
		m_AMTStatusData.lanWired.DhcpEnabled = lanSettings.DhcpEnabled;
		m_AMTStatusData.lanWired.DhcpIpMode  = lanSettings.DhcpIpMode;
		m_AMTStatusData.lanWired.LinkStatus  = lanSettings.LinkStatus;
		memcpy(&(m_AMTStatusData.lanWired.MacAddress), lanSettings.MacAddress, 6);
		m_AMTStatusData.content |= CONTENT_LANWIRED;

		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Lan Wired: " << ((0 != lanSettings.Enabled) ? "Enabled" : "Disabled");
			ss << "\n IPv4: 0x" << std::setfill('0') << std::setw(8) << std::hex << lanSettings.Ipv4Address;
			ss << "\n Dhcp: " << ((0 != lanSettings.DhcpEnabled) ? "Enabled" : "Disabled")
			    << "\n Dhcp IP mode: " << (int)lanSettings.DhcpIpMode
			    << "\n Link status: " << (int)lanSettings.LinkStatus
			    << "\n Mac: ";
			ss << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[0];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[1];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[2];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[3];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[4];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[5];
			debugLog(ss.str().c_str());
		}
	}
	else {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "LAN wired interface - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
		result = false;
	}
	AMTinterface = WIRELESS;
	if (AMT_STATUS_SUCCESS == (hres = pthiCommand.GetLanInterfaceSettings(AMTinterface, lanSettings))) {
		m_AMTStatusData.lanWireless.Enabled     = lanSettings.Enabled;
		m_AMTStatusData.lanWireless.Ipv4Address = lanSettings.Ipv4Address;
		m_AMTStatusData.lanWireless.DhcpEnabled = lanSettings.DhcpEnabled;
		m_AMTStatusData.lanWireless.DhcpIpMode  = lanSettings.DhcpIpMode;
		m_AMTStatusData.lanWireless.LinkStatus  = lanSettings.LinkStatus;
		memcpy(&(m_AMTStatusData.lanWireless.MacAddress), lanSettings.MacAddress, 6);
		m_AMTStatusData.content |= CONTENT_LANWLESS;

		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Lan Wireless: " << ((0 != lanSettings.Enabled) ? "Enabled" : "Disabled");
			ss << "\n IPv4: 0x" << std::setfill('0') << std::setw(8) << std::hex << lanSettings.Ipv4Address;
			ss << "\n Dhcp: " << ((0 != lanSettings.DhcpEnabled) ? "Enabled" : "Disabled")
			    << "\n Dhcp IP mode: " << (int)lanSettings.DhcpIpMode
			    << "\n Link status: " << (int)lanSettings.LinkStatus
			    << "\n Mac: ";
			ss << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[0];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[1];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[2];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[3];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[4];
			ss << ":" << std::setfill('0') << std::setw(2) << std::hex << (int)lanSettings.MacAddress[5];
			debugLog(ss.str().c_str());
		}
	}
	else {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "LAN wireless interface - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
	}

	return result;
}

bool MEIAMTStatus::getRemoteAccessStatus()
{
	UINT32 hres;
	REMOTE_ACCESS_STATUS remoteAccessStatus;

	remoteAccessStatus.MpsHostname.Buffer = NULL;
	if (AMT_STATUS_SUCCESS != (hres = pthiCommand.GetRemoteAccessConnectionStatus(remoteAccessStatus))) {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "Remote access status - get failed: " << (unsigned int)hres;
			debugLog(ss.str().c_str());
		}
		return false;
	}

	switch (remoteAccessStatus.AmtNetworkConnectionStatus)
	{
	case AMT_NETWORK_CONNECTION_DIRECT:
		m_AMTStatusData.amtNetConnStatus = AMT_NET_CONN_DIRECT;
		break;
	case AMT_NETWORK_CONNECTION_VPN:
		m_AMTStatusData.amtNetConnStatus = AMT_NET_CONN_VPN;
		break;
	case AMT_NETWORK_CONNECTION_OUTSIDE_ENTERPRISE:
		m_AMTStatusData.amtNetConnStatus = AMT_NET_CONN_OUTSIDE_ENTERPRISE;
		break;
	}

	switch (remoteAccessStatus.RemoteAccessConnectionStatus)
	{
	case REMOTE_ACCESS_CONNECTION_STATUS_NOT_CONNECTED:
		m_AMTStatusData.remoteAccessConnStatus = RA_CONN_NOT_CONNECTED;
		break;
	case REMOTE_ACCESS_CONNECTION_STATUS_CONNECTING:
		m_AMTStatusData.remoteAccessConnStatus = RA_CONN_CONNECTING;
		break;
	case REMOTE_ACCESS_CONNECTION_STATUS_CONNECTED:
		m_AMTStatusData.remoteAccessConnStatus = RA_CONN_CONNECTED;
		break;
	}

	switch (remoteAccessStatus.RemoteAccessConnectionTrigger)
	{
	case REMOTE_ACCESS_CONNECTION_TRIGGER_USER_INITIATED:
		m_AMTStatusData.remoteAccessConnTrigger = RA_TRIGGER_USER_INITIATED;
		break;
	case REMOTE_ACCESS_CONNECTION_TRIGGER_ALERT:
		m_AMTStatusData.remoteAccessConnTrigger = RA_TRIGGER_ALERT;
		break;
	case REMOTE_ACCESS_CONNECTION_TRIGGER_PROVISIONING:
		m_AMTStatusData.remoteAccessConnTrigger = RA_TRIGGER_PROVISIONING;
		break;
	case REMOTE_ACCESS_CONNECTION_TRIGGER_PERIODIC:
		m_AMTStatusData.remoteAccessConnTrigger = RA_TRIGGER_PERIODIC;
		break;
	}

#if 0
	memset(m_AMTStatusData.mpsHostname, 0, sizeof(m_AMTStatusData.mpsHostname));
	if (remoteAccessStatus.MpsHostname.Length < sizeof(m_AMTStatusData.mpsHostname)) {
		memcpy(m_AMTStatusData.mpsHostname,
		       remoteAccessStatus.MpsHostname.Buffer,
		       remoteAccessStatus.MpsHostname.Length);
	}
	else {
		memcpy(m_AMTStatusData.mpsHostname,
		       remoteAccessStatus.MpsHostname.Buffer,
		       sizeof(m_AMTStatusData.mpsHostname) - 1);
	}
#endif

	m_AMTStatusData.content |= CONTENT_RA;

	if (isDebugEnabled()) {
		std::stringstream ss;
		ss << "Remote access status :";
		ss << "\n AMT Network Connection Status : " << remoteAccessStatus.AmtNetworkConnectionStatus;
		ss << "\n Remote Access Connection Status : " << remoteAccessStatus.RemoteAccessConnectionStatus;
		ss << "\n Remote Access Connection Trigger : " << remoteAccessStatus.RemoteAccessConnectionTrigger;
#if 0
		ss << "\n MPS Hostname : " << m_AMTStatusData.mpsHostname;
#endif
		debugLog(ss.str().c_str());
	}

	if (remoteAccessStatus.MpsHostname.Buffer) {
		free(remoteAccessStatus.MpsHostname.Buffer);
	}

	return false;
}

bool MEIAMTStatus::getLmsVersion()
{
	std::string lmsVersion;
	bool isLmsRunning = ATVersion::GetAppVersion("lms", lmsVersion);
	m_AMTStatusData.lmsRunning = ((isLmsRunning) ? 1 : 0);
	if (!lmsVersion.empty()) {
		strncpy(m_AMTStatusData.lmsVersion, lmsVersion.c_str(), sizeof(m_AMTStatusData.lmsVersion));
		m_AMTStatusData.lmsVersion[sizeof(m_AMTStatusData.lmsVersion) - 1] = '\0';
		m_AMTStatusData.content |= CONTENT_LMS_VER;
		return true;
	}
	return false;
}

bool MEIAMTStatus::getLmsVersionCached(time_t ttl)
{
	bool ret = false;
	time_t curTs = time(NULL);

	if (curTs - m_cachedLmsTs > ttl) {
		ret = getLmsVersion();
		if (ret) {
			memcpy(m_cachedLmsVer, m_AMTStatusData.lmsVersion, sizeof(m_cachedLmsVer));
			m_cachedLmsVer[sizeof(m_cachedLmsVer) - 1] = '\0';
		}
		m_cachedLmsIsVer = ret;
		m_cachedLmsTs = curTs;
	} else {
		std::list<unsigned long> tmppids;

		ret = m_cachedLmsIsVer;
		bool isLmsRunning = ATVersion::IsAppRunning("lms", tmppids);
		m_AMTStatusData.lmsRunning = ((isLmsRunning) ? 1 : 0);
		if (ret) {
			memcpy(m_AMTStatusData.lmsVersion, m_cachedLmsVer, sizeof(m_AMTStatusData.lmsVersion));
			m_AMTStatusData.lmsVersion[sizeof(m_AMTStatusData.lmsVersion) - 1] = '\0';
			m_AMTStatusData.content |= CONTENT_LMS_VER;
		}
	}
	return ret;
}

bool MEIAMTStatus::checkAMTDisabled(bool useFWU)
{
	FWU_GET_VERSION_MSG_REPLY verMsg;
	FWU_GET_INFO_MSG_REPLY infoMsg;
	HECI_STATUS heciRet;
	MEFWCAPS_SKU skuBits;
	bool AMTOnSKU = false;

	if (m_AMTStatusData.amtState != AMTSTATE_DISABLED) {
		return false;
	}
	
	m_AMTStatusData.amtState = AMTSTATE_NOT_DETECTED;

	if (!useFWU) {
		if (m_wasAMTDisabled) {
			m_AMTStatusData.amtState = AMTSTATE_DISABLED;
		}
		return false;
	}

	m_wasAMTDisabled = false;

	if (!fwulCommand.FWULClient.Init()) {
		if (isDebugEnabled()) {
			debugLog("Failed to connect FWUpdate client via MEI");
		}
		return false;
	}

	heciRet = fwulCommand.GetFWUVersionAndInfo(verMsg, infoMsg);
	fwulCommand.FWULClient.Deinit();
	if (HECI_STATUS_OK != heciRet) {
		if (isDebugEnabled()) {
			std::stringstream ss;
			ss << "GetFWUVersionAndInfo failed: " << (unsigned int)heciRet;
			debugLog(ss.str().c_str());
		}
		return false;
	}

	skuBits.Data = verMsg.Sku;
	if (skuBits.Fields.Amt) {
		AMTOnSKU = true;
	}

	if (FWU_GET_INFO_REPLY != infoMsg.MessageType) {
		return false;
	}

	if (AMTOnSKU &&
	    (MEFWCAPS_MANAGEABILITY_SUPP_AMT != infoMsg.ManageabilityMode)) {
		/* AMT is disabled */
		m_AMTStatusData.amtState = AMTSTATE_DISABLED;
		m_wasAMTDisabled = true;
	}

	return true;
}

bool MEIAMTStatus::GetAMTStatusData(AMTStatusData &statusData, bool useFWU)
{
	memset(&m_AMTStatusData, 0, sizeof(m_AMTStatusData));
	m_AMTStatusData.amtState = AMTSTATE_NOT_DETECTED;
	m_AMTStatusData.content = 0;

	getAMTStatus();
	checkAMTDisabled(useFWU);
	getAMTInfo();
	getLmsVersionCached();

	memcpy(&statusData, &m_AMTStatusData, sizeof(AMTStatusData));

	return ((AMTSTATE_NOT_DETECTED == m_AMTStatusData.amtState) ? false : true);
}

