/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "iatshareddata.h"
#include "EventHistory.h"
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <unistd.h>

namespace ATEvents
{

const std::string EventHistory::defaultEHFileName = IATSHAREDDATADIR "/iATEventHistory.dat";
const std::string EventHistory::userEHStateFileName = ".IntelATEHState.dat";

EventHistory::EventHistory()
{
	Init(defaultEHFileName, 0);
}

EventHistory::~EventHistory()
{
}

bool EventHistory::Init(const std::string &filename, unsigned int maxsize)
{
	char *home;

	m_fileName = filename;
	m_maxSize = maxsize;

	home = getenv("HOME");
	m_home = ((NULL != home) ? home : "");

	return true;
}

int EventHistory::AddEvent(unsigned char id, time_t ts, unsigned char data1, unsigned char data2, unsigned char data3)
{
	EventHistory::Event ev;

	ev.ts = ((ts != 0) ? ts : time(NULL));
	ev.id = id;
	ev.data[0] = data1;
	ev.data[1] = data2;
	ev.data[2] = data3;

	return addEvent(ev);
}

int EventHistory::AddEvent(const EventNode &event)
{
	EventHistory::Event ev;

	ev.ts = ((event.m_ts != 0) ? event.m_ts : time(NULL));
	ev.id = event.m_id;
	ev.data[0] = event.m_data1;
	ev.data[1] = event.m_data2;
	ev.data[2] = event.m_data3;

	return addEvent(ev);
}

int EventHistory::AddEvents(const std::list<EventNode> &events)
{
	EventHistory::Event ev;
	int i = 0;
	time_t ts = time(NULL);
	
	if (m_fileName.empty()) {
		return -1;
	}

	if (m_maxSize) {
		checkShorten(events.size());
	}

	std::ofstream file(m_fileName.c_str(), std::ios_base::out | std::ios_base::app | std::ios_base::binary);
	if (!file.is_open()) {
		return -2;
	}

	std::list<EventNode>::const_iterator iter = events.begin();
	std::list<EventNode>::const_iterator end = events.end();
	for (i = 0; iter != end; iter++, i++) {
		ev.ts = ((0 != iter->m_ts) ? iter->m_ts : ts);
		ev.id = iter->m_id;
		ev.data[0] = iter->m_data1;
		ev.data[1] = iter->m_data2;
		ev.data[2] = iter->m_data3;
		file.write((const char *)&ev, sizeof(EventHistory::Event));
		if (file.fail()) {
			file.close();
			return i;
		}
	}
	file.flush();
	if (file.fail()) {
		file.close();
		return -4;
	}
	file.close();

	return i;
}
	
int EventHistory::GetEvents(std::list<EventNode> &events)
{
	EventHistory::Event ev;
	int i = 0;

	events.clear();

	if (m_fileName.empty()) {
		return -1;
	}
	std::ifstream file(m_fileName.c_str(), std::ios_base::in | std::ios_base::binary);
	if (!file.is_open()) {
		return -2;
	}

	do {
		file.read((char *)&ev, sizeof(EventHistory::Event));
		if (file.good()) {
			events.push_back(EventNode(ev.id, ev.ts, ev.data[0], ev.data[1], ev.data[2]));
		}
	} while (file.good());
	if ((!file.eof()) || (file.bad())) {
		i = -3;
	}
	file.close();

	return i;
}

int EventHistory::GetLastEvents(std::list<EventNode> &events, unsigned int maxRsize, bool bigskip)
{
	EventHistory::Event ev;
	std::ios_base::openmode mode;
	unsigned int i = 0;

	events.clear();

	if (m_fileName.empty()) {
		return -1;
	}
	mode = std::ios_base::in | std::ios_base::binary;
	if (bigskip) {
		mode |= std::ios_base::ate;
	}
	std::ifstream file(m_fileName.c_str(), mode);
	if (!file.is_open()) {
		return -2;
	}

	if (bigskip) {
		unsigned long seekoff = 0;
		unsigned long fsize = (unsigned long)file.tellg();
		unsigned long numels = fsize / sizeof(EventHistory::Event);
		if (numels > (maxRsize + 1)) {
			numels -= (maxRsize + 1);
			seekoff = numels * sizeof(EventHistory::Event);
		}
		file.seekg(seekoff, std::ios_base::beg);
	}

	do {
		file.read((char *)&ev, sizeof(EventHistory::Event));
		if (file.good()) {
			events.push_back(EventNode(ev.id, ev.ts, ev.data[0], ev.data[1], ev.data[2]));
			if (i >= maxRsize) {
				events.pop_front();
			} else {
				i++;
			}
		}
	} while (file.good());
	if ((!file.eof()) || (file.bad())) {
		file.close();
		return -3;
	}
	file.close();

	return 0;
}

int EventHistory::checkShorten(unsigned int xsize)
{
	if (m_fileName.empty()) {
		return -11;
	}

	std::ifstream file(m_fileName.c_str(), std::ios_base::in | std::ios_base::binary);
	if (!file.is_open()) {
		return -12;
	}
	file.seekg(0, std::ios_base::end);
	int pos = file.tellg();
	file.close();

	unsigned int size = pos / sizeof(EventHistory::Event);
	size += xsize;
	if ((pos > 0) && (size > m_maxSize)) {
		std::string tmpFileName = std::string(m_fileName + ".tmp");
		std::list<EventNode> events;

		if (0 > GetEvents(events)) {
			return -13;
		}

		for (int i = (size - m_maxSize); i > 0; i--) {
			events.erase(events.begin());
		}

		std::ofstream tfile(tmpFileName.c_str(), std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
		if (!tfile.is_open()) {
			events.clear();
			return -14;
		}
		EventHistory::Event ev;
		std::list<EventNode>::const_iterator end = events.end();
		for (std::list<EventNode>::iterator iter = events.begin(); iter != end; iter++) {
			ev.ts = iter->m_ts;
			ev.id = iter->m_id;
			ev.data[0] = iter->m_data1;
			ev.data[1] = iter->m_data2;
			ev.data[2] = iter->m_data3;
			tfile.write((const char *)&ev, sizeof(EventHistory::Event));
			if (tfile.fail()) {
				events.clear();
				tfile.close();
				return -15;
			}
		}
		events.clear();
		tfile.flush();
		if (tfile.fail()) {
			tfile.close();
			return -16;
		}
		tfile.close();

		if (0 != std::rename(tmpFileName.c_str(), m_fileName.c_str())) {
			std::string tmp2FileName = std::string(m_fileName + ".~tmp");
			std::ifstream mfile(m_fileName.c_str(), std::ios_base::in | std::ios_base::binary);
			if (!mfile.is_open()) {
				return -18;
			}
			std::ofstream wfile(tmp2FileName.c_str(), std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
			if (!wfile.is_open()) {
				mfile.close();
				return -19;
			}
			wfile << mfile.rdbuf();
			if (wfile.bad()) {
				mfile.close();
				wfile.close();
				return -20;
			}
			mfile.close();
			wfile.close();

			std::ifstream sfile(tmpFileName.c_str(), std::ios_base::in | std::ios_base::binary);
			if (!sfile.is_open()) {
				return -21;
			}
			std::ofstream dfile(m_fileName.c_str(), std::ios_base::out | std::ios_base::trunc | std::ios_base::binary);
			if (!dfile.is_open()) {
				sfile.close();
				return -22;
			}
			dfile << sfile.rdbuf();
			if (dfile.bad()) {
				sfile.close();
				dfile.close();
				unlink(m_fileName.c_str());
				if (0 != std::rename(tmpFileName.c_str(), m_fileName.c_str())) {
					std::rename(tmp2FileName.c_str(), m_fileName.c_str());
					return -23;
				}
			}
			sfile.close();
			dfile.close();
		}
	}

	return 0;
}

int EventHistory::addEvent(const EventHistory::Event &ev)
{
	if (m_fileName.empty()) {
		return -1;
	}

	if (m_maxSize) {
		checkShorten(1);
	}

	std::ofstream file(m_fileName.c_str(), std::ios_base::out | std::ios_base::app | std::ios_base::binary);
	if (!file.is_open()) {
		return -2;
	}
	file.write((const char *)&ev, sizeof(EventHistory::Event));
	if (file.fail()) {
		file.close();
		return -3;
	}
	file.flush();
	if (file.fail()) {
		file.close();
		return -4;
	}
	file.close();

	return 0;
}

int EventHistory::LoadUserLastEvent(EventNode &readEvent, EventNode &seenEvent)
{
	EventHistory::Event evLast;
	EventHistory::Event evSeen;

	std::ifstream file((m_home + "/" + userEHStateFileName).c_str(), std::ios_base::in | std::ios_base::binary);
	if (!file.is_open()) {
		return -1;
	}
	file.read((char *)&evLast, sizeof(EventHistory::Event));
	if (!file.good()) {
		file.close();
		return -2;
	}
	file.read((char *)&evSeen, sizeof(EventHistory::Event));
	if (!file.good()) {
		file.close();
		return -3;
	}
	file.close();

	readEvent.m_ts = evLast.ts;
	readEvent.m_id = evLast.id;
	readEvent.m_data1 = evLast.data[0];
	readEvent.m_data2 = evLast.data[1];
	readEvent.m_data3 = evLast.data[2];

	seenEvent.m_ts = evSeen.ts;
	seenEvent.m_id = evSeen.id;
	seenEvent.m_data1 = evSeen.data[0];
	seenEvent.m_data2 = evSeen.data[1];
	seenEvent.m_data3 = evSeen.data[2];

	return 0;
}

int EventHistory::SaveUserLastEvent(const EventNode &readEvent, const EventNode &seenEvent)
{
	EventHistory::Event evLast;
	EventHistory::Event evSeen;

	evLast.ts = readEvent.m_ts;
	evLast.id = readEvent.m_id;
	evLast.data[0] = readEvent.m_data1;
	evLast.data[1] = readEvent.m_data2;
	evLast.data[2] = readEvent.m_data3;

	evSeen.ts = seenEvent.m_ts;
	evSeen.id = seenEvent.m_id;
	evSeen.data[0] = seenEvent.m_data1;
	evSeen.data[1] = seenEvent.m_data2;
	evSeen.data[2] = seenEvent.m_data3;

	std::ofstream file((m_home + "/" + userEHStateFileName).c_str(), std::ios_base::out | std::ios_base::binary);
	if (!file.is_open()) {
		return -1;
	}
	file.write((const char *)&evLast, sizeof(EventHistory::Event));
	if (file.fail()) {
		file.close();
		unlink(userEHStateFileName.c_str());
		return -2;
	}
	file.write((const char *)&evSeen, sizeof(EventHistory::Event));
	if (file.fail()) {
		file.close();
		unlink(userEHStateFileName.c_str());
		return -3;
	}
	file.flush();
	if (file.fail()) {
		file.close();
		unlink(userEHStateFileName.c_str());
		return -4;
	}
	file.close();

	return 0;
}


}

