/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

//*****************************************************************************
//
// Class:			IPCiATStatus
//
// Description:	    IPCiATStatus is an object that holds the status of AMT/TPM,
//                  and this status can be shared across processes.
//                  When creating the object, you must specify whether you are
//                  the server or client of this status - being the server
//                  means you create the status and only write to it, and being
//                  a client means you connect to an existing status and
//                  only read from it.
//                  After initialization, you should use isOk() and lastError()
//                  to check the error status of the object.
//*****************************************************************************
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "iatshareddata.h"
#include "IPCiATStatus.h"

#include <string>
#include <cstring>

const char *IPCiATStatus::BUF_NAME = IATSHAREDDATADIR "/iATStatus";


//default ctor: must use init() afterwards
IPCiATStatus::IPCiATStatus()
{
	m_bWriteOk = true;
	m_bServer = false;
	m_bInitialized = false;
	m_shmReconnect = false;
	m_haveEnaData = false;
}

/*
ctor: initialize this object.
Arguments:
	bServer - specify whether this object should create the status, or just
	          try to open an existing one.*/
IPCiATStatus::IPCiATStatus(bool bServer)
{
	m_bWriteOk = true;
	m_bInitialized = false;
	m_shmReconnect = false;
	m_haveEnaData = false;

	init(bServer);
}

/*set option - reconnect for each shared memory access */
void IPCiATStatus::setShmReconnect(bool reconnect)
{
	if (!m_bServer) {
		m_shmReconnect = reconnect;
		if (!m_shmReconnect) {
			m_IPCBuffer.shmConnect();
		}
	}
}

#define IPC_AT_STATUS_SHM_RECONNECT_CONNECT if (m_shmReconnect) { \
		if (!m_IPCBuffer.shmConnect()) { \
			if (!shmInit()) { \
				return false; \
			} \
		} \
	}

#define IPC_AT_STATUS_SHM_RECONNECT_DISCONNECT if (m_shmReconnect) { \
		m_IPCBuffer.shmDisconnect(); \
	}

#define IPC_AT_STATUS_SHM_LOCK if (!m_IPCBuffer.shmLock()) { \
		return false; \
	}

#define IPC_AT_STATUS_SHM_UNLOCK m_IPCBuffer.shmUnLock();


//initialize as reader
bool IPCiATStatus::InitReader()
{
	init(false);
	//reconnect to know if UNS is still running
	setShmReconnect(true);

	return true;
}

//initialize data in IPC Buffer
bool IPCiATStatus::init(bool bServer)
{
	m_bServer = bServer;

	memset(&m_data, 0, sizeof(m_data));
	m_data.header.dwType = TYPE_ENABLEMENT_ONLY;
	m_data.header.dwBodyLength = sizeof(m_data.body.enablementOnly);

	//Initializing the IPC buffer
	if (!shmInit()) {
		return false;
	}

	m_bWriteOk = true;
	if (m_bServer) {
		//If we are the server, we need to prepare the status struct in the shared
		//memory.
		m_bWriteOk = _SafeResetEnablement(false);
		if (!m_bWriteOk) {
			return false;
		}

		if (m_shmReconnect) {
			//reconnect is for clients only
			m_shmReconnect = false;
		}
	}

	if (m_shmReconnect) {
		m_IPCBuffer.shmDisconnect();
	}

	m_bInitialized = true;

	return true;
}

//Initialize the IPC buffer
bool IPCiATStatus::shmInit()
{
	unsigned long nAccessType = IPCBuffer::IPC_BUF_ACCESS_READ;
	//Server can read/write, client can only read.
	if (m_bServer) {
		nAccessType |= IPCBuffer::IPC_BUF_ACCESS_WRITE;
	}

	//Initializing the IPC buffer
	if (!m_IPCBuffer.init(BUF_NAME, m_bServer, BUF_SIZE, true, nAccessType)) {
		return false;
	}

	return true;
}

//close connection to IPC buffer
void IPCiATStatus::close()
{
	m_bInitialized = false;
	if (m_bServer) {
		_SafeResetEnablement(true);
	}
	m_IPCBuffer.close();
}

/*return the error code of the last error*/
unsigned long IPCiATStatus::lastError() const
{
	return m_IPCBuffer.lastError();
}

/*returns true if this object is ok (no errors).*/
bool IPCiATStatus::isOk() const
{
	return m_IPCBuffer.isOk() && m_bWriteOk;
}

/*
Set the status of this object.
Arguments:
	enablement - the status to set
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::SetEnablement(IPCiATStatus::Enablement enablement)
{
	m_data.body.data.enablement = enablement;
	return _SetStatus();
}

/*
Prepare SKU info to be set
Arguments:
	sku - sku info
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::PutSku(unsigned int sku)
{
	m_data.body.data.sku = sku;
	return true;
}

/*
Set the AMT status data.
Arguments:
	statusData - AMT status data to set
	enablement - the status to set
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::SetAMTStatusData(const AMTStatusData &statusData, IPCiATStatus::Enablement enablement)
{
	PutAMTStatusData(statusData);
	m_data.header.dwType = TYPE_AMT_DATA;
	return SetEnablement(enablement);
}

/*
Prepare the AMT status data to be set.
Arguments:
	statusData - AMT status data to set
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::PutAMTStatusData(const AMTStatusData &statusData)
{
	m_data.header.dwType |= TYPE_AMT_DATA;
	m_data.header.dwBodyLength = sizeof(m_data.body.data);
	memcpy(&m_data.body.data.amtData, &statusData, sizeof(AMTStatusData));

	return true;
}

#ifdef WITH_TPM
/*
Set the TPM status data.
Arguments:
	statusData - TPM status data to set
	enablement - the status to set
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::SetTPMStatusData(const TPMStatusData &statusData, IPCiATStatus::Enablement enablement)
{
	PutTPMStatusData(statusData);
	m_data.header.dwType = TYPE_TPM_DATA;
	return SetEnablement(enablement);
}

/*
Prepare the TPM status data to be set.
Arguments:
	statusData - TPM status data to set
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::PutTPMStatusData(const TPMStatusData &statusData)
{
	m_data.header.dwType |= TYPE_TPM_DATA;
	m_data.header.dwBodyLength = sizeof(m_data.body.data);
	memcpy(&m_data.body.data.tpmData, &statusData, sizeof(TPMStatusData));

	return true;
}
#endif

/*
Set AT events
Arguments:
	events - list of events
	enablement - the status to set
Return value:
	true on success, false on failure.
*/
bool IPCiATStatus::SetEvents(const std::list<ATEvents::EventNode> &events, IPCiATStatus::Enablement enablement)
{
	PutEvents(events);
	m_data.header.dwType = TYPE_EVENTS;
	return SetEnablement(enablement);
}

/*
Prepare AT events to set
Arguments:
	events - list of events
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::PutEvents(const std::list<ATEvents::EventNode> &events)
{
	m_data.body.data.events.eventCount = 0;
	return AddEvents(events);
}

/*
Add AT events to preprared to set
Arguments:
	events - list of events
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::AddEvents(const std::list<ATEvents::EventNode> &events)
{
	bool ret = true;

	m_data.header.dwType |= TYPE_EVENTS;
	m_data.header.dwBodyLength = sizeof(m_data.body.data);
	{
		unsigned int firstEvent = m_data.body.data.events.eventCount;
		std::list<ATEvents::EventNode>::const_iterator iter = events.begin();
		unsigned int esize = firstEvent + (unsigned int)events.size();

		if (esize > MAX_IPC_EVENTS) {
			esize = MAX_IPC_EVENTS;
			ret = false; //truncate event list
		}
		for (unsigned int i = firstEvent; i < esize; iter++, i++) {
			m_data.body.data.events.event[i].ts = iter->m_ts;
			m_data.body.data.events.event[i].id = iter->m_id;
			m_data.body.data.events.event[i].data[0] = iter->m_data1;
			m_data.body.data.events.event[i].data[1] = iter->m_data2;
			m_data.body.data.events.event[i].data[2] = iter->m_data3;
		}
		m_data.body.data.events.eventCount = esize;
	}

	return ret;
}

/*
Check if status data has valid type.
Arguments:
	statusData - the status data to check
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::typeValid(Status &status)
{
	switch (status.header.dwType)
	{
	case TYPE_ENABLEMENT_ONLY:
	case TYPE_EVENTS:
	case TYPE_AMT_DATA:
	case TYPE_AMT_DATA_EVENTS:
#ifdef WITH_TPM
	case TYPE_TPM_DATA:
	case TYPE_TPM_DATA_EVENTS:
	case TYPE_DATA:
	case TYPE_DATA_EVENTS:
#endif
		return true;
		break;

	default: //unknown type
		return false;
	}
	return false;
}

/*
Set UNKNOWN STATUS
Arguments:
	true if need check prev status
Return value:
	true on success, false on failure.*/
bool IPCiATStatus::_SafeResetEnablement(bool nonInitial) {
	IPC_AT_STATUS_SHM_LOCK;
	bool result = _ResetEnablement(nonInitial);
	IPC_AT_STATUS_SHM_UNLOCK;
	return result;
}

bool IPCiATStatus::_ResetEnablement(bool nonInitial)
{
	Status status;

	m_data.header.dwType = TYPE_ENABLEMENT_ONLY;
	m_data.header.dwBodyLength = sizeof(m_data.body.enablementOnly);
	m_data.header.dwTS = time(NULL);
	m_data.body.enablementOnly.enablement = IPCiATStatus::STATUS_UNKNOWN;

	if (nonInitial) {
		if (!readData(status)) {
			return false;
		}

		if (!typeValid(status)) {
			return false;
		}
	}

	memcpy(&status, &m_data, sizeof(IPCiATStatus::Status));

	return writeData(status);
}

/*
Get SKU info
Return value:
	SKU info, or 0
*/
unsigned int IPCiATStatus::GetSku()
{
	if (!m_haveEnaData) {
		if (!_GetData()) {
			return 0;
		}
	}
	if (TYPE_ENABLEMENT_ONLY == m_data.header.dwType) {
		return 0;
	}

	return m_data.body.data.sku;
}

/*
Get AMT status data
Arguments:
	statusData - reference to AMT status data structure
Return value:
	true on success, false on failure.
*/
bool IPCiATStatus::GetAMTStatusData(AMTStatusData &statusData, bool useFWU)
{
	memset(&statusData, 0, sizeof(AMTStatusData));

	if (!m_haveEnaData) {
		if (!_GetData()) {
			return false;
		}
	}

	if (0 == (TYPE_AMT_DATA & m_data.header.dwType)) {
		return false;
	}

	memcpy(&statusData, &m_data.body.data.amtData, sizeof(AMTStatusData));

	return true;
}

#ifdef WITH_TPM
/*
Get TPM status data
Arguments:
	statusData - reference to TPM status data structure
Return value:
	true on success, false on failure.
*/
bool IPCiATStatus::GetTPMStatusData(TPMStatusData &statusData, unsigned int vendor)
{
	memset(&statusData, 0, sizeof(TPMStatusData));

	if (!m_haveEnaData) {
		if (!_GetData()) {
			return false;
		}
	}

	if (0 == (TYPE_TPM_DATA & m_data.header.dwType)) {
		return false;
	}

	memcpy(&statusData, &m_data.body.data.tpmData, sizeof(TPMStatusData));

	return true;
}
#endif

/*
Get AT events
Arguments:
	events - reference to list of events
Return value:
	true on success, false on failure.
*/
bool IPCiATStatus::GetEvents(std::list<ATEvents::EventNode> &events)
{
	events.clear();

	if (!m_haveEnaData) {
		if (!_GetData()) {
			return false;
		}
	}

	if (0 == (TYPE_EVENTS & m_data.header.dwType)) {
		return false;
	}

	for (unsigned int i = 0; ((i < m_data.body.data.events.eventCount) && (i < MAX_IPC_EVENTS)); i++) {
		events.push_back(ATEvents::EventNode(
			m_data.body.data.events.event[i].id,
			m_data.body.data.events.event[i].ts,
			m_data.body.data.events.event[i].data[0],
			m_data.body.data.events.event[i].data[1],
			m_data.body.data.events.event[i].data[2]));
	}
	return true;
}

/*
Get the status of this object.
Arguments:
	enablement - reference to Enablement data
Return value:
	true on success, false on failure.
*/
bool IPCiATStatus::GetEnablement(IPCiATStatus::Enablement &enablement)
{
	enablement = STATUS_UNKNOWN;

	m_haveEnaData = _GetData();
	if (!m_haveEnaData) {
		return false;
	}

	if (!typeValid(m_data)) {
		return false;
	}

	if (TYPE_ENABLEMENT_ONLY == m_data.header.dwType) {
		enablement = m_data.body.enablementOnly.enablement;
	} else {
		enablement = m_data.body.data.enablement;
	}

	return true;
}


//safe read data from IPC buffer
bool IPCiATStatus::_GetData()
{
	IPC_AT_STATUS_SHM_RECONNECT_CONNECT;
	IPC_AT_STATUS_SHM_LOCK;

	bool result = readData(m_data);

	IPC_AT_STATUS_SHM_UNLOCK;
	IPC_AT_STATUS_SHM_RECONNECT_DISCONNECT;

	if ((((unsigned long)time(NULL)) - m_data.header.dwTS) > 60) {
		//ignore old data
		return false;
	}

	return result;
}

/*
save write iAT status to IPC buffer
Return value:
	true on success, false on failure.
*/
bool IPCiATStatus::_SetStatus()
{
	IPC_AT_STATUS_SHM_RECONNECT_CONNECT;
	IPC_AT_STATUS_SHM_LOCK;

	bool result = __SetData();

	IPC_AT_STATUS_SHM_UNLOCK;
	IPC_AT_STATUS_SHM_RECONNECT_DISCONNECT;

	return result;
}

//write iAT status to IPC buffer
bool IPCiATStatus::__SetData()
{
	Status status;

	if (!readData(status)) {
		return false;
	}

	if (!typeValid(status)) {
		return false;
	}

	memcpy(&status, &m_data, sizeof(IPCiATStatus::Status));
	status.header.dwTS = time(NULL);

	if (!writeData(status)) {
		return false;
	}

	return true;
}


//write data to IPC buffer
bool IPCiATStatus::writeData(const Status &status)
{
	unsigned int nOffset = 0;

	//write header
	if (!m_IPCBuffer.write(nOffset, &status.header, sizeof(status.header))) {
		return false;
	}
	nOffset += sizeof(status.header);
	//write body
	if (!m_IPCBuffer.write(nOffset, &status.body, status.header.dwBodyLength)) {
		return false;
	}

	return true;
}

//read data from IPC buffer
bool IPCiATStatus::readData(Status &status) const
{
	unsigned int nOffset = 0;

	//read header
	if (!m_IPCBuffer.read(nOffset, &status.header, sizeof(status.header))) {
		return false;
	}
	nOffset += sizeof(status.header);
	//read body
	if (!m_IPCBuffer.read(nOffset, &status.body, status.header.dwBodyLength)) {
		return false;
	}

	return true;
}
