/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifndef _IPC_AT_STATUS_H
#define _IPC_AT_STATUS_H

#include "IPCBuffer.h"
#include "AMTStatusData.h"
#ifdef WITH_TPM
#include "TPMStatusData.h"
#endif
#include <list>
#include "EventNode.h"
#include "IMEAMTStatus.h"
#ifdef WITH_TPM
#include "IMETPMStatus.h"
#endif

//*****************************************************************************
//
// Class:			IPCiATStatus
// Description:	    IPCiATStatus is an object that holds the status of AMT/TPM,
//                  and this status can be shared across processes.
//                  When creating the object, you must specify whether you are
//                  the server or client of this status - being the server
//                  means you create the status and only write to it, and being
//                  a client means you connect to an existing status and
//                  only read from it.
//                  After initialization, you should use isOk() and lastError()
//                  to check the error status of the object.
//*****************************************************************************
class IPCiATStatus : public IMEAMTStatus
#ifdef WITH_TPM
    , public IMETPMStatus
#endif
{
public:
	//\En*a"ble*ment\, n. The act of enabling, or the state of being enabled; ability.
	enum Enablement {
			 STATUS_UNKNOWN = 0,
			 STATUS_DISABLED = 1,
			 STATUS_ENABLED = 2
#ifdef WITH_TPM
			 ,STATUS_AMT_DISABLED = 3 //TPM unknown
			 ,STATUS_AMT_NOT_DISABLED = 4 //TPM disabled
			 ,STATUS_AMT_ENABLED = 5 //TPM unknown
			 ,STATUS_TPM_DISABLED = 6 //AMT unknown
			 ,STATUS_TPM_NOT_DISABLED = 7  //AMT disabled
			 ,STATUS_TPM_ENABLED = 8 //AMT unknown
#endif
	 };

	static const unsigned int SKU_AMT = (1 << 0);
#ifdef WITH_TPM
	static const unsigned int SKU_TPM = (1 << 1);
#endif
	static const unsigned int SKU_AMTFUND = (1 << 2);

	//default ctor: must use init() afterwards
	IPCiATStatus();
	//ctor
	IPCiATStatus(bool bServer);
	//dtor
	virtual ~IPCiATStatus() { close(); };

	/* Reader API */
	//initialize as reader. returns false on failure.
	bool InitReader();
	//initialize as reader
	void CloseReader() { close(); };
	//get the status data. returns false on failure.
	bool GetAMTStatusData(AMTStatusData &statusData, bool useFWU = false);
#ifdef WITH_TPM
	bool GetTPMStatusData(TPMStatusData &statusData, unsigned int vendor = 0);
#endif

	//initialize this object after it was created
	bool init(bool bServer);
	//release resources
	void close();

	//get error code of last error
	unsigned long lastError() const;
	//is this IPCiATStatus object ok (no errors)
	bool isOk() const;
	//was this object properly written to shared memory, in case it's a server.
	//If it isn't, writeOk() returns true always.
	bool writeOk() const { return m_bWriteOk; }


	//enable/disable connecting to shm before each shared memory access
	void setShmReconnect(bool reconnect);

	/* Getters and setters - not in Reader API */
	//set the status data. returns false on failure.
	bool SetAMTStatusData(const AMTStatusData &statusData, Enablement enablement);
	//prepare status data to be set. returns false on failure.
	bool PutAMTStatusData(const AMTStatusData &statusData);
#ifdef WITH_TPM
	bool SetTPMStatusData(const TPMStatusData &statusData, Enablement enablement);
	bool PutTPMStatusData(const TPMStatusData &statusData);
#endif

	//get the enablement. returns false on failure.
	bool GetEnablement(Enablement &enablement);
	//set the enablement. returns false on failure.
	bool SetEnablement(Enablement enablement);

	//get the SKU. returns SKU
	unsigned int GetSku();
	//prepare SKU to be set. returns false on failure.
	bool PutSku(unsigned int sku);

	//get event list. returns false on failure.
	bool GetEvents(std::list<ATEvents::EventNode> &events);
	//set event list. returns false on failure.
	bool SetEvents(const std::list<ATEvents::EventNode> &events, Enablement enablement = STATUS_UNKNOWN);
	//prepare event list to be set. returns false on failure.
	bool PutEvents(const std::list<ATEvents::EventNode> &events);
	//append event list to events prepared to be set. returns false on failure.
	bool AddEvents(const std::list<ATEvents::EventNode> &events);

private:
	static const char *BUF_NAME;
	static const unsigned int BUF_SIZE              = 2048;
	static const unsigned int MAX_IPC_EVENTS        = 20;

	static const unsigned int TYPE_ENABLEMENT_ONLY = 0;
	static const unsigned int TYPE_EVENTS          = (1 << 0);
	static const unsigned int TYPE_AMT_DATA        = (1 << 1);
	static const unsigned int TYPE_AMT_DATA_EVENTS = ((1 << 1) | (1 << 0));
#ifdef WITH_TPM
	static const unsigned int TYPE_TPM_DATA        = (1 << 2);
	static const unsigned int TYPE_TPM_DATA_EVENTS = ((1 << 2) | (1 << 0));
	static const unsigned int TYPE_DATA            = ((1 << 2) | (1 << 1));
	static const unsigned int TYPE_DATA_EVENTS     = ((1 << 2) | (1 << 1) | (1 << 0));
#endif

	typedef struct _IPCEvent {
		time_t ts;
		unsigned char id;
		unsigned char data[3];
	} IPCEvent;

	typedef struct _Status {
		struct Header {
			unsigned int dwType;
			unsigned int dwBodyLength;
			__u_quad_t dwTS;
		} header;

		union StatusBody {
			struct _EnablementOnly {
				Enablement enablement;
			} enablementOnly;

			struct _EnablementAndData {
				Enablement enablement;
				unsigned int sku;
				AMTStatusData amtData;
				struct _events {
					unsigned int  eventCount;
					IPCEvent      event[MAX_IPC_EVENTS];
				} events;
#ifdef WITH_TPM
				TPMStatusData tpmData;
#endif
			} data;
		} body;
	} Status;

	bool typeValid(Status &status);

	/* internal data setters */
	//reset the enablement. returns false on failure.
	bool _SafeResetEnablement(bool nonInitial);
	bool _ResetEnablement(bool nonInitial);

	/* internal data getters and setters */
	//get data. returns false on failure.
	bool _GetData();
	//set data. returns false on failure.
	bool _SetStatus();
	bool __SetData();


	/* internal io subroutines */
	bool writeData(const Status &status);
	bool readData(Status &status) const;


	bool shmInit();


	IPCBuffer m_IPCBuffer;
	bool m_bWriteOk;
	bool m_bServer;
	bool m_bInitialized;
	bool m_shmReconnect;
	Status m_data;
	bool m_haveEnaData;
};

#endif //_IPC_AT_STATUS_H
