/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 * 
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 * 
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#ifdef WITH_TPM
#include "TPMStatus.h"

#include <sstream>
#include <string>
#include <iomanip>
#include <cerrno>
#include <fstream>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <cstring>

// Constructor
TPMStatus::TPMStatus()
{
	memset(&m_TPMStatusData, 0, sizeof(m_TPMStatusData));
	m_TPMStatusData.content = 0;
}

const unsigned char TPMStatus::req_VER[] = {0,0xC1, 0,0,0,18, 0,0,0,0x65, 0,0,0,0x1A, 0,0,0,0};
const unsigned char TPMStatus::req_ENA[] = {0,0xC1, 0,0,0,22, 0,0,0,0x65, 0,0,0,4, 0,0,0,4, 0,0,1,8};//16/16 18/-
const unsigned char TPMStatus::req_ACT[] = {0,0xC1, 0,0,0,22, 0,0,0,0x65, 0,0,0,4, 0,0,0,4, 0,0,1,9};//-/16
const unsigned char TPMStatus::req_OWN[] = {0,0xC1, 0,0,0,22, 0,0,0,0x65, 0,0,0,5, 0,0,0,4, 0,0,1,0x11};//14
const unsigned char TPMStatus::req_OPE[] = {0,0xC1, 0,0,0,10, 0,0,0,0x54};//14-17
#define ITPM_TAG_VER  0x30
#define ITPM_TAG_ENA  0x1F
#define ITPM_TAG_ENA2 0x07
#define ITPM_TAG_ACT  0x20
#define ITPM_DMIN_RESP_VER  15
#define ITPM_DMIN_RESP_ENA  21
#define ITPM_DMIN_RESP_ACT  7
#define ITPM_DMIN_RESP_OWN  1
#define ITPM_DMIN_RESP_OPE  2

int TPMStatus::transmit(int fd, unsigned char *blob, unsigned int minDlen)
{
	size_t wsize = (size_t)ntohl(*(uint32_t *)&blob[2]);

	ssize_t wlen = write(fd, blob, wsize);
	if ((size_t)wlen != wsize) {
		return -1;
	}
	ssize_t len = read(fd, blob, 4096);
	if ((len < 10) || (len != (ssize_t)ntohl(*(uint32_t *)&blob[2]))) {
		return -2;
	}
	if ((blob[0] != 0) || (blob[1] != 0xC4)) {
		return -3;
	}
	uint32_t rc = ntohl(*(uint32_t *)&blob[6]);
	if (rc != 0) {
		return (int)rc;
	}
	if (10 == len) {
		return -4;
	}
	uint32_t resp_dlen = ntohl(*(uint32_t *)&blob[10]);
	if (ntohl(*(uint32_t *)&blob[2]) != (resp_dlen + 14)) {
		return -5;
	}
	if (resp_dlen < minDlen) {
		return -6;
	}
	return 0;
}

int TPMStatus::getVersion(int fd)
{
	unsigned char blob[4096];
	int ret;
	
	memcpy(blob, req_VER, sizeof(req_VER));
	ret = transmit(fd, blob, ITPM_DMIN_RESP_VER);
	if (0 != ret) {
		return ret;
	}
	if (ITPM_TAG_VER != ntohs(*(uint16_t *)&blob[14])) {
		return -7;
	}

	//ver-16,17 rev-18,19 level-20+21 errata-22
	m_TPMStatusData.TCGVersion.major     = blob[16];
	m_TPMStatusData.TCGVersion.minor     = blob[17];
	m_TPMStatusData.TCGVersion.revMajor  = blob[18];
	m_TPMStatusData.TCGVersion.revMinor  = blob[19];
	m_TPMStatusData.TCGVersion.specLevel = ntohs(*(uint16_t *)&blob[20]);
	m_TPMStatusData.TCGVersion.errata    = blob[22];
	m_TPMStatusData.vendor               = ntohl(*(uint32_t *)&blob[23]);
	m_TPMStatusData.content |= CONTENT_TPM_TCG_VERSION;

	if (8 == ntohs(*(uint16_t *)&blob[27])) {
		//29,30 . 31,32 . 33,34 . 35,36
		m_TPMStatusData.TPMVersion.major  = ntohs(*(uint16_t *)&blob[29]);
		m_TPMStatusData.TPMVersion.minor  = ntohs(*(uint16_t *)&blob[31]);
		m_TPMStatusData.TPMVersion.hotfix = ntohs(*(uint16_t *)&blob[33]);
		m_TPMStatusData.TPMVersion.build  = ntohs(*(uint16_t *)&blob[35]);
		m_TPMStatusData.content |= CONTENT_TPM_TPM_VERSION;
	}

	return 0;
}

int TPMStatus::getOperationState(int fd)
{
	unsigned char blob[4096];
	int ret;
	
	memcpy(blob, req_OPE, sizeof(req_OPE));
	ret = transmit(fd, blob, ITPM_DMIN_RESP_OPE);
	if (0 != ret) {
		return ret;
	}

	for (uint32_t ii = ntohl(*(uint32_t *)&blob[10]); ii < 4; ii++) {
		blob[14 + ii] = 0;
	}
	memcpy(m_TPMStatusData.operationState, &blob[14], 4);
	m_TPMStatusData.content |= CONTENT_TPM_OPERATION_STATE;

	return 0;
}

int TPMStatus::getOwnershipState(int fd)
{
	unsigned char blob[4096];
	int ret;
	
	memcpy(blob, req_OWN, sizeof(req_OWN));
	ret = transmit(fd, blob, ITPM_DMIN_RESP_OWN);
	if (0 != ret) {
		return ret;
	}

	m_TPMStatusData.ownershipState = blob[14];
	m_TPMStatusData.content |= CONTENT_TPM_OWNERSHIP_STATE;

	return 0;
}

int TPMStatus::getEnablementState(int fd)
{
	unsigned char blob[4096];
	int ret;
	
	memcpy(blob, req_ENA, sizeof(req_ENA));
	ret = transmit(fd, blob, ITPM_DMIN_RESP_ENA);
	if (0 != ret) {
		return ret;
	}
	unsigned short ftag = ntohs(*(uint16_t *)&blob[14]);
	if ((ITPM_TAG_ENA2 != ftag) && (ITPM_TAG_ENA != ftag)) {
		return -7;
	}

	m_TPMStatusData.enablementState = !blob[16];
	m_TPMStatusData.content |= CONTENT_TPM_ENABLEMENT_STATE;
	m_TPMStatusData.activationState = !blob[18];
	m_TPMStatusData.content |= CONTENT_TPM_ACTIVATION_STATE;

	return 0;
}

int TPMStatus::getActivationState(int fd)
{
	unsigned char blob[4096];
	int ret;
	
	memcpy(blob, req_ACT, sizeof(req_ACT));
	ret = transmit(fd, blob, ITPM_DMIN_RESP_ACT);
	if (0 != ret) {
		return ret;
	}
	if (ITPM_TAG_ACT != ntohs(*(uint16_t *)&blob[14])) {
		return -7;
	}

	m_TPMStatusData.tempActivationState = !blob[16];
	m_TPMStatusData.content |= CONTENT_TPM_TEMP_ACTIV_STATE;

	return 0;
}

#ifdef HAVE_TROUSERS_TSS_H
TSS_RESULT TPMStatus::getTspiVersion(TSS_HCONTEXT &ctx, TSS_HTPM &tpmo)
{
	TSS_RESULT res;
	UINT32 len = 0;
	BYTE *blob;

	res = Tspi_TPM_GetCapability(tpmo,
			TSS_TPMCAP_VERSION_VAL, 0, NULL,
			&len, &blob);
	if (TSS_SUCCESS != res) {
		return res;
	}
	if (len < ITPM_DMIN_RESP_VER) {
		res = -6;
		goto verout;
	}
	if (ITPM_TAG_VER != ntohs(*(uint16_t *)&blob[0])) {
		res = -7;
		goto verout;
	}

	m_TPMStatusData.TCGVersion.major     = blob[2];
	m_TPMStatusData.TCGVersion.minor     = blob[3];
	m_TPMStatusData.TCGVersion.revMajor  = blob[4];
	m_TPMStatusData.TCGVersion.revMinor  = blob[5];
	m_TPMStatusData.TCGVersion.specLevel = ntohs(*(uint16_t *)&blob[6]);
	m_TPMStatusData.TCGVersion.errata    = blob[8];
	m_TPMStatusData.vendor               = ntohl(*(uint32_t *)&blob[9]);
	m_TPMStatusData.content |= CONTENT_TPM_TCG_VERSION;

	if (8 == ntohs(*(uint16_t *)&blob[13])) {
		m_TPMStatusData.TPMVersion.major  = ntohs(*(uint16_t *)&blob[15]);
		m_TPMStatusData.TPMVersion.minor  = ntohs(*(uint16_t *)&blob[17]);
		m_TPMStatusData.TPMVersion.hotfix = ntohs(*(uint16_t *)&blob[19]);
		m_TPMStatusData.TPMVersion.build  = ntohs(*(uint16_t *)&blob[21]);
		m_TPMStatusData.content |= CONTENT_TPM_TPM_VERSION;
	}

verout:
	Tspi_Context_FreeMemory(ctx, blob);
	return res;
}

TSS_RESULT TPMStatus::getTspiOperationState(TSS_HCONTEXT &ctx, TSS_HTPM &tpmo)
{
	TSS_RESULT res;
	UINT32 len = 0;
	BYTE *blob;

	res = Tspi_TPM_GetTestResult(tpmo, &len, &blob);
	if (TSS_SUCCESS != res) {
		return res;
	}

	for (uint32_t ii = 0; ii < 4; ii++) {
		m_TPMStatusData.operationState[ii] = ((ii < len) ? blob[ii] : 0);
	}
	m_TPMStatusData.content |= CONTENT_TPM_OPERATION_STATE;

	Tspi_Context_FreeMemory(ctx, blob);
	return res;
}

TSS_RESULT TPMStatus::getTspiOwnershipState(TSS_HCONTEXT &ctx, TSS_HTPM &tpmo)
{
	TSS_RESULT res;
	UINT32 len = 0;
	BYTE *blob;
	UINT32 subcap = TSS_TPMCAP_PROP_OWNER;

	res = Tspi_TPM_GetCapability(tpmo,
			TSS_TPMCAP_PROPERTY, sizeof(UINT32), (BYTE *)&subcap,
			&len, &blob);
	if (TSS_SUCCESS != res) {
		return res;
	}
	if (len < ITPM_DMIN_RESP_OWN) {
		res = -6;
		goto ownout;
	}

	m_TPMStatusData.ownershipState = blob[0];
	m_TPMStatusData.content |= CONTENT_TPM_OWNERSHIP_STATE;

ownout:
	Tspi_Context_FreeMemory(ctx, blob);
	return res;
}

TSS_RESULT TPMStatus::getTspiEnablementState(TSS_HCONTEXT &ctx, TSS_HTPM &tpmo)
{
	TSS_RESULT res;
	TSS_BOOL val;

	res = Tspi_TPM_GetStatus(tpmo, TSS_TPMSTATUS_DISABLED, &val);
	if (TSS_SUCCESS != res) {
		return res;
	}

	m_TPMStatusData.enablementState = !val;
	m_TPMStatusData.content |= CONTENT_TPM_ENABLEMENT_STATE;
	return res;
}

TSS_RESULT TPMStatus::getTspiActivationState(TSS_HCONTEXT &ctx, TSS_HTPM &tpmo)
{
	TSS_RESULT res;
	TSS_BOOL val;

	res = Tspi_TPM_GetStatus(tpmo, TSS_TPMSTATUS_DEACTIVATED, &val);
	if (TSS_SUCCESS != res) {
		return res;
	}

	m_TPMStatusData.activationState = !val;
	m_TPMStatusData.content |= CONTENT_TPM_ACTIVATION_STATE;
	return res;
}

TSS_RESULT TPMStatus::getTspiTempActivationState(TSS_HCONTEXT &ctx, TSS_HTPM &tpmo)
{
	TSS_RESULT res;
	TSS_BOOL val;

	res = Tspi_TPM_GetStatus(tpmo, TSS_TPMSTATUS_SETTEMPDEACTIVATED, &val);
	if (TSS_SUCCESS != res) {
		return res;
	}

	m_TPMStatusData.tempActivationState = !val;
	m_TPMStatusData.content |= CONTENT_TPM_TEMP_ACTIV_STATE;
	return res;
}

#endif


const char *TPMStatus::tpmNodes[] = {
	"/dev/tpm0",
	"/udev/tpm0",
	"/dev/tpm1",
	"/udev/tpm1",
	"/dev/tpm2",
	"/udev/tpm2",
	NULL,
	NULL
};

int TPMStatus::openTPM(int &error, int &idx)
{
	error = ENOENT;

	for (int i = ++idx; TPMStatus::tpmNodes[i] != NULL; i++) {
		int fd;
		if ((fd = open(TPMStatus::tpmNodes[i], O_RDWR)) == -1) {
			if (ENOENT != errno) {
				error = errno;
			} else {
				continue;
			}
		}
		idx = i;
		return fd;
	}
	return -1;
}

std::string TPMStatus::getSysFsTpmDev(int idx)
{
	std::stringstream ss;
	ss << "/sys/class/misc/tpm" << (int)(idx/2) << "/device/";
	return ss.str();
}

int TPMStatus::getSysCaps(std::string devDir)
{
	char line[1024];
	unsigned int vendor = 0;
	int vmaj, vmin;

	std::ifstream file((devDir + "caps").c_str(), std::ios_base::in);
	if (!file.is_open()) {
		return -2;
	}

	file.getline(line, 1024);
	if (file.bad() ||
	    (1 != sscanf(line, "Manufacturer: %x", &vendor))) {
		file.close();
		return -3;
	}
	m_TPMStatusData.vendor = vendor;

	file.getline(line, 1024);
	if (file.bad() ||
	    (2 != sscanf(line, "TCG version: %d.%d", &vmaj, &vmin))) {
		file.close();
		return -4;
	}
	m_TPMStatusData.TCGVersion.major = (unsigned char)vmaj;
	m_TPMStatusData.TCGVersion.minor = (unsigned char)vmin;

	file.getline(line, 1024);
	if (file.bad() ||
	    (2 != sscanf(line, "Firmware version: %d.%d", &vmaj, &vmin))) {
		file.close();
		return -5;
	}
	m_TPMStatusData.TCGVersion.revMajor = (unsigned char)vmaj;
	m_TPMStatusData.TCGVersion.revMinor = (unsigned char)vmin;

	file.close();

	m_TPMStatusData.content |= CONTENT_TPM_TCG_VERSION;

	return 0;
}

int TPMStatus::getSysActive(std::string devDir)
{
	int value = 0;

	std::ifstream file((devDir + "active").c_str(), std::ios_base::in);
	if (!file.is_open()) {
		return -2;
	}
	file >> value;
	if (file.bad()) {
		file.close();
		return -3;
	}
	file.close();

	m_TPMStatusData.activationState = !(!(unsigned char)value);
	m_TPMStatusData.content |= CONTENT_TPM_ACTIVATION_STATE;

	return 0;
}

int TPMStatus::getSysEnabled(std::string devDir)
{
	int value = 0;

	std::ifstream file((devDir + "enabled").c_str(), std::ios_base::in);
	if (!file.is_open()) {
		return -2;
	}
	file >> value;
	if (file.bad()) {
		file.close();
		return -3;
	}
	file.close();

	m_TPMStatusData.enablementState = !(!(unsigned char)value);
	m_TPMStatusData.content |= CONTENT_TPM_ENABLEMENT_STATE;

	return 0;
}

int TPMStatus::getSysOwned(std::string devDir)
{
	int value = 0;

	std::ifstream file((devDir + "owned").c_str(), std::ios_base::in);
	if (!file.is_open()) {
		return -2;
	}
	file >> value;
	if (file.bad()) {
		file.close();
		return -3;
	}
	file.close();

	m_TPMStatusData.ownershipState = !(!(unsigned char)value);
	m_TPMStatusData.content |= CONTENT_TPM_OWNERSHIP_STATE;

	return 0;
}

int TPMStatus::getSysTempDeactivated(std::string devDir)
{
	int value = 0;

	std::ifstream file((devDir + "temp_deactivated").c_str(), std::ios_base::in);
	if (!file.is_open()) {
		return -2;
	}
	file >> value;
	if (file.bad()) {
		file.close();
		return -3;
	}
	file.close();

	m_TPMStatusData.tempActivationState = !(unsigned char)value;
	m_TPMStatusData.content |= CONTENT_TPM_TEMP_ACTIV_STATE;

	return 0;
}

bool TPMStatus::GetTPMStatusData(TPMStatusData &statusData, unsigned int vendor)
{
	std::string devDir = getSysFsTpmDev(0);

	memset(&m_TPMStatusData, 0, sizeof(m_TPMStatusData));
	m_TPMStatusData.content = 0;

#ifdef HAVE_TROUSERS_TSS_H
	TSS_HCONTEXT ctx;

	if (TSS_SUCCESS == Tspi_Context_Create(&ctx)) {
		TSS_HTPM tpmo;
#if 0
		TSS_HPOLICY policy;
		BYTE wellK[] = TSS_WELL_KNOWN_SECRET;
#endif

		if (TSS_SUCCESS != Tspi_Context_Connect(ctx, NULL)) {
			goto tpmtssfreemem;
		}
		if (TSS_SUCCESS != Tspi_Context_GetTpmObject(ctx, &tpmo)) {
			goto tpmtssclose;
		}

		getTspiVersion(ctx, tpmo);

		if ((vendor != 0) && (vendor != m_TPMStatusData.vendor)) {
			goto tpmtssclose;
		}

		getTspiOperationState(ctx, tpmo);
		getTspiOwnershipState(ctx, tpmo);

#if 0
		/* NOT IMPLEMENTED YET IN TSPI */
		if (TSS_SUCCESS != Tspi_GetPolicyObject(tpmo, TSS_POLICY_USAGE, &policy)) {
			goto tpmtssclose;
		}
		if (TSS_SUCCESS != Tspi_Policy_SetSecret(policy, TSS_SECRET_MODE_SHA1, sizeof(wellK), wellK)) {
			goto tpmtssclose;
		}

		getTspiEnablementState(ctx, tpmo);
		getTspiActivationState(ctx, tpmo);
		if (((m_TPMStatusData.content & CONTENT_TPM_ACTIVATION_STATE) != 0) &&
		    (m_TPMStatusData.activationState != 0)) {
			getTspiTempActivationState(ctx, tpmo);
		}
#endif

tpmtssclose:
		Tspi_Context_Close(ctx);

tpmtssfreemem:
		Tspi_Context_FreeMemory(ctx, NULL);
	}
#else
#if 0
	int error = 0;
	int idx = -1;

	do {
		memset(&m_TPMStatusData, 0, sizeof(m_TPMStatusData));
		m_TPMStatusData.content = 0;

		int fd = openTPM(error, idx);
		if (-1 == fd) {
			if (EBUSY != error) {
				return false;
			}
			m_TPMStatusData.content |= CONTENT_TPM_BUSY;
			devDir = getSysFsTpmDev(idx);
		} else {
			getVersion(fd);
			getOperationState(fd);
			getOwnershipState(fd);
			getEnablementState(fd);
			if (((m_TPMStatusData.content & CONTENT_TPM_ACTIVATION_STATE) != 0) &&
			    (m_TPMStatusData.activationState != 0)) {
				getActivationState(fd);
			}
			close(fd);
		}
	} while ((vendor != 0) && (vendor != m_TPMStatusData.vendor));
#endif
#endif

	if ((m_TPMStatusData.content & CONTENT_TPM_OWNERSHIP_STATE) == 0) {
		if (0 != getSysOwned(devDir)) {
			return false;
		}
	}

	if ((m_TPMStatusData.content & CONTENT_TPM_TCG_VERSION) == 0) {
		if (0 != getSysCaps(devDir)) {
			return false;
		}
	}

	if ((vendor != 0) && (vendor != m_TPMStatusData.vendor)) {
		return false;
	}

	if ((m_TPMStatusData.content & CONTENT_TPM_ENABLEMENT_STATE) == 0) {
		getSysActive(devDir);
		getSysEnabled(devDir);
		if (((m_TPMStatusData.content & CONTENT_TPM_ACTIVATION_STATE) != 0) &&
		    (m_TPMStatusData.activationState != 0)) {
			getSysTempDeactivated(devDir);
		}
	}

	memcpy(&statusData, &m_TPMStatusData, sizeof(TPMStatusData));

	return (((m_TPMStatusData.content & CONTENT_TPM_TCG_VERSION) != 0) ? true : false);
}

#endif

