/*******************************************************************************
 * Copyright (C) 2004-2008 Intel Corp. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   - Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright notice,
 *     this list of conditions and the following disclaimer in the documentation
 *     and/or other materials provided with the distribution.
 *
 *   - Neither the name of Intel Corp. nor the names of its
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *******************************************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include "iatshareddata.h"
#ifdef WITH_TPM
#include "TPMStatusColl.h"
#include <memory>
#include <cstring>
#include <fstream>
#include <unistd.h>

const std::string TPMStatusColl::defaultLastTPMCollStateFileName = IATSHAREDDATADIR "/iTPMCollState.dat";

TPMStatusColl::TPMStatusColl()
{
	memset(&m_baseTPMStatusData, 0, sizeof(TPMStatusData));
	m_bInitialized = false;
	m_bBaseDataSaved = false;
	m_bAutosave = false;
	m_lastTPMCollStateFileName = defaultLastTPMCollStateFileName;
}

int TPMStatusColl::Init(bool autosave, std::string fname)
{
	m_bAutosave = autosave;
	m_lastTPMCollStateFileName = fname;
	m_bInitialized = true;
	return loadBaseData();
}

void TPMStatusColl::GetBaseData(TPMStatusData &baseTPMStatusData)
{
	memcpy(&baseTPMStatusData, &m_baseTPMStatusData, sizeof(TPMStatusData));
}

void TPMStatusColl::SetBaseData(const TPMStatusData &baseTPMStatusData)
{
	memcpy(&m_baseTPMStatusData, &baseTPMStatusData, sizeof(TPMStatusData));
	m_bInitialized = true;
}

/*
 * Returns TPM operational state
 * -1 - unknown
 *  0 - operational
 * >0 - no operational
 */
int TPMStatusColl::TPMOperationalState(const TPMStatusData &someTPMStatusData)
{
	if (0 == (someTPMStatusData.content & CONTENT_TPM_OPERATION_STATE)) {
		return TPM_COLL_OPER_UNKNOWN;
	}

	if ((0 == someTPMStatusData.operationState[0])
	    && (0 == someTPMStatusData.operationState[1])
	    )
	{
		if ((0 == someTPMStatusData.operationState[2])
		    && (0 == someTPMStatusData.operationState[3])
		    )
		{
			return TPM_COLL_OPER_OPERATIONAL;
		}

		if (5 == someTPMStatusData.operationState[2]) {
			switch ((unsigned int)(someTPMStatusData.operationState[3]))
			{
			case 7:
				return TPM_COLL_OPER_ME_RESET;
				break;

			case 4:
			case 5:
			case 6:
				return TPM_COLL_OPER_HW_FAILURE;
				break;

			default:
				return TPM_COLL_OPER_ERROR;
			}
		}
	}

	return TPM_COLL_OPER_NO_OPERATIONAL;
}

/*
 * collate new status data with base one, set events
 * return: <0 - on error, 0 - if no difference, >0 - 1 + number of difference based events
 **/
int TPMStatusColl::Collate(const TPMStatusData &newTPMStatusData, std::list<ATEvents::EventNode> *events)
{
	int diffs = 0;
	bool operational = false;

	if (!m_bInitialized) {
		return -2;
	}

	//ignore TS
	m_baseTPMStatusData.dataTS = newTPMStatusData.dataTS;

	if (0 == memcmp(&m_baseTPMStatusData, &newTPMStatusData, sizeof(TPMStatusData))) {
		return 0;
	}

	diffs = 1;

	if (NULL == events) {
		updateBaseData(newTPMStatusData);
		return 1;
	}

	int operState = TPMOperationalState(newTPMStatusData);

	operational = (TPM_COLL_OPER_OPERATIONAL == operState);

	//here events born
	if (TPM_COLL_OPER_UNKNOWN == operState) {
		//not detected
		if (0 != (m_baseTPMStatusData.content & CONTENT_TPM_OPERATION_STATE)) {
			//previously was operational or error
			events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_NOT_DETECTED));
			diffs++;
		}
	} else {
		if ((0 == (m_baseTPMStatusData.content & CONTENT_TPM_OPERATION_STATE))
		    || (0 != memcmp(m_baseTPMStatusData.operationState, newTPMStatusData.operationState, 4))
		    )
		{
			if (operational) {
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_OPERATIONAL));
			} else {
				unsigned char failReason;

				switch (operState)
				{
				case TPM_COLL_OPER_ME_RESET:
					failReason = ATEvents::EventNode::TPM_FAIL_ME_RESET;
					break;

				case TPM_COLL_OPER_HW_FAILURE:
					failReason = ATEvents::EventNode::TPM_FAIL_HW_FAILURE;
					break;

				default:
					failReason = ATEvents::EventNode::TPM_FAIL_UNKNOWN;
				}
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_FAIL, 0, failReason));
			}
			diffs++;
		}
	}


	if ((operational)
	    && (0 != (newTPMStatusData.content & CONTENT_TPM_OPERATION_SUBSTATES))
	    )
	{
		if ((0 != (newTPMStatusData.content & CONTENT_TPM_ACTIVATION_STATE))
		    && (0 != (m_baseTPMStatusData.content & CONTENT_TPM_ACTIVATION_STATE))
		    && (newTPMStatusData.activationState != m_baseTPMStatusData.activationState)
		    )
		{
			if (0 != newTPMStatusData.activationState) {
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_ACTIVE));
			} else {
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_NOT_ACTIVE));
			}
			diffs++;
		}

		if ((0 != (newTPMStatusData.content & CONTENT_TPM_ENABLEMENT_STATE))
		    && (0 != (m_baseTPMStatusData.content & CONTENT_TPM_ENABLEMENT_STATE))
		    && (newTPMStatusData.enablementState != m_baseTPMStatusData.enablementState)
		    )
		{
			if (0 != newTPMStatusData.enablementState) {
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_ENABLE));
			} else {
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_NOT_ENABLE));
			}
			diffs++;
		}

		if ((0 != (newTPMStatusData.content & CONTENT_TPM_OWNERSHIP_STATE))
		    && (0 != (m_baseTPMStatusData.content & CONTENT_TPM_OWNERSHIP_STATE))
		    && (newTPMStatusData.ownershipState != m_baseTPMStatusData.ownershipState)
		    )
		{
			if (0 != newTPMStatusData.activationState) {
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_OWNED));
			} else {
				events->push_back(ATEvents::EventNode(ATEvents::EventNode::TPM_NOT_OWNED));
			}
			diffs++;
		}
	}

	updateBaseData(newTPMStatusData);

	return diffs;
}

int TPMStatusColl::updateBaseData(const TPMStatusData &newTPMStatusData)
{
	m_baseTPMStatusData.dataTS = newTPMStatusData.dataTS;

	if (0 != (newTPMStatusData.content & CONTENT_TPM_TCG_VERSION)) {
		memcpy(&m_baseTPMStatusData.TCGVersion, &newTPMStatusData.TCGVersion, sizeof(m_baseTPMStatusData.TCGVersion));
		m_baseTPMStatusData.content |= CONTENT_TPM_TCG_VERSION;
	}
	if (0 != (newTPMStatusData.content & CONTENT_TPM_TPM_VERSION)) {
		memcpy(&m_baseTPMStatusData.TPMVersion, &newTPMStatusData.TPMVersion, sizeof(m_baseTPMStatusData.TPMVersion));
		m_baseTPMStatusData.content |= CONTENT_TPM_TPM_VERSION;
	}
	if (0 != (newTPMStatusData.content & CONTENT_TPM_OPERATION_STATE)) {
		memcpy(m_baseTPMStatusData.operationState, newTPMStatusData.operationState, sizeof(m_baseTPMStatusData.operationState));
		m_baseTPMStatusData.content |= CONTENT_TPM_OPERATION_STATE;
	}
	if (0 != (newTPMStatusData.content & CONTENT_TPM_ACTIVATION_STATE)) {
		m_baseTPMStatusData.activationState = newTPMStatusData.activationState;
		m_baseTPMStatusData.content |= CONTENT_TPM_ACTIVATION_STATE;
	}
	if (0 != (newTPMStatusData.content & CONTENT_TPM_ENABLEMENT_STATE)) {
		m_baseTPMStatusData.enablementState = newTPMStatusData.enablementState;
		m_baseTPMStatusData.content |= CONTENT_TPM_ENABLEMENT_STATE;
	}
	if (0 != (newTPMStatusData.content & CONTENT_TPM_OWNERSHIP_STATE)) {
		m_baseTPMStatusData.ownershipState = newTPMStatusData.ownershipState;
		m_baseTPMStatusData.content |= CONTENT_TPM_OWNERSHIP_STATE;
	}

	if (m_bAutosave) {
		m_bBaseDataSaved = ((0 == saveBaseData()) ? true : false);
	}

	return 0;
}

int TPMStatusColl::loadBaseData()
{
	TPMStatusData tmpData;

	std::ifstream file(m_lastTPMCollStateFileName.c_str(), std::ios_base::in | std::ios_base::binary);
	if (!file.is_open()) {
		return -1;
	}
	file.read((char *)&tmpData, sizeof(TPMStatusData));
	if (!file.good()) {
		file.close();
		return -2;
	}
	file.close();

	memcpy(&m_baseTPMStatusData, &tmpData, sizeof(TPMStatusData));

	return 0;
}

int TPMStatusColl::saveBaseData()
{
	std::ofstream file(m_lastTPMCollStateFileName.c_str(), std::ios_base::out | std::ios_base::binary | std::ios_base::trunc);
	if (!file.is_open()) {
		unlink(m_lastTPMCollStateFileName.c_str());
		return -1;
	}
	file.write((const char *)&m_baseTPMStatusData, sizeof(TPMStatusData));
	if (file.fail()) {
		file.close();
		unlink(m_lastTPMCollStateFileName.c_str());
		return -2;
	}
	file.flush();
	if (file.fail()) {
		file.close();
		unlink(m_lastTPMCollStateFileName.c_str());
		return -4;
	}
	file.close();
	return 0;
}

#endif
